import { NotificationMessage, SubscribeMessage, UnsubscribeMessage } from './messages';
import { Observable } from 'rxjs';
declare type IncomingMessage = SubscribeMessage | UnsubscribeMessage | NotificationMessage;
export declare const enum BinaryRxServerError {
    Unknown = 0,
    IdTaken = 1,
    TooManySubscriptions = 2,
    InvalidData = 3
}
export interface BinaryRxServerParams<Ctx = unknown> {
    send: (message: Uint8Array) => void;
    call: (name: string, data: Uint8Array | undefined, ctx: Ctx) => Promise<Uint8Array> | Observable<Uint8Array> | Promise<Observable<Uint8Array>>;
    notify: (name: string, data: Uint8Array | undefined, ctx: Ctx) => void;
    maxActiveSubscriptions?: number;
    bufferSize?: number;
    bufferTime?: number;
}
export declare class BinaryRxServer<Ctx = unknown> {
    private send;
    private call;
    private notify;
    private readonly active;
    private readonly maxActiveSubscriptions;
    private readonly encoder;
    constructor({ send, call, notify, maxActiveSubscriptions, bufferSize, bufferTime, }: BinaryRxServerParams<Ctx>);
    private sendError;
    private onSubscribe;
    private onUnsubscribe;
    private onNotification;
    onArray(arr: Uint8Array, ctx: Ctx): void;
    onMessages(messages: IncomingMessage[], ctx: Ctx): void;
    stop(): void;
}
export {};
