"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const header_1 = require("./header");
const DataMessage_1 = require("../messages/DataMessage");
const NotificationMessage_1 = require("../messages/NotificationMessage");
const SubscribeMessage_1 = require("../messages/SubscribeMessage");
const CompleteMessage_1 = require("../messages/CompleteMessage");
const ErrorMessage_1 = require("../messages/ErrorMessage");
const UnsubscribeMessage_1 = require("../messages/UnsubscribeMessage");
class Encoder {
    constructor() {
        this.uint8 = new Uint8Array(0);
        this.offset = 0;
    }
    encode(messages) {
        let size = 0;
        const length = messages.length;
        for (let i = 0; i < length; i++)
            size += messages[i].size();
        this.uint8 = new Uint8Array(size);
        this.offset = 0;
        for (let i = 0; i < length; i++) {
            const message = messages[i];
            if (message instanceof NotificationMessage_1.NotificationMessage)
                this.notif(message);
            else if (message instanceof SubscribeMessage_1.SubscribeMessage)
                this.sub(message);
            else if (message instanceof DataMessage_1.DataMessage)
                this.data(message);
            else if (message instanceof CompleteMessage_1.CompleteMessage)
                this.comp(message);
            else if (message instanceof ErrorMessage_1.ErrorMessage)
                this.err(message);
            else if (message instanceof UnsubscribeMessage_1.UnsubscribeMessage)
                this.unsub(message);
        }
        return this.uint8;
    }
    notif(message) {
        const { method, data } = message;
        const length = data ? data.byteLength : 0;
        this.header(0, length);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    sub(message) {
        const { id, method, data } = message;
        const length = data ? data.byteLength : 0;
        this.header(1, length);
        this.id(id);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    data(message) {
        this.msgWithId(2, message.id, message.data);
    }
    comp(message) {
        this.msgWithId(3, message.id, message.data);
    }
    err(message) {
        this.msgWithId(5, message.id, message.data);
    }
    unsub(message) {
        this.uint8[this.offset++] = 4 << 5;
        this.id(message.id);
    }
    msgWithId(code, id, data) {
        const length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    header(code, length) {
        this.offset = (0, header_1.writeHeader)(this.uint8, this.offset, code, length);
    }
    id(id) {
        this.uint8[this.offset++] = id >>> 8;
        this.uint8[this.offset++] = id & 0xff;
    }
    method(method) {
        const uint8 = this.uint8;
        const length = method.length;
        let offset = this.offset;
        uint8[offset++] = length;
        for (let i = 0; i < length; i++)
            uint8[offset++] = method.charCodeAt(i);
        this.offset = offset;
    }
}
exports.Encoder = Encoder;
