"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicalVectorClock = exports.LogicalClock = exports.LogicalTimespan = exports.LogicalTimestamp = void 0;
const server_1 = require("./server");
class LogicalTimestamp extends server_1.ServerTimestamp {
    constructor(sessionId, time) {
        super(time);
        this.sessionId = sessionId;
        this.time = time;
    }
    getSessionId() {
        return this.sessionId;
    }
    stamp(sessionId, time) {
        return new LogicalTimestamp(sessionId, time);
    }
    tick(cycles) {
        return new LogicalTimestamp(this.getSessionId(), this.time + cycles);
    }
    interval(cycles, span) {
        return new LogicalTimespan(this.getSessionId(), this.time + cycles, span);
    }
    clock() {
        return new LogicalClock(this.getSessionId(), this.time);
    }
}
exports.LogicalTimestamp = LogicalTimestamp;
class LogicalTimespan extends LogicalTimestamp {
    constructor(sessionId, time, span) {
        super(sessionId, time);
        this.span = span;
    }
}
exports.LogicalTimespan = LogicalTimespan;
class LogicalClock extends LogicalTimestamp {
    tick(cycles) {
        const timestamp = new LogicalTimestamp(this.sessionId, this.time);
        this.time += cycles;
        return timestamp;
    }
}
exports.LogicalClock = LogicalClock;
class LogicalVectorClock extends LogicalClock {
    constructor() {
        super(...arguments);
        this.clocks = new Map();
    }
    observe(ts, span) {
        const time = ts.time + span - 1;
        const clock = this.clocks.get(ts.getSessionId());
        if (!clock)
            this.clocks.set(ts.getSessionId(), ts.tick(span - 1));
        else if (time > clock.time)
            clock.time = ts.time;
        if (time >= this.time)
            this.time = time + 1;
    }
    clone() {
        return this.fork(this.getSessionId());
    }
    fork(sessionId) {
        const clock = new LogicalVectorClock(sessionId, this.time);
        for (const ts of this.clocks.values())
            clock.observe(ts.tick(0), 1);
        return clock;
    }
}
exports.LogicalVectorClock = LogicalVectorClock;
