"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
const CrdtDecoder_1 = require("../../util/binary/CrdtDecoder");
const clock_1 = require("../../clock");
const PatchBuilder_1 = require("../../PatchBuilder");
class Decoder extends CrdtDecoder_1.CrdtDecoder {
    decode(data) {
        this.reset(data);
        const [isServerClock, x] = this.b1vuint56();
        const clock = isServerClock ? new clock_1.ServerVectorClock(x) : new clock_1.LogicalVectorClock(x, this.vuint57());
        this.patchId = clock.stamp(clock.getSessionId(), clock.time);
        this.builder = new PatchBuilder_1.PatchBuilder(clock);
        this.decodeOperations();
        return this.builder.patch;
    }
    decodeId() {
        const [isServerClock, x] = this.b1vuint56();
        if (isServerClock) {
            return new clock_1.ServerTimestamp(x);
        }
        else {
            const patchId = this.patchId;
            if (x === 1) {
                const delta = this.vuint57();
                return new clock_1.LogicalTimestamp(patchId.getSessionId(), patchId.time + delta);
            }
            else {
                const time = this.vuint57();
                return new clock_1.LogicalTimestamp(x, time);
            }
        }
    }
    decodeOperations() {
        while (this.x < this.uint8.length)
            this.decodeOperation();
    }
    decodeOperation() {
        const opcode = this.u8();
        switch (opcode) {
            case 0: {
                this.builder.obj();
                return;
            }
            case 1: {
                this.builder.arr();
                return;
            }
            case 2: {
                this.builder.str();
                return;
            }
            case 14: {
                this.builder.val(this.val());
                return;
            }
            case 3: {
                this.builder.num();
                return;
            }
            case 4: {
                this.builder.root(this.decodeId());
                return;
            }
            case 5: {
                const object = this.decodeId();
                const fields = this.vuint57();
                const tuples = [];
                for (let i = 0; i < fields; i++) {
                    const value = this.decodeId();
                    const key = this.val();
                    if (typeof key !== 'string')
                        continue;
                    tuples.push([key, value]);
                }
                this.builder.setKeys(object, tuples);
                return;
            }
            case 15: {
                const after = this.decodeId();
                const value = this.val();
                this.builder.setVal(after, value);
                return;
            }
            case 6: {
                const after = this.decodeId();
                const value = this.val();
                if (typeof value !== 'number')
                    return;
                this.builder.setNum(after, value);
                return;
            }
            case 7: {
                const obj = this.decodeId();
                const after = this.decodeId();
                const str = this.val();
                if (typeof str !== 'string')
                    return;
                this.builder.insStr(obj, after, str);
                return;
            }
            case 8: {
                const arr = this.decodeId();
                const after = this.decodeId();
                const length = this.vuint57();
                const elements = [];
                for (let i = 0; i < length; i++)
                    elements.push(this.decodeId());
                this.builder.insArr(arr, after, elements);
                return;
            }
            case 10: {
                const obj = this.decodeId();
                const after = this.decodeId();
                const length = this.vuint57();
                this.builder.del(obj, after, length);
                return;
            }
            case 9: {
                const obj = this.decodeId();
                const after = this.decodeId();
                this.builder.del(obj, after, 1);
                return;
            }
            case 11: {
                this.builder.noop(1);
                return;
            }
            case 12: {
                this.builder.noop(this.vuint57());
                return;
            }
            case 16: {
                this.builder.bin();
                return;
            }
            case 17: {
                const obj = this.decodeId();
                const after = this.decodeId();
                const length = this.vuint57();
                const data = this.bin(length);
                this.builder.insBin(obj, after, data);
                return;
            }
            default: {
                throw new Error('UNKNOWN_OP');
            }
        }
    }
}
exports.Decoder = Decoder;
