"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = void 0;
const encode_1 = require("../../../util/base64/encode");
const DeleteOperation_1 = require("../../operations/DeleteOperation");
const InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
const InsertBinaryDataOperation_1 = require("../../operations/InsertBinaryDataOperation");
const InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
const MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
const MakeBinaryOperation_1 = require("../../operations/MakeBinaryOperation");
const MakeConstantOperation_1 = require("../../operations/MakeConstantOperation");
const MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
const MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
const MakeStringOperation_1 = require("../../operations/MakeStringOperation");
const MakeValueOperation_1 = require("../../operations/MakeValueOperation");
const NoopOperation_1 = require("../../operations/NoopOperation");
const SetNumberOperation_1 = require("../../operations/SetNumberOperation");
const SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
const SetRootOperation_1 = require("../../operations/SetRootOperation");
const SetValueOperation_1 = require("../../operations/SetValueOperation");
const encode = (patch) => {
    const id = patch.getId();
    if (!id)
        throw new Error('PATCH_EMPTY');
    const sessionId = id.getSessionId();
    const { time } = id;
    const res = sessionId === 1 ? [time] : [[sessionId, time]];
    const pushTimestamp = (ts) => {
        const tsSessionId = ts.getSessionId();
        if (tsSessionId === 1)
            res.push(ts.time);
        else if (tsSessionId === sessionId && ts.time >= time)
            res.push(time - ts.time - 1);
        else
            res.push([tsSessionId, ts.time]);
    };
    for (const op of patch.ops) {
        if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
            res.push(0);
            continue;
        }
        if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
            res.push(1);
            continue;
        }
        if (op instanceof MakeStringOperation_1.MakeStringOperation) {
            res.push(2);
            continue;
        }
        if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation) {
            res.push(16);
            continue;
        }
        if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
            res.push(3);
            continue;
        }
        if (op instanceof SetRootOperation_1.SetRootOperation) {
            const { value } = op;
            res.push(4);
            pushTimestamp(value);
            continue;
        }
        if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
            const { object, tuples } = op;
            res.push(5, tuples.length);
            pushTimestamp(object);
            for (const [key, value] of tuples) {
                res.push(key);
                pushTimestamp(value);
            }
            continue;
        }
        if (op instanceof SetNumberOperation_1.SetNumberOperation) {
            const { num, value } = op;
            res.push(6, value);
            pushTimestamp(num);
            continue;
        }
        if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
            const { obj, after, substring } = op;
            res.push(7, substring);
            pushTimestamp(obj);
            pushTimestamp(after);
            continue;
        }
        if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation) {
            const { obj, after, data } = op;
            res.push(17, (0, encode_1.toBase64)(data));
            pushTimestamp(obj);
            pushTimestamp(after);
            continue;
        }
        if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
            const { arr, after, elements } = op;
            res.push(8, elements.length);
            pushTimestamp(arr);
            pushTimestamp(after);
            for (const element of elements)
                pushTimestamp(element);
            continue;
        }
        if (op instanceof DeleteOperation_1.DeleteOperation) {
            const { obj, after } = op;
            const length = after.span;
            if (length === 1) {
                res.push(9);
                pushTimestamp(obj);
                pushTimestamp(after);
            }
            else {
                res.push(10, length);
                pushTimestamp(obj);
                pushTimestamp(after);
            }
            continue;
        }
        if (op instanceof NoopOperation_1.NoopOperation) {
            const { length } = op;
            if (length === 1)
                res.push(11);
            else
                res.push(12, length);
            continue;
        }
        if (op instanceof MakeConstantOperation_1.MakeConstantOperation) {
            res.push(13);
            res.push(op.value);
            continue;
        }
        if (op instanceof MakeValueOperation_1.MakeValueOperation) {
            res.push(14);
            res.push(op.value);
            continue;
        }
        if (op instanceof SetValueOperation_1.SetValueOperation) {
            res.push(15);
            pushTimestamp(op.obj);
            res.push(op.value);
            continue;
        }
    }
    return res;
};
exports.encode = encode;
