"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
const ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
const ArrayType_1 = require("../../types/rga-array/ArrayType");
const BinaryChunk_1 = require("../../types/rga-binary/BinaryChunk");
const BinaryType_1 = require("../../types/rga-binary/BinaryType");
const ClockDecoder_1 = require("../../../json-crdt-patch/codec/clock/ClockDecoder");
const ConstantType_1 = require("../../types/const/ConstantType");
const CrdtDecoder_1 = require("../../../json-crdt-patch/util/binary/CrdtDecoder");
const DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
const constants_1 = require("../../../json-crdt-patch/constants");
const constants_2 = require("../../constants");
const clock_1 = require("../../../json-crdt-patch/clock");
const model_1 = require("../../model");
const ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
const ObjectType_1 = require("../../types/lww-object/ObjectType");
const StringChunk_1 = require("../../types/rga-string/StringChunk");
const StringType_1 = require("../../types/rga-string/StringType");
const ValueType_1 = require("../../types/lww-value/ValueType");
class Decoder extends CrdtDecoder_1.CrdtDecoder {
    constructor() {
        super(...arguments);
        this.time = -1;
    }
    decode(data) {
        delete this.clockDecoder;
        this.time = -1;
        this.reset(data);
        const [isServerTime, x] = this.b1vuint56();
        if (isServerTime) {
            this.time = x;
            this.doc = model_1.Model.withServerClock(x);
        }
        else {
            this.decodeClockTable(x);
            this.doc = model_1.Model.withLogicalClock(this.clockDecoder.clock);
        }
        this.literals = this.val();
        if (!(this.literals instanceof Array))
            throw new Error('INVALID_LITERALS');
        this.decodeRoot();
        delete this.clockDecoder;
        delete this.literals;
        return this.doc;
    }
    decodeClockTable(length) {
        const [sessionId, time] = this.uint53vuint39();
        this.clockDecoder = new ClockDecoder_1.ClockDecoder(sessionId, 0);
        this.clockDecoder.pushTuple(sessionId, time);
        for (let i = 1; i < length; i++) {
            const ts = this.uint53vuint39();
            this.clockDecoder.pushTuple(ts[0], ts[1]);
        }
    }
    ts() {
        if (this.time < 0) {
            const [sessionIndex, timeDiff] = this.id();
            return this.clockDecoder.decodeId(sessionIndex, timeDiff);
        }
        else {
            const [isServerTime, x] = this.b1vuint56();
            if (isServerTime) {
                return new clock_1.ServerTimestamp(this.time - x);
            }
            else {
                const sessionId = x;
                const time = this.vuint57();
                if (sessionId === 0) {
                    switch (time) {
                        case 1:
                            return constants_1.NULL_ID;
                        case 2:
                            return constants_1.TRUE_ID;
                        case 3:
                            return constants_1.FALSE_ID;
                        case 4:
                            return constants_1.UNDEFINED_ID;
                    }
                }
                return new clock_1.LogicalTimestamp(sessionId, time);
            }
        }
    }
    decodeRoot() {
        const doc = this.doc;
        const id = this.ts();
        const node = this.x >= this.uint8.byteLength ? null : this.decodeNode();
        doc.root = new DocRootType_1.DocRootType(doc, id, node);
    }
    decodeNode() {
        const byte = this.u8();
        if (byte < 0b10000000)
            return this.createConst(byte);
        else if (byte <= 0b10001111)
            return this.decodeObj(byte & 0b1111);
        else if (byte <= 0b10011111)
            return this.decodeArr(byte & 0b1111);
        else if (byte <= 0b10111111)
            return this.decodeStr(byte & 0b11111);
        else if (byte >= 0b11100000)
            return this.createConst(byte - 0x100);
        else {
            switch (byte) {
                case 0xc0:
                    return constants_2.NULL;
                case 0xc1:
                    return constants_2.UNDEFINED;
                case 0xc2:
                    return constants_2.FALSE;
                case 0xc3:
                    return constants_2.TRUE;
                case 0xc4:
                    return this.decodeBin(this.u8());
                case 0xc5:
                    return this.decodeBin(this.u16());
                case 0xc6:
                    return this.decodeBin(this.u32());
                case 0xca:
                    return this.createConst(this.f32());
                case 0xcb:
                    return this.createConst(this.f64());
                case 0xcc:
                    return this.createConst(this.u8());
                case 0xcd:
                    return this.createConst(this.u16());
                case 0xce:
                    return this.createConst(this.u32());
                case 0xcf:
                    return this.createConst(this.u32() * 4294967296 + this.u32());
                case 0xd0:
                    return this.createConst(this.i8());
                case 0xd1:
                    return this.createConst(this.i16());
                case 0xd2:
                    return this.createConst(this.i32());
                case 0xd3:
                    return this.createConst(this.i32() * 4294967296 + this.i32());
                case 0xd4:
                    return this.createConst(this.val());
                case 0xd5:
                    return this.decodeVal();
                case 0xd6:
                    return this.decodeValLiteral();
                case 0xde:
                    return this.decodeObj(this.u16());
                case 0xdf:
                    return this.decodeObj(this.u32());
                case 0xdc:
                    return this.decodeArr(this.u16());
                case 0xdd:
                    return this.decodeArr(this.u32());
                case 0xd9:
                    return this.decodeStr(this.u8());
                case 0xda:
                    return this.decodeStr(this.u16());
                case 0xdb:
                    return this.decodeStr(this.u32());
            }
        }
        return constants_2.NULL;
    }
    createConst(value) {
        return new ConstantType_1.ConstantType(constants_1.ORIGIN, value);
    }
    decodeObj(length) {
        const id = this.ts();
        const obj = new ObjectType_1.ObjectType(this.doc, id);
        for (let i = 0; i < length; i++)
            this.decodeObjChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    }
    decodeObjChunk(obj) {
        const id = this.ts();
        const [isLiteralIndex, x] = this.b1vuint56();
        let key;
        if (isLiteralIndex) {
            key = this.literals[x];
        }
        else {
            key = this.str(x);
        }
        const node = this.decodeNode();
        const chunk = new ObjectChunk_1.ObjectChunk(id, node);
        obj.putChunk(key, chunk);
    }
    decodeArr(length) {
        const id = this.ts();
        const obj = new ArrayType_1.ArrayType(this.doc, id);
        for (let i = 0; i < length; i++)
            this.decodeArrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    }
    decodeArrChunk(obj) {
        const [deleted, length] = this.b1vuint56();
        const id = this.ts();
        if (deleted) {
            const chunk = new ArrayChunk_1.ArrayChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            const nodes = [];
            for (let i = 0; i < length; i++)
                nodes.push(this.decodeNode());
            const chunk = new ArrayChunk_1.ArrayChunk(id, nodes);
            obj.append(chunk);
        }
    }
    decodeStr(length) {
        const id = this.ts();
        const obj = new StringType_1.StringType(this.doc, id);
        for (let i = 0; i < length; i++)
            this.decodeStrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    }
    decodeStrChunk(obj) {
        const [deleted, length] = this.b1vuint56();
        const id = this.ts();
        if (deleted) {
            const chunk = new StringChunk_1.StringChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            let text;
            if (length === 0) {
                const literalIndex = this.vuint57();
                text = this.literals[literalIndex];
            }
            else {
                text = this.str(length);
            }
            const chunk = new StringChunk_1.StringChunk(id, text);
            obj.append(chunk);
        }
    }
    decodeBin(length) {
        const id = this.ts();
        const obj = new BinaryType_1.BinaryType(this.doc, id);
        for (let i = 0; i < length; i++)
            this.decodeBinChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    }
    decodeBinChunk(obj) {
        const [deleted, length] = this.b1vuint56();
        const id = this.ts();
        if (deleted) {
            const chunk = new BinaryChunk_1.BinaryChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            const text = this.bin(length);
            const chunk = new BinaryChunk_1.BinaryChunk(id, text);
            obj.append(chunk);
        }
    }
    decodeVal() {
        const id = this.ts();
        const writeId = this.ts();
        const value = this.val();
        const obj = new ValueType_1.ValueType(id, writeId, value);
        this.doc.nodes.index(obj);
        return obj;
    }
    decodeValLiteral() {
        const id = this.ts();
        const writeId = this.ts();
        const literalIndex = this.vuint57();
        const value = this.literals[literalIndex];
        const obj = new ValueType_1.ValueType(id, writeId, value);
        this.doc.nodes.index(obj);
        return obj;
    }
}
exports.Decoder = Decoder;
