"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Finder = void 0;
const ArrayApi_1 = require("./ArrayApi");
const ArrayType_1 = require("../../types/rga-array/ArrayType");
const BinaryApi_1 = require("./BinaryApi");
const BinaryType_1 = require("../../types/rga-binary/BinaryType");
const constants_1 = require("../../constants");
const ObjectApi_1 = require("./ObjectApi");
const ObjectType_1 = require("../../types/lww-object/ObjectType");
const StringApi_1 = require("./StringApi");
const StringType_1 = require("../../types/rga-string/StringType");
const ValueApi_1 = require("./ValueApi");
const ValueType_1 = require("../../types/lww-value/ValueType");
class Finder {
    constructor(node, api) {
        this.node = node;
        this.api = api;
    }
    find(steps) {
        let node = this.node || constants_1.NULL;
        const length = steps.length;
        if (!length)
            return node;
        let i = 0;
        while (i < length) {
            const step = steps[i++];
            if (node instanceof ObjectType_1.ObjectType) {
                const id = node.get(String(step));
                if (!id)
                    return constants_1.UNDEFINED;
                const nextNode = this.api.model.node(id);
                if (!nextNode)
                    return constants_1.UNDEFINED;
                node = nextNode;
            }
            else if (node instanceof ArrayType_1.ArrayType) {
                const id = node.findValue(Number(step));
                const nextNode = this.api.model.node(id);
                if (!nextNode)
                    return constants_1.UNDEFINED;
                node = nextNode;
                continue;
            }
        }
        return node;
    }
    val(path) {
        const node = this.find(path);
        if (node instanceof ValueType_1.ValueType)
            return new ValueApi_1.ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    }
    str(path) {
        const node = this.find(path);
        if (node instanceof StringType_1.StringType)
            return new StringApi_1.StringApi(this.api, node);
        throw new Error('NOT_STR');
    }
    bin(path) {
        const node = this.find(path);
        if (node instanceof BinaryType_1.BinaryType)
            return new BinaryApi_1.BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    }
    arr(path) {
        const node = this.find(path);
        if (node instanceof ArrayType_1.ArrayType)
            return new ArrayApi_1.ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    }
    obj(path) {
        const obj = this.find(path);
        if (obj instanceof ObjectType_1.ObjectType)
            return new ObjectApi_1.ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    }
}
exports.Finder = Finder;
