"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringChunk = void 0;
const clock_1 = require("../../../json-crdt-patch/clock");
class StringChunk {
    constructor(id, str) {
        this.id = id;
        this.str = str;
        this.left = null;
        this.right = null;
    }
    span() {
        return this.deleted || this.str.length;
    }
    split(time) {
        const newSpan = 1 + time - this.id.time;
        const newId = new clock_1.LogicalTimestamp(this.id.getSessionId(), time + 1);
        if (this.str) {
            const newChunkStr = this.str.substr(newSpan);
            this.str = this.str.substr(0, newSpan);
            return new StringChunk(newId, newChunkStr);
        }
        else {
            const chunk = new StringChunk(newId, undefined);
            chunk.deleted = this.span() - newSpan;
            this.deleted = newSpan;
            return chunk;
        }
    }
    delete() {
        this.deleted = this.span();
        delete this.str;
    }
    merge(substring) {
        this.str += substring;
    }
    clone() {
        const chunk = new StringChunk(this.id, this.str);
        if (this.deleted) {
            chunk.deleted = this.deleted;
            delete chunk.str;
        }
        return chunk;
    }
    toString(tab = '') {
        const str = `${tab}${this.constructor.name}(${this.id.toDisplayString()}) { ${!this.str ? `[${this.deleted || this.span()}]` : this.str.length > 10 ? this.str.substr(0, 10) + '…' : this.str} }`;
        return str;
    }
}
exports.StringChunk = StringChunk;
