"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toJsonSchema = void 0;
const util_1 = require("../json-type/util");
const augmentNode = (type, node) => {
    if (type.title)
        node.title = type.title;
    if (type.description)
        node.description = type.description;
    if (type.examples)
        node.examples = type.examples.map((example) => example.value);
};
const toJsonSchema = (type, ctx) => {
    switch (type.__t) {
        case 'str': {
            const node = { type: 'string' };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'num': {
            const node = {
                type: 'number',
            };
            if (type.format)
                if (util_1.INTS.indexOf(type.format) > -1)
                    node.type = 'integer';
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'bool': {
            const node = {
                type: 'boolean',
            };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'nil': {
            const node = {
                type: 'null',
            };
            augmentNode(type, node);
            return node;
        }
        case 'arr': {
            const node = {
                type: 'array',
                items: (0, exports.toJsonSchema)(type.type, ctx),
            };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'obj': {
            const node = {
                type: 'object',
                properties: {},
            };
            const required = [];
            for (const field of type.fields) {
                node.properties[field.key] = (0, exports.toJsonSchema)(field.type, ctx);
                if (field.isOptional === undefined || !field.isOptional)
                    required.push(field.key);
            }
            if (required.length)
                node.required = required;
            augmentNode(type, node);
            return node;
        }
        case 'ref': {
            if (!ctx.ref) {
                const node = { $ref: '/' + type.ref };
                return node;
            }
            else if (ctx.defs) {
                const node = { $ref: '#/$defs/' + type.ref };
                if (!ctx.defs[type.ref])
                    ctx.defs[type.ref] = (0, exports.toJsonSchema)(ctx.ref(type.ref), ctx);
                return node;
            }
            return (0, exports.toJsonSchema)(ctx.ref(type.ref), ctx);
        }
        case 'any':
        default: {
            const node = {
                type: ['string', 'number', 'boolean', 'null', 'array', 'object'],
            };
            augmentNode(type, node);
            return node;
        }
    }
};
exports.toJsonSchema = toJsonSchema;
