import type { MsgPack, Encoder } from '../json-pack';
import type { EncoderFull } from '../json-pack/EncoderFull';
import { TArray, TBoolean, TNumber, TObject, TObjectField, TString, TType, TRef } from '../json-type/types';
import { JsExpression } from '../util/codegen/util/JsExpression';
import { Codegen, JavaScriptLinked } from '../util/codegen';
export declare type EncoderFn = <T>(value: T) => MsgPack<T>;
export declare type PartialEncoderFn = <T>(value: T, encoder: EncoderFull) => void;
export interface MsgPackSerializerCodegenOptions {
    encoder: EncoderFull;
    type: TType;
    ref?: (id: string) => PartialEncoderFn | TType | undefined;
}
export declare class MsgPackSerializerCodegen {
    protected codegen: Codegen<EncoderFn>;
    protected options: Required<MsgPackSerializerCodegenOptions>;
    constructor(opts: MsgPackSerializerCodegenOptions);
    protected execJs(js: string): void;
    protected writeBlob(arr: Uint8Array): void;
    protected getBlob(callback: (encoder: Encoder) => void): Uint8Array;
    protected registerCounter: number;
    protected getRegister(): string;
    protected genAndWriteBlob(callback: (encoder: Encoder) => void): void;
    onString(str: TString, value: JsExpression): void;
    onNumber(num: TNumber, value: JsExpression): void;
    onBoolean(bool: TBoolean, value: JsExpression): void;
    onNull(): void;
    onArray(arr: TArray, value: JsExpression): void;
    onObject(obj: TObject, value: JsExpression): void;
    onObjectVariableLength(obj: TObject, value: JsExpression): void;
    onObjectFixedLength(obj: TObject, value: JsExpression): void;
    protected onRequiredFields(requiredFields: TObjectField[], value: JsExpression, r: string): void;
    onBinary(value: JsExpression): void;
    onRef(ref: TRef, value: JsExpression): void;
    onAny(value: JsExpression): void;
    onType(type: TType, value: JsExpression): void;
    private codegenBlob;
    run(): this;
    generate(): JavaScriptLinked<EncoderFn>;
    compile(): EncoderFn;
    compilePartial(): PartialEncoderFn;
}
