"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createObjValidator = exports.createStrValidator = exports.createBoolValidator = exports.createValidator = exports.JsonTypeValidatorCodegen = exports.JsonTypeValidatorErrorMessage = exports.JsonTypeValidatorError = void 0;
const codegen_1 = require("../util/codegen");
const __deepEqual_1 = require("../json-equal/$$deepEqual");
const normalizeAccessor_1 = require("../util/codegen/util/normalizeAccessor");
const util_1 = require("../json-type/util");
class EncodingPlanStepExecJs {
    constructor(js) {
        this.js = js;
    }
}
const canSkipObjectKeyUndefinedCheck = (type) => {
    switch (type) {
        case 'bool':
        case 'num':
        case 'str':
        case 'nil':
        case 'obj':
        case 'arr':
        case 'bin':
            return true;
        default:
            return false;
    }
};
var JsonTypeValidatorError;
(function (JsonTypeValidatorError) {
    JsonTypeValidatorError[JsonTypeValidatorError["STR"] = 0] = "STR";
    JsonTypeValidatorError[JsonTypeValidatorError["STR_CONST"] = 1] = "STR_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["NUM"] = 2] = "NUM";
    JsonTypeValidatorError[JsonTypeValidatorError["NUM_CONST"] = 3] = "NUM_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["BOOL"] = 4] = "BOOL";
    JsonTypeValidatorError[JsonTypeValidatorError["BOOL_CONST"] = 5] = "BOOL_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["NIL"] = 6] = "NIL";
    JsonTypeValidatorError[JsonTypeValidatorError["ARR"] = 7] = "ARR";
    JsonTypeValidatorError[JsonTypeValidatorError["OBJ"] = 8] = "OBJ";
    JsonTypeValidatorError[JsonTypeValidatorError["KEY"] = 9] = "KEY";
    JsonTypeValidatorError[JsonTypeValidatorError["KEYS"] = 10] = "KEYS";
    JsonTypeValidatorError[JsonTypeValidatorError["BIN"] = 11] = "BIN";
    JsonTypeValidatorError[JsonTypeValidatorError["OR"] = 12] = "OR";
    JsonTypeValidatorError[JsonTypeValidatorError["REF"] = 13] = "REF";
    JsonTypeValidatorError[JsonTypeValidatorError["ENUM"] = 14] = "ENUM";
    JsonTypeValidatorError[JsonTypeValidatorError["VALIDATION"] = 15] = "VALIDATION";
    JsonTypeValidatorError[JsonTypeValidatorError["INT"] = 16] = "INT";
    JsonTypeValidatorError[JsonTypeValidatorError["UINT"] = 17] = "UINT";
    JsonTypeValidatorError[JsonTypeValidatorError["STR_LEN"] = 18] = "STR_LEN";
})(JsonTypeValidatorError = exports.JsonTypeValidatorError || (exports.JsonTypeValidatorError = {}));
exports.JsonTypeValidatorErrorMessage = {
    [JsonTypeValidatorError.STR]: 'Not a string.',
    [JsonTypeValidatorError.STR_CONST]: 'Invalid string constant.',
    [JsonTypeValidatorError.NUM]: 'Not a number.',
    [JsonTypeValidatorError.NUM_CONST]: 'Invalid number constant.',
    [JsonTypeValidatorError.BOOL]: 'Not a boolean.',
    [JsonTypeValidatorError.BOOL_CONST]: 'Invalid boolean constant.',
    [JsonTypeValidatorError.NIL]: 'Not null.',
    [JsonTypeValidatorError.ARR]: 'Not an array.',
    [JsonTypeValidatorError.OBJ]: 'Not an object.',
    [JsonTypeValidatorError.KEY]: 'Missing key.',
    [JsonTypeValidatorError.KEYS]: 'Too many or missing object keys.',
    [JsonTypeValidatorError.BIN]: 'Not a binary.',
    [JsonTypeValidatorError.OR]: 'None of types matched.',
    [JsonTypeValidatorError.REF]: 'Validation error in referenced type.',
    [JsonTypeValidatorError.ENUM]: 'Not an enum value.',
    [JsonTypeValidatorError.VALIDATION]: 'Custom validator failed.',
    [JsonTypeValidatorError.INT]: 'Not an integer.',
    [JsonTypeValidatorError.UINT]: 'Not an unsigned integer.',
    [JsonTypeValidatorError.STR_LEN]: 'Invalid string length.',
};
class JsonTypeValidatorCodegen {
    constructor(options = {}) {
        this.steps = [];
        this.registerCounter = 0;
        this.dependencies = [];
        this.consts = [];
        this.options = {
            errorReporting: 'boolean',
            skipObjectExtraFieldsCheck: false,
            unsafeMode: false,
            ref: () => undefined,
            customValidators: [],
            ...options,
        };
    }
    js(js) {
        this.steps.push(new EncodingPlanStepExecJs(js));
    }
    getRegister() {
        return `r${++this.registerCounter}`;
    }
    addDependencies(deps) {
        const symbols = [];
        for (const dep of deps) {
            this.dependencies.push(dep);
            symbols.push(`d${this.dependencies.length - 1}`);
        }
        return symbols;
    }
    addConsts(consts) {
        const symbols = [];
        for (const const_ of consts) {
            this.consts.push(const_);
            symbols.push(`c${this.consts.length - 1}`);
        }
        return symbols;
    }
    err(code, path, opts = {}) {
        switch (this.options.errorReporting) {
            case 'boolean':
                return 'true';
            case 'string': {
                let out = "'[" + JSON.stringify(JsonTypeValidatorError[code]);
                for (const step of path) {
                    if (typeof step === 'object') {
                        out += ",' + JSON.stringify(" + step.r + ") + '";
                    }
                    else {
                        out += ',' + JSON.stringify(step);
                    }
                }
                return out + "]'";
            }
            case 'object':
            default: {
                let out = '{code: ' +
                    JSON.stringify(JsonTypeValidatorError[code]) +
                    ', errno: ' +
                    JSON.stringify(code) +
                    ', message: ' +
                    JSON.stringify(exports.JsonTypeValidatorErrorMessage[code]) +
                    ', path: [';
                let i = 0;
                for (const step of path) {
                    if (i)
                        out += ', ';
                    if (typeof step === 'object') {
                        out += step.r;
                    }
                    else {
                        out += JSON.stringify(step);
                    }
                    i++;
                }
                out += ']';
                if (this.type.id) {
                    out += ', type: ' + JSON.stringify(this.type.id || '');
                }
                if (opts.refError) {
                    out += ', ref: ' + opts.refError;
                }
                if (opts.validator) {
                    out += ', validator: ' + JSON.stringify(opts.validator);
                }
                return out + '}';
            }
        }
    }
    refErr(path, r) {
        switch (this.options.errorReporting) {
            case 'boolean':
                return r;
            case 'string': {
                return this.err(JsonTypeValidatorError.REF, [...path, { r }]);
            }
            case 'object':
            default: {
                return this.err(JsonTypeValidatorError.REF, [...path], { refError: r });
            }
        }
    }
    emitValidations(path, validatorNames, type, r) {
        for (const validatorName of validatorNames) {
            const v = this.linkValidator(validatorName, type);
            const rerr = this.getRegister();
            const rc = this.getRegister();
            const err = this.err(JsonTypeValidatorError.VALIDATION, path, { validator: validatorName, refError: rerr });
            const errInCatch = this.err(JsonTypeValidatorError.VALIDATION, path, { validator: validatorName, refError: rc });
            this.js(`try {`);
            this.js(`const ${rerr} = ${v}(${r});`);
            this.js(`if (${rerr}) return ${err};`);
            this.js(`} catch (e) {`);
            this.js(`var ${rc} = e ? e : new Error('Validator ${JSON.stringify(validatorName)} failed.');`);
            this.js(`return ${errInCatch};`);
            this.js(`}`);
        }
    }
    linkValidator(name, type) {
        for (const validator of this.options.customValidators) {
            if (name !== validator.name)
                continue;
            if (validator.types.indexOf(type) === -1)
                continue;
            return this.addDependencies([validator.fn])[0];
        }
        throw new Error(`Custom validator "${name}" not found.`);
    }
    onString(path, str, r) {
        if (str.const !== undefined) {
            const error = this.err(JsonTypeValidatorError.STR_CONST, path);
            this.js(`if(${r} !== ${JSON.stringify(str.const)}) return ${error};`);
        }
        else {
            const error = this.err(JsonTypeValidatorError.STR, path);
            this.js(`if(typeof ${r} !== "string") return ${error};`);
            if (typeof str.minLength === 'number')
                if (!Number.isInteger(str.minLength) || str.minLength < 0)
                    throw new Error(`Invalid minLength ${str.minLength} for string type.`);
            if (typeof str.maxLength === 'number')
                if (!Number.isInteger(str.maxLength) || str.maxLength < 0)
                    throw new Error(`Invalid maxLength ${str.maxLength} for string type.`);
            if (typeof str.minLength === 'number' && typeof str.maxLength === 'number') {
                if (str.minLength > str.maxLength)
                    throw new Error(`Invalid minLength ${str.minLength} and maxLength ${str.maxLength} for string type.`);
            }
            if (typeof str.minLength === 'number' && typeof str.maxLength === 'number' && str.minLength === str.maxLength) {
                const strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                this.js(`if(${r}.length !== ${str.minLength}) return ${strLenError};`);
            }
            else {
                if (typeof str.minLength === 'number') {
                    const strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                    this.js(`if(${r}.length < ${str.minLength}) return ${strLenError};`);
                }
                if (typeof str.maxLength === 'number') {
                    const strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                    this.js(`if(${r}.length > ${str.maxLength}) return ${strLenError};`);
                }
            }
        }
        if (str.validator) {
            this.emitValidations(path, str.validator instanceof Array ? str.validator : [str.validator], 'string', r);
        }
    }
    onNumber(path, num, r) {
        if (num.const !== undefined) {
            const err = this.err(JsonTypeValidatorError.NUM_CONST, path);
            this.js(`if(${r} !== ${JSON.stringify(num.const)}) return ${err};`);
        }
        else {
            if (num.format && util_1.INTS.indexOf(num.format) !== -1) {
                const errInt = this.err(JsonTypeValidatorError.INT, path);
                this.js(`if(!Number.isInteger(${r})) return ${errInt};`);
                if (util_1.UINTS.indexOf(num.format) !== -1) {
                    const errUint = this.err(JsonTypeValidatorError.UINT, path);
                    this.js(`if(${r} < 0) return ${errUint};`);
                    switch (num.format) {
                        case 'u8': {
                            this.js(`if(${r} > 0xFF) return ${errUint};`);
                            break;
                        }
                        case 'u16': {
                            this.js(`if(${r} > 0xFFFF) return ${errUint};`);
                            break;
                        }
                        case 'u32': {
                            this.js(`if(${r} > 0xFFFFFFFF) return ${errUint};`);
                            break;
                        }
                    }
                }
                else {
                    switch (num.format) {
                        case 'i8': {
                            this.js(`if(${r} > 0x7F || ${r} < -0x80) return ${errInt};`);
                            break;
                        }
                        case 'i16': {
                            this.js(`if(${r} > 0x7FFF || ${r} < -0x8000) return ${errInt};`);
                            break;
                        }
                        case 'i32': {
                            this.js(`if(${r} > 0x7FFFFFFF || ${r} < -0x80000000) return ${errInt};`);
                            break;
                        }
                    }
                }
            }
            else {
                const errNum = this.err(JsonTypeValidatorError.NUM, path);
                this.js(`if(typeof ${r} !== "number") return ${errNum};`);
            }
        }
    }
    onBoolean(path, bool, r) {
        if (typeof bool.const === 'boolean') {
            const err = this.err(JsonTypeValidatorError.BOOL_CONST, path);
            this.js(`if(${r} !== ${bool.const ? 'true' : 'false'}) return ${err};`);
        }
        else {
            const err = this.err(JsonTypeValidatorError.BOOL, path);
            this.js(`if(typeof ${r} !== "boolean") return ${err};`);
        }
    }
    onNull(path, r) {
        const err = this.err(JsonTypeValidatorError.NIL, path);
        this.js(`if(${r} !== null) return ${err};`);
    }
    onArray(path, arr, expr) {
        const r = this.getRegister();
        const ri = this.getRegister();
        const rv = this.getRegister();
        this.js(`var ${r} = ${expr};`);
        const err = this.err(JsonTypeValidatorError.ARR, path);
        this.js(`if (!(${r} instanceof Array)) return ${err};`);
        this.js(`for (var ${rv}, ${ri} = ${r}.length; ${ri}-- !== 0;) {`);
        this.js(`${rv} = ${r}[${ri}];`);
        this.onType([...path, { r: ri }], arr.type, rv);
        this.js(`}`);
    }
    onObject(path, obj, expr) {
        const r = this.getRegister();
        this.js(`var ${r} = ${expr};`);
        const canSkipObjectTypeCheck = this.options.unsafeMode && obj.fields.length > 0;
        if (!canSkipObjectTypeCheck) {
            const err = this.err(JsonTypeValidatorError.OBJ, path);
            this.js(`if (typeof ${r} !== 'object' || !${r} || (${r} instanceof Array)) return ${err};`);
        }
        if (obj.fields.length && !obj.unknownFields && !this.options.skipObjectExtraFieldsCheck) {
            const rk = this.getRegister();
            this.js(`for (var ${rk} in ${r}) {`);
            this.js(`switch (${rk}) { case ${obj.fields
                .map((field) => JSON.stringify(field.key))
                .join(': case ')}: break; default: return ${this.err(JsonTypeValidatorError.KEYS, [...path, { r: rk }])};}`);
            this.js(`}`);
        }
        for (let i = 0; i < obj.fields.length; i++) {
            const field = obj.fields[i];
            const rv = this.getRegister();
            const accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            const keyPath = [...path, field.key];
            if (field.isOptional) {
                this.js(`var ${rv} = ${r}${accessor};`);
                this.js(`if (${rv} !== undefined) {`);
                this.onType(keyPath, field.type, rv);
                this.js(`}`);
            }
            else {
                if (canSkipObjectKeyUndefinedCheck(field.type.__t)) {
                    this.onType(keyPath, field.type, `${r}${accessor}`);
                }
                else {
                    this.js(`var ${rv} = ${r}${accessor};`);
                    const err = this.err(JsonTypeValidatorError.KEY, [...path, field.key]);
                    this.js(`if (${rv} === undefined) return ${err};`);
                    this.onType(keyPath, field.type, rv);
                }
            }
        }
    }
    onBinary(path, expr) {
        const hasBuffer = typeof Buffer === 'function';
        const r = this.getRegister();
        this.js(`var ${r} = ${expr};`);
        const err = this.err(JsonTypeValidatorError.BIN, path);
        this.js(`if(!(${r} instanceof Uint8Array)${hasBuffer ? ` && !Buffer.isBuffer(${r})` : ''}) return ${err};`);
    }
    onRef(path, ref, expr) {
        const validator = this.options.ref(ref.ref);
        if (!validator) {
            throw new Error(`Could not resolve validator for [ref = ${ref.ref}]. Provide it through .ref option.`);
        }
        const [d] = this.addDependencies([validator]);
        const rerr = this.getRegister();
        this.js(`var ${rerr} = ${d}(${expr});`);
        this.js(`if (${rerr}) return ${this.refErr(path, rerr)};`);
    }
    onOr(path, or, expr) {
        if (or.types.length === 1) {
            this.onType(path, or.types[0], expr);
            return;
        }
        const consts = this.addConsts(or.types.map((type) => {
            const codegen = new JsonTypeValidatorCodegen({ ...this.options, errorReporting: 'boolean' });
            const fn = codegen.generate(type);
            const symbols = this.addDependencies(fn.deps);
            return `((${fn.js})(${symbols.join(', ')}))`;
        })).map((c) => `${c}(${expr})`);
        const err = this.err(JsonTypeValidatorError.OR, path);
        this.js(`if (${consts.join(' && ')}) return ${err}`);
    }
    onEnum(path, type, r) {
        if (!type.values.length)
            throw new Error('Enum values are not specified.');
        const equals = type.values.map((value) => (0, __deepEqual_1.$$deepEqual)(value));
        const consts = this.addConsts(equals);
        const exprs = consts.map((c) => `!${c}(${r})`);
        this.js(`if (${exprs.join(' && ')}) return ${this.err(JsonTypeValidatorError.ENUM, path)}`);
    }
    onType(path, type, expr) {
        switch (type.__t) {
            case 'str': {
                this.onString(path, type, expr);
                break;
            }
            case 'num': {
                this.onNumber(path, type, expr);
                break;
            }
            case 'bool': {
                this.onBoolean(path, type, expr);
                break;
            }
            case 'nil': {
                this.onNull(path, expr);
                break;
            }
            case 'arr': {
                this.onArray(path, type, expr);
                break;
            }
            case 'obj': {
                this.onObject(path, type, expr);
                break;
            }
            case 'bin': {
                this.onBinary(path, expr);
                break;
            }
            case 'any': {
                break;
            }
            case 'ref': {
                this.onRef(path, type, expr);
                break;
            }
            case 'or': {
                this.onOr(path, type, expr);
                break;
            }
            case 'enum': {
                this.onEnum(path, type, expr);
                break;
            }
            default: {
                throw new Error(`Unknown node type [__t = ${type.__t}].`);
            }
        }
    }
    generate(type) {
        this.type = type;
        this.onType([], type, 'r0');
        const successResult = this.options.errorReporting === 'boolean' ? 'false' : this.options.errorReporting === 'string' ? "''" : 'null';
        const dependencyArgs = this.dependencies.map((_, index) => `d${index}`).join(', ');
        const js = `(function(${dependencyArgs}){
${this.consts.map((value, index) => `var c${index} = (${value});`).join('\n')}
return function(r0){
${this.steps.map((step) => step.js).join('\n')}
return ${successResult};
}})`;
        return {
            deps: this.dependencies,
            js,
        };
    }
}
exports.JsonTypeValidatorCodegen = JsonTypeValidatorCodegen;
const createValidator = (type, options = {}) => {
    const codegen = new JsonTypeValidatorCodegen(options);
    const fn = codegen.generate(type);
    return (0, codegen_1.compileClosure)(fn);
};
exports.createValidator = createValidator;
const createBoolValidator = (type, options = {}) => {
    return (0, exports.createValidator)(type, {
        ...options,
        errorReporting: 'boolean',
    });
};
exports.createBoolValidator = createBoolValidator;
const createStrValidator = (type, options = {}) => {
    return (0, exports.createValidator)(type, {
        ...options,
        errorReporting: 'string',
    });
};
exports.createStrValidator = createStrValidator;
const createObjValidator = (type, options = {}) => {
    return (0, exports.createValidator)(type, {
        ...options,
        errorReporting: 'object',
    });
};
exports.createObjValidator = createObjValidator;
