import { TNull, TBoolean, TNumber, TString, TArray, TObject, TObjectField, NoT, TBinary, TType, TAny, TRef, TOr, TEnum } from './types/type';
export declare class JsonTypeBuilder {
    get str(): TString;
    get num(): TNumber;
    get bool(): TBoolean;
    get nil(): TNull;
    get arr(): TArray;
    get obj(): TObject;
    get bin(): TBinary;
    get any(): TAny;
    Null(options?: NoT<TNull>): TNull;
    Boolean(id: string, options?: Omit<NoT<TBoolean>, 'id'>): TBoolean;
    Boolean(options?: NoT<TBoolean>): TBoolean;
    Number(id: string, options?: Omit<NoT<TNumber>, 'id'>): TNumber;
    Number(options?: NoT<TNumber>): TNumber;
    String(id: string, options?: NoT<TString>): TString;
    String(options?: NoT<TString>): TString;
    Array(id: string, type: TType, options?: Omit<NoT<TArray>, 'id' | 'type'>): TArray;
    Array(type: TType, options?: Omit<NoT<TArray>, 'type'>): TArray;
    Object(id: string, options: Omit<NoT<TObject>, 'id'>): TObject;
    Object(fields: TObject['fields'], options?: Omit<NoT<TObject>, 'fields'>): TObject;
    Object(id: string, fields: TObject['fields'], options?: Omit<NoT<TObject>, 'id' | 'fields'>): TObject;
    Object(options: NoT<TObject>): TObject;
    Field(key: string, type: TType, options?: Omit<TObjectField, 'key' | 'type'>): TObjectField;
    Binary(options?: NoT<TBinary>): TBinary;
    Any(options?: NoT<TAny>): TAny;
    Ref(ref: string): TRef;
    Or(types: TType[]): TOr;
    Or(...types: TType[]): TOr;
    Enum(options: NoT<TEnum>): TEnum;
    Enum(values: unknown[], options?: Omit<NoT<TEnum>, 'values'>): TEnum;
}
