"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const binary_1 = require("../../messages/binary");
const nominal_1 = require("../../messages/nominal");
const Encoder_1 = require("../binary/Encoder");
const util_1 = require("../../../../json-pack/util");
const EMPTY = new Uint8Array([]);
class Encoder {
    constructor() {
        this.binary = new Encoder_1.Encoder();
        this.msgpack = util_1.encoderFull;
    }
    encodeData(data) {
        if (data === undefined)
            return EMPTY;
        return this.msgpack.encode(data);
    }
    convertMessage(message) {
        if (message instanceof nominal_1.ResponseDataMessage)
            return new binary_1.BinaryResponseDataMessage(message.id, this.encodeData(message.data));
        else if (message instanceof nominal_1.ResponseCompleteMessage)
            return new binary_1.BinaryResponseCompleteMessage(message.id, this.encodeData(message.data));
        else if (message instanceof nominal_1.ResponseErrorMessage)
            return new binary_1.BinaryResponseErrorMessage(message.id, this.encodeData(message.data));
        else if (message instanceof nominal_1.ResponseUnsubscribeMessage)
            return new binary_1.BinaryResponseUnsubscribeMessage(message.id);
        else if (message instanceof nominal_1.RequestDataMessage)
            return new binary_1.BinaryRequestDataMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof nominal_1.RequestCompleteMessage)
            return new binary_1.BinaryRequestCompleteMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof nominal_1.RequestErrorMessage)
            return new binary_1.BinaryRequestErrorMessage(message.id, message.method, this.encodeData(message.data));
        else if (message instanceof nominal_1.RequestUnsubscribeMessage)
            return new binary_1.BinaryRequestUnsubscribeMessage(message.id);
        return new binary_1.BinaryNotificationMessage(message.method, this.encodeData(message.data));
    }
    encode(messages) {
        const binaryMessages = [];
        const length = messages.length;
        for (let i = 0; i < length; i++)
            binaryMessages.push(this.convertMessage(messages[i]));
        return this.binary.encode(binaryMessages);
    }
}
exports.Encoder = Encoder;
