"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.decodeMsg = void 0;
const nominal_1 = require("../../messages/nominal");
function decodeMsg(message) {
    if (!(message instanceof Array)) {
        throw new Error('Invalid message');
    }
    const length = message.length;
    if (length < 1 || length > 4) {
        throw new Error('Invalid message');
    }
    const first = message[0];
    if (typeof first === 'number') {
        switch (first) {
            case 0: {
                const [, id, data] = message;
                return new nominal_1.ResponseCompleteMessage(id, data);
            }
            case -1: {
                const [, id, data] = message;
                return new nominal_1.ResponseErrorMessage(id, data);
            }
            case -2: {
                const [, id, data] = message;
                return new nominal_1.ResponseDataMessage(id, data);
            }
            case -3: {
                const [, id] = message;
                return new nominal_1.ResponseUnsubscribeMessage(id);
            }
            default: {
                const [, second] = message;
                switch (second) {
                    case 0: {
                        const [, , name, data] = message;
                        return new nominal_1.RequestDataMessage(first, name, data);
                    }
                    case 1: {
                        const [, , method, data] = message;
                        return new nominal_1.RequestErrorMessage(first, method, data);
                    }
                    case 2: {
                        return new nominal_1.RequestUnsubscribeMessage(first);
                    }
                    default: {
                        const [, name, data] = message;
                        return new nominal_1.RequestCompleteMessage(first, name, data);
                    }
                }
            }
        }
    }
    const [method, data] = message;
    return new nominal_1.NotificationMessage(method, data);
}
exports.decodeMsg = decodeMsg;
function decode(messages) {
    if (messages[0] instanceof Array) {
        const length = messages.length;
        const out = [];
        for (let i = 0; i < length; i++)
            out.push(decodeMsg(messages[i]));
        return out;
    }
    else
        return decodeMsg(messages);
}
exports.decode = decode;
