import { ReactiveRpcRequestMessage, ReactiveRpcResponseMessage, NotificationMessage, RequestCompleteMessage, RequestDataMessage, RequestErrorMessage, RequestUnsubscribeMessage, ResponseCompleteMessage, ResponseDataMessage, ResponseErrorMessage, ResponseUnsubscribeMessage } from '../messages/nominal';
import { RpcApi } from './types';
import { RpcServerError } from './constants';
import { ErrorFormatter } from './error';
import { Call, RpcApiCaller } from './RpcApiCaller';
export { RpcServerError };
declare type Send = (messages: (ReactiveRpcResponseMessage<unknown> | NotificationMessage<unknown>)[]) => void;
export interface RpcServerParams<Ctx = unknown> {
    caller: RpcApiCaller<any, Ctx>;
    error?: ErrorFormatter<unknown>;
    send: Send;
    onNotification: (name: string, data: unknown | undefined, ctx: Ctx) => void;
    bufferSize?: number;
    bufferTime?: number;
}
export interface RpcServerFromApiParams<Ctx = unknown> extends Omit<RpcServerParams<Ctx>, 'onCall'> {
    api: RpcApi<Ctx, unknown>;
}
export declare class RpcServer<Ctx = unknown> {
    protected readonly caller: RpcApiCaller<any, Ctx>;
    readonly error: ErrorFormatter<unknown>;
    private readonly activeStreamCalls;
    protected send: (message: ReactiveRpcResponseMessage<unknown> | NotificationMessage<unknown>) => void;
    onSend: Send;
    onNotification: RpcServerParams<Ctx>['onNotification'];
    constructor({ caller, error, send, onNotification: notify, bufferSize, bufferTime }: RpcServerParams<Ctx>);
    protected notifMessage(method: string, data: unknown): NotificationMessage<unknown>;
    protected resCompleteMessage(id: number, data: unknown): ResponseCompleteMessage<unknown>;
    protected resDataMessage(id: number, data: unknown): ResponseDataMessage<unknown>;
    protected resErrorMessage(id: number, data: unknown): ResponseErrorMessage<unknown>;
    protected reqUnsubscribeMessage(id: number): RequestUnsubscribeMessage;
    protected createStaticCall(name: string, request: unknown, ctx: Ctx): Promise<unknown>;
    protected execStaticCall(id: number, name: string, request: unknown, ctx: Ctx): void;
    protected createCall(name: string, ctx: Ctx): Call<unknown, unknown>;
    protected onStreamError: (id: number, error: unknown) => void;
    onMessage(message: ReactiveRpcRequestMessage<unknown>, ctx: Ctx): void;
    onMessages(messages: ReactiveRpcRequestMessage<unknown>[], ctx: Ctx): void;
    stop(reason?: RpcServerError): void;
    disconnect(): void;
    private sendError;
    private createStreamCall;
    onRequestDataMessage(message: RequestDataMessage<unknown>, ctx: Ctx): void;
    onRequestCompleteMessage(message: RequestCompleteMessage<unknown>, ctx: Ctx): void;
    onRequestErrorMessage(message: RequestErrorMessage, ctx: Ctx): void;
    onUnsubscribeMessage(message: ResponseUnsubscribeMessage): void;
    onNotificationMessage(message: NotificationMessage<unknown>, ctx: Ctx): void;
}
