"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapMethod = exports.RpcMethodStreamingWrap = exports.RpcMethodStaticWrap = void 0;
const json_brand_1 = require("../../../json-brand");
const util_1 = require("../../../json-pack/util");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
class RpcMethodStaticWrap {
    constructor(method, types) {
        this.isStreaming = false;
        this.req = method.req || '';
        this.res = method.res || '';
        this.validate = method.validate ? method.validate.bind(method) : undefined;
        this.onPreCall = method.onPreCall ? method.onPreCall.bind(method) : undefined;
        this.pretty = !!method.pretty;
        const call = method.call ? method.call.bind(method) : undefined;
        const callJson = method.callJson ? method.callJson.bind(method) : undefined;
        const callMsgPack = method.callMsgPack ? method.callMsgPack.bind(method) : undefined;
        if (call)
            this.call = call;
        else if (callJson) {
            this.call = async (ctx, request) => json_brand_1.JSON.parse(await callJson(ctx, request));
        }
        else if (callMsgPack) {
            this.call = async (ctx, request) => (0, util_1.decode)(await callMsgPack(ctx, request));
        }
        else
            throw new Error('No call method defined.');
        if (callJson) {
            this.callJson = callJson;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getJsonSerializer(this.res) : json_brand_1.JSON.stringify;
            this.callJson = async (ctx, request) => serializer(await this.call(ctx, request));
        }
        if (callMsgPack) {
            this.callMsgPack = callMsgPack;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getMsgPackEncoder(this.res) : util_1.encodeFull;
            this.callMsgPack = async (ctx, request) => serializer(await this.call(ctx, request));
        }
        this.call$ = (ctx, request$) => (0, rxjs_1.from)((async () => {
            const request = await (0, rxjs_1.firstValueFrom)(request$);
            return await this.call(ctx, request);
        })());
        this.callJson$ = (ctx, request$) => (0, rxjs_1.from)((async () => {
            const request = await (0, rxjs_1.firstValueFrom)(request$);
            return await this.callJson(ctx, request);
        })());
        this.callMsgPack$ = (ctx, request$) => (0, rxjs_1.from)((async () => {
            const request = await (0, rxjs_1.firstValueFrom)(request$);
            return await this.callMsgPack(ctx, request);
        })());
    }
}
exports.RpcMethodStaticWrap = RpcMethodStaticWrap;
class RpcMethodStreamingWrap {
    constructor(method, types) {
        this.isStreaming = true;
        this.req = method.req || '';
        this.res = method.res || '';
        this.validate = method.validate ? method.validate.bind(method) : undefined;
        this.onPreCall = method.onPreCall ? method.onPreCall.bind(method) : undefined;
        this.preCallBufferSize = method.preCallBufferSize;
        this.pretty = !!method.pretty;
        const call$ = method.call$ ? method.call$.bind(method) : undefined;
        const callJson$ = method.callJson$ ? method.callJson$.bind(method) : undefined;
        const callMsgPack$ = method.callMsgPack$ ? method.callMsgPack$.bind(method) : undefined;
        if (call$)
            this.call$ = call$;
        else if (callJson$) {
            this.call$ = (ctx, request) => callJson$(ctx, request).pipe((0, operators_1.map)((json) => json_brand_1.JSON.parse(json)));
        }
        else if (callMsgPack$) {
            this.call$ = (ctx, request) => callMsgPack$(ctx, request).pipe((0, operators_1.map)((blob) => (0, util_1.decode)(blob)));
        }
        else
            throw new Error('No call method defined.');
        if (callJson$) {
            this.callJson$ = callJson$;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getJsonSerializer(this.res) : json_brand_1.JSON.stringify;
            this.callJson$ = (ctx, request) => this.call$(ctx, request).pipe((0, operators_1.map)((res) => serializer(res)));
        }
        if (callMsgPack$) {
            this.callMsgPack$ = callMsgPack$;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getMsgPackEncoder(this.res) : util_1.encodeFull;
            this.callMsgPack$ = (ctx, request) => this.call$(ctx, request).pipe((0, operators_1.map)((res) => serializer(res)));
        }
        this.call = (ctx, request) => (0, rxjs_1.firstValueFrom)(this.call$(ctx, (0, rxjs_1.of)(request)));
        this.callJson = (ctx, request) => (0, rxjs_1.firstValueFrom)(this.callJson$(ctx, (0, rxjs_1.of)(request)));
        this.callMsgPack = (ctx, request) => (0, rxjs_1.firstValueFrom)(this.callMsgPack$(ctx, (0, rxjs_1.of)(request)));
    }
}
exports.RpcMethodStreamingWrap = RpcMethodStreamingWrap;
const wrapMethod = (method, types) => {
    return method.isStreaming
        ? new RpcMethodStreamingWrap(method, types)
        : new RpcMethodStaticWrap(method, types);
};
exports.wrapMethod = wrapMethod;
