"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createConnectionContext = void 0;
const createConnectionContext = (req, res) => {
    const ip = req.getHeader('x-forwarded-for') ||
        req.getHeader('x-real-ip') ||
        Buffer.from(res.getRemoteAddressAsText()).toString('utf8');
    let token = req.getHeader('authorization') || '';
    if (!token) {
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        token = params.get('access_token') || '';
        if (!token)
            token = params.get('token') || '';
    }
    if (!token) {
        const secWebSocketProtocol = String(req.getHeader('sec-websocket-protocol')) || '';
        if (secWebSocketProtocol) {
            const protocols = secWebSocketProtocol.split(',');
            const length = protocols.length;
            for (let i = 0; i < length; i++) {
                let protocol = protocols[i].trim();
                if (protocol.indexOf('X-Authorization=') === 0) {
                    protocol = protocol.substr('X-Authorization='.length);
                    if (protocol) {
                        token = Buffer.from(protocol).toString('base64');
                        break;
                    }
                }
            }
        }
    }
    return { ip, token };
};
exports.createConnectionContext = createConnectionContext;
