"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpRpcMsgPackGetApi = exports.enableHttpRpcMsgPackPostApi = void 0;
const rpc_1 = require("../../../../common/rpc");
const util_1 = require("../../../../../json-pack/util");
const context_1 = require("../../context");
const util_2 = require("../../util");
const constants_1 = require("./constants");
const bufferToUint8Array_1 = require("../../../../../util/bufferToUint8Array");
const util_3 = require("../util");
const DEFAULT_ROUTE = '/rpc/msgpack/*';
function sendError(res, error) {
    if (res.aborted)
        return;
    res.cork(() => {
        const body = (0, util_1.encode)((0, rpc_1.formatError)(error));
        res.writeStatus(constants_1.STATUS_400).end(body);
    });
}
function processRequest(res, ctx, name, json, caller) {
    try {
        caller
            .call(name, json, ctx)
            .then((result) => {
            if (res.aborted)
                return;
            res.cork(() => {
                res.end((0, util_1.encode)(result));
            });
        })
            .catch((error) => {
            sendError(res, error);
        });
    }
    catch {
        const error = new Error('Could not execute request');
        sendError(res, error);
    }
}
const enableHttpRpcMsgPackPostApi = (params) => {
    const { uws, route = DEFAULT_ROUTE, createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        (0, util_2.readBody)(res, (buffer) => {
            try {
                const length = (ctx.payloadSize = buffer.length);
                const json = length ? (0, util_1.decode)((0, bufferToUint8Array_1.bufferToUint8Array)(buffer)) : null;
                processRequest(res, ctx, name, json, caller);
            }
            catch {
                const error = new Error('Could not parse payload');
                sendError(res, error);
            }
        });
    });
};
exports.enableHttpRpcMsgPackPostApi = enableHttpRpcMsgPackPostApi;
const enableHttpRpcMsgPackGetApi = (params) => {
    const { uws, route = DEFAULT_ROUTE, createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        try {
            const json = (0, util_3.parsePayload)(ctx, body);
            processRequest(res, ctx, name, json, caller);
        }
        catch {
            const error = new Error('Could not parse payload');
            sendError(res, error);
        }
    });
};
exports.enableHttpRpcMsgPackGetApi = enableHttpRpcMsgPackGetApi;
