"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableSseGetRpcApi = exports.enableSsePostRpcApi = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const rpc_1 = require("../../../common/rpc");
const context_1 = require("../context");
const util_1 = require("../util");
const util_2 = require("./util");
const enableSsePostRpcApi = (params) => {
    const { uws, route = '/sse/*', createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const origin = req.getHeader('origin');
        const ctx = createContext(req, res);
        const aborted$ = new rxjs_1.Subject();
        res.onAborted(() => {
            res.aborted = true;
            aborted$.next(true);
        });
        (0, util_1.readBody)(res, (buffer) => {
            processSseRequest(res, ctx, name, buffer, aborted$, origin, caller);
        });
    });
};
exports.enableSsePostRpcApi = enableSsePostRpcApi;
const enableSseGetRpcApi = (params) => {
    const { uws, route = '/sse/*', createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const origin = req.getHeader('origin');
        const ctx = createContext(req, res);
        const aborted$ = new rxjs_1.Subject();
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        res.onAborted(() => {
            res.aborted = true;
            aborted$.next(true);
        });
        processSseRequest(res, ctx, name, body, aborted$, origin, caller);
    });
};
exports.enableSseGetRpcApi = enableSseGetRpcApi;
const sendSseError = (res, error) => {
    if (res.aborted)
        return;
    res.aborted = true;
    const errorFormatted = (0, rpc_1.formatError)(error);
    res.end('event: err\ndata: ' + JSON.stringify(errorFormatted) + '\n\n');
};
function processSseRequest(res, ctx, name, body, aborted$, origin, caller) {
    let closed = false;
    try {
        const json = (0, util_2.parsePayload)(ctx, body);
        res.cork(() => {
            (0, util_2.writeSseAndNdjsonHeaders)(res, origin);
        });
        caller
            .call$(name, (0, rxjs_1.of)(json), ctx)
            .pipe((0, operators_1.takeUntil)(aborted$))
            .subscribe({
            next: (data) => {
                if (closed)
                    return;
                if (res.aborted)
                    return;
                res.write('data: ' + JSON.stringify(data) + '\n\n');
            },
            error: (error) => {
                if (closed)
                    return;
                closed = true;
                sendSseError(res, error);
            },
            complete: () => {
                if (closed)
                    return;
                closed = true;
                if (!res.aborted)
                    res.end();
            },
        });
    }
    catch {
        if (closed)
            return;
        closed = true;
        const error = new Error('Could not parse payload');
        sendSseError(res, error);
    }
}
