"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsCompactReactiveRpcApi = void 0;
const common_1 = require("../../../common");
const Decoder_1 = require("../../../common/codec/compact-json/Decoder");
const Decoder_2 = require("../../../common/codec/compact-msgpack/Decoder");
const compact_msgpack_binary_1 = require("../../../common/codec/compact-msgpack-binary");
const compact_json_string_1 = require("../../../common/codec/compact-json_string");
const context_1 = require("../context");
const json_1 = require("../../../common/messages/json");
const json_brand_1 = require("../../../../json-brand");
const util_1 = require("../../../../json-pack/util");
const enableWsCompactReactiveRpcApi = (params) => {
    const decoderJson = new Decoder_1.Decoder();
    const decoderMsgPack = new Decoder_2.Decoder();
    const encoderCompactJsonString = new compact_json_string_1.Encoder();
    const encoderCompactMsgPackBinary = new compact_msgpack_binary_1.Encoder();
    const invalidPayloadJson = encoderCompactJsonString.encode([
        new json_1.JsonNotificationMessage('.err', json_brand_1.JSON.stringify('CODING')),
    ]);
    const invalidPayloadMsgPack = encoderCompactMsgPackBinary.encode([
        new common_1.BinaryNotificationMessage('.err', (0, util_1.encode)('CODING')),
    ]);
    const { route = '/rpc/compact', uws, createRpcServerJson, createRpcServerMsgPack, onNotification, createContext = context_1.createConnectionContext, compression, idleTimeout = 0, maxBackpressure = 3145728, maxPayloadLength = 1048576, } = params;
    uws.ws(route, {
        idleTimeout,
        compression,
        maxBackpressure,
        maxPayloadLength,
        upgrade: (res, req, context) => {
            const secWebSocketKey = req.getHeader('sec-websocket-key');
            const secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            const secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            const ctx = createContext(req, res);
            const isBinary = secWebSocketProtocol.indexOf('MsgPack') > -1;
            res.upgrade({ ctx, isBinary }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: (ws) => {
            const { isBinary } = ws;
            let rpc;
            if (isBinary) {
                rpc = createRpcServerMsgPack({
                    send: (messages) => {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        const encoded = encoderCompactMsgPackBinary.encode(messages);
                        ws.send(encoded, true);
                    },
                });
            }
            else {
                rpc = createRpcServerJson({
                    send: (messages) => {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        const encoded = encoderCompactJsonString.encode(messages);
                        ws.send(encoded, false);
                    },
                });
            }
            if (onNotification) {
                rpc.onNotification = (name, data, ctx) => {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: (ws, buf, isBinary) => {
            const { ctx, rpc } = ws;
            let messages;
            try {
                messages = isBinary
                    ? decoderMsgPack.decode(new Uint8Array(buf))
                    : decoderJson.decode(Buffer.from(buf).toString('utf8'));
            }
            catch (error) {
                if (ws.getBufferedAmount() > maxBackpressure)
                    return;
                ws.send(isBinary ? invalidPayloadMsgPack : invalidPayloadJson, isBinary);
                return;
            }
            if (messages instanceof Array)
                rpc.onMessages(messages, ctx);
            else
                rpc.onMessage(messages, ctx);
        },
        close: (ws, code, message) => {
            ws.rpc.stop();
        },
    });
};
exports.enableWsCompactReactiveRpcApi = enableWsCompactReactiveRpcApi;
