"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeFullMessages = exports.decodeFullMessage = void 0;
const CompleteMessage_1 = require("../messages/CompleteMessage");
const DataMessage_1 = require("../messages/DataMessage");
const ErrorMessage_1 = require("../messages/ErrorMessage");
const NotificationMessage_1 = require("../messages/NotificationMessage");
const SubscribeMessage_1 = require("../messages/SubscribeMessage");
const UnsubscribeMessage_1 = require("../messages/UnsubscribeMessage");
const header_1 = require("./header");
const readMethod = (arr, offset) => {
    const size = arr[offset++];
    let str = '';
    for (let i = 0; i < size; i++)
        str += String.fromCharCode(arr[offset++]);
    return [str, offset];
};
const decodeFullMessage = (arr, offset) => {
    const byte1 = arr[offset++];
    const code = byte1 >>> 5;
    switch (code) {
        case 0: {
            const [length, off1] = (0, header_1.readHeader)(byte1, arr, offset);
            const [method, off2] = readMethod(arr, off1);
            const data = arr.subarray(off2, off2 + length);
            return [new NotificationMessage_1.NotificationMessage(method, data), off2 + length];
        }
        case 1: {
            const [length, off1] = (0, header_1.readHeader)(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            const [method, off2] = readMethod(arr, off1 + 2);
            const data = arr.subarray(off2, off2 + length);
            return [new SubscribeMessage_1.SubscribeMessage(id, method, data), off2 + length];
        }
        case 2:
        case 3:
        case 5: {
            const [length, off1] = (0, header_1.readHeader)(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            offset = off1 + 2;
            const data = arr.subarray(offset, offset + length);
            const message = code === 2
                ? new DataMessage_1.DataMessage(id, data)
                : code === 3
                    ? new CompleteMessage_1.CompleteMessage(id, data)
                    : new ErrorMessage_1.ErrorMessage(id, data);
            return [message, offset + length];
        }
        case 4: {
            const o1 = arr[offset++];
            const o2 = arr[offset++];
            const id = (o1 << 8) | o2;
            return [new UnsubscribeMessage_1.UnsubscribeMessage(id), offset];
        }
    }
    throw new Error('UNKNOWN_MESSAGE');
};
exports.decodeFullMessage = decodeFullMessage;
const decodeFullMessages = (arr, offset) => {
    const messages = [];
    const length = arr.length;
    while (offset < length) {
        const [message, off] = (0, exports.decodeFullMessage)(arr, offset);
        offset = off;
        messages.push(message);
    }
    return messages;
};
exports.decodeFullMessages = decodeFullMessages;
