"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Branch = exports.BasicBlock = void 0;
const tslib_1 = require("tslib");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
class BasicBlock {
    constructor(version, model) {
        this.model = model;
        this.version$ = new rxjs_1.BehaviorSubject(version);
    }
    fork() {
        return new BasicBlock(this.version$.getValue(), this.model.fork());
    }
    data$() {
        return this.version$.pipe((0, operators_1.switchMap)(() => (0, rxjs_1.of)(this.model.getData())));
    }
    getData() {
        return this.model.getData();
    }
    apply(patch) {
        this.model.apply(patch);
        this.version$.next(this.version$.getValue() + 1);
    }
}
exports.BasicBlock = BasicBlock;
class Branch {
    constructor(base) {
        this.merging = 0;
        this.base$ = new rxjs_1.BehaviorSubject(base);
        this.patches = [];
        this.head$ = new rxjs_1.BehaviorSubject(base.fork());
    }
    merge(opts) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                const base = this.base$.getValue();
                const baseVersion = base.version$.getValue();
                const batch = [...this.patches];
                this.merging = batch.length;
                const res = yield opts.merge(baseVersion, batch);
            }
            catch (error) {
                this.merging = 0;
            }
        });
    }
    commit(patch) {
        this.patches.push(patch);
        this.head$.getValue().apply(patch);
    }
    rebase(patches) {
        const base = this.base$.getValue();
        for (const patch of patches)
            base.apply(patch);
        const head = base.fork();
        for (const patch of this.patches)
            head.apply(patch);
        this.head$.next(head);
    }
    reset(base) {
        this.base$.next(base);
        const patches = this.patches;
        this.patches = [];
        this.head$.next(base.fork());
        return patches;
    }
}
exports.Branch = Branch;
