"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Patch = void 0;
const DeleteOperation_1 = require("./operations/DeleteOperation");
const InsertArrayElementsOperation_1 = require("./operations/InsertArrayElementsOperation");
const InsertBinaryDataOperation_1 = require("./operations/InsertBinaryDataOperation");
const InsertStringSubstringOperation_1 = require("./operations/InsertStringSubstringOperation");
const MakeArrayOperation_1 = require("./operations/MakeArrayOperation");
const MakeBinaryOperation_1 = require("./operations/MakeBinaryOperation");
const MakeConstantOperation_1 = require("./operations/MakeConstantOperation");
const MakeNumberOperation_1 = require("./operations/MakeNumberOperation");
const MakeObjectOperation_1 = require("./operations/MakeObjectOperation");
const MakeStringOperation_1 = require("./operations/MakeStringOperation");
const MakeValueOperation_1 = require("./operations/MakeValueOperation");
const NoopOperation_1 = require("./operations/NoopOperation");
const SetNumberOperation_1 = require("./operations/SetNumberOperation");
const SetObjectKeysOperation_1 = require("./operations/SetObjectKeysOperation");
const SetRootOperation_1 = require("./operations/SetRootOperation");
const SetValueOperation_1 = require("./operations/SetValueOperation");
const clock_1 = require("./clock");
class Patch {
    constructor() {
        this.ops = [];
    }
    getId() {
        const op = this.ops[0];
        if (!op)
            return undefined;
        return op.id;
    }
    span() {
        let span = 0;
        for (const op of this.ops)
            span += op.span();
        return span;
    }
    nextTime() {
        if (!this.ops.length)
            return 0;
        const lastOp = this.ops[this.ops.length - 1];
        return lastOp.id.time + lastOp.span();
    }
    rewriteTime(ts) {
        const patch = new Patch();
        const ops = this.ops;
        const length = ops.length;
        for (let i = 0; i < length; i++) {
            const op = ops[i];
            if (op instanceof DeleteOperation_1.DeleteOperation)
                patch.ops.push(new DeleteOperation_1.DeleteOperation(ts(op.id), ts(op.obj), op.after));
            else if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation)
                patch.ops.push(new InsertArrayElementsOperation_1.InsertArrayElementsOperation(ts(op.id), ts(op.arr), ts(op.after), op.elements.map(ts)));
            else if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation)
                patch.ops.push(new InsertStringSubstringOperation_1.InsertStringSubstringOperation(ts(op.id), ts(op.obj), ts(op.after), op.substring));
            else if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation)
                patch.ops.push(new InsertBinaryDataOperation_1.InsertBinaryDataOperation(ts(op.id), ts(op.obj), ts(op.after), op.data));
            else if (op instanceof MakeArrayOperation_1.MakeArrayOperation)
                patch.ops.push(new MakeArrayOperation_1.MakeArrayOperation(ts(op.id)));
            else if (op instanceof MakeConstantOperation_1.MakeConstantOperation)
                patch.ops.push(new MakeConstantOperation_1.MakeConstantOperation(ts(op.id), op.value));
            else if (op instanceof MakeValueOperation_1.MakeValueOperation)
                patch.ops.push(new MakeValueOperation_1.MakeValueOperation(ts(op.id), op.value));
            else if (op instanceof MakeNumberOperation_1.MakeNumberOperation)
                patch.ops.push(new MakeNumberOperation_1.MakeNumberOperation(ts(op.id)));
            else if (op instanceof MakeObjectOperation_1.MakeObjectOperation)
                patch.ops.push(new MakeObjectOperation_1.MakeObjectOperation(ts(op.id)));
            else if (op instanceof MakeStringOperation_1.MakeStringOperation)
                patch.ops.push(new MakeStringOperation_1.MakeStringOperation(ts(op.id)));
            else if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation)
                patch.ops.push(new MakeBinaryOperation_1.MakeBinaryOperation(ts(op.id)));
            else if (op instanceof SetNumberOperation_1.SetNumberOperation)
                patch.ops.push(new SetNumberOperation_1.SetNumberOperation(ts(op.id), ts(op.num), op.value));
            else if (op instanceof SetValueOperation_1.SetValueOperation)
                patch.ops.push(new SetValueOperation_1.SetValueOperation(ts(op.id), ts(op.obj), op.value));
            else if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation)
                patch.ops.push(new SetObjectKeysOperation_1.SetObjectKeysOperation(ts(op.id), ts(op.object), op.tuples.map(([key, value]) => [key, ts(value)])));
            else if (op instanceof SetRootOperation_1.SetRootOperation)
                patch.ops.push(new SetRootOperation_1.SetRootOperation(ts(op.id), ts(op.value)));
            else if (op instanceof NoopOperation_1.NoopOperation)
                patch.ops.push(new NoopOperation_1.NoopOperation(ts(op.id), op.length));
        }
        return patch;
    }
    rebase(serverTime, transformHorizon) {
        const id = this.getId();
        if (!id)
            throw new Error('EMPTY_PATCH');
        const patchStartTime = id.time;
        if (patchStartTime === serverTime)
            return this;
        const delta = serverTime - patchStartTime;
        return this.rewriteTime((id) => {
            const sessionId = id.getSessionId();
            const isServerTimestamp = sessionId === 1;
            if (!isServerTimestamp)
                return id;
            const time = id.time;
            if (time < transformHorizon)
                return id;
            return new clock_1.ServerTimestamp(time + delta);
        });
    }
    clone() {
        return this.rewriteTime((id) => id);
    }
}
exports.Patch = Patch;
