"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const CrdtEncoder_1 = require("../../util/binary/CrdtEncoder");
const DeleteOperation_1 = require("../../operations/DeleteOperation");
const InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
const InsertBinaryDataOperation_1 = require("../../operations/InsertBinaryDataOperation");
const InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
const MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
const MakeBinaryOperation_1 = require("../../operations/MakeBinaryOperation");
const MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
const MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
const MakeStringOperation_1 = require("../../operations/MakeStringOperation");
const MakeValueOperation_1 = require("../../operations/MakeValueOperation");
const NoopOperation_1 = require("../../operations/NoopOperation");
const SetNumberOperation_1 = require("../../operations/SetNumberOperation");
const SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
const SetRootOperation_1 = require("../../operations/SetRootOperation");
const SetValueOperation_1 = require("../../operations/SetValueOperation");
class Encoder extends CrdtEncoder_1.CrdtEncoder {
    encode(patch) {
        this.reset();
        const id = (this.patchId = patch.getId());
        const isServerClock = id.getSessionId() === 1;
        if (isServerClock) {
            this.b1vuint56(true, id.time);
        }
        else {
            this.b1vuint56(false, id.getSessionId());
            this.vuint57(id.time);
        }
        this.encodeOperations(patch);
        return this.flush();
    }
    encodeOperations(patch) {
        const ops = patch.ops;
        for (let i = 0; i < ops.length; i++) {
            const op = ops[i];
            this.encodeOperation(op);
        }
    }
    encodeId(id) {
        const sessionId = id.getSessionId();
        const time = id.time;
        if (sessionId === 1) {
            this.b1vuint56(true, id.time);
        }
        else {
            const patchId = this.patchId;
            if (sessionId === patchId.getSessionId() && time >= patchId.time) {
                this.b1vuint56(false, 1);
                this.vuint57(time - patchId.time);
            }
            else {
                this.b1vuint56(false, sessionId);
                this.vuint57(time);
            }
        }
    }
    encodeOperation(op) {
        if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
            this.u8(0);
            return;
        }
        else if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
            this.u8(1);
            return;
        }
        else if (op instanceof MakeStringOperation_1.MakeStringOperation) {
            this.u8(2);
            return;
        }
        else if (op instanceof MakeValueOperation_1.MakeValueOperation) {
            this.u8(14);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
            this.u8(3);
            return;
        }
        else if (op instanceof SetRootOperation_1.SetRootOperation) {
            this.u8(4);
            this.encodeId(op.value);
            return;
        }
        else if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
            this.u8(5);
            this.encodeId(op.object);
            this.vuint57(op.tuples.length);
            for (const [key, value] of op.tuples) {
                this.encodeId(value);
                this.encodeString(key);
            }
            return;
        }
        else if (op instanceof SetValueOperation_1.SetValueOperation) {
            this.u8(15);
            this.encodeId(op.obj);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof SetNumberOperation_1.SetNumberOperation) {
            this.u8(6);
            this.encodeId(op.num);
            this.encodeNumber(op.value);
            return;
        }
        else if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
            this.u8(7);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.encodeString(op.substring);
            return;
        }
        else if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
            const { arr, after, elements } = op;
            const length = elements.length;
            this.u8(8);
            this.encodeId(arr);
            this.encodeId(after);
            this.vuint57(length);
            for (let i = 0; i < length; i++)
                this.encodeId(elements[i]);
            return;
        }
        else if (op instanceof DeleteOperation_1.DeleteOperation) {
            const { obj, after } = op;
            const length = after.span;
            if (length > 1) {
                this.u8(10);
                this.encodeId(obj);
                this.encodeId(after);
                this.vuint57(length);
                return;
            }
            this.u8(9);
            this.encodeId(obj);
            this.encodeId(after);
            return;
        }
        else if (op instanceof NoopOperation_1.NoopOperation) {
            const { length } = op;
            if (length > 1) {
                this.u8(12);
                this.vuint57(length);
                return;
            }
            this.u8(11);
            return;
        }
        else if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation) {
            this.u8(16);
            return;
        }
        else if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation) {
            const buf = op.data;
            const length = buf.length;
            this.u8(17);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.vuint57(length);
            this.buf(buf, length);
            return;
        }
        throw new Error('UNKNOWN_OP');
    }
}
exports.Encoder = Encoder;
