"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
const Batch_1 = require("../../../json-crdt-patch/Batch");
const Finder_1 = require("./Finder");
const constants_1 = require("../../constants");
const PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
class ModelApi {
    constructor(model) {
        this.model = model;
        this.batch = new Batch_1.Batch([]);
        this.builder = new PatchBuilder_1.PatchBuilder(model.clock);
    }
    getApi() {
        return this;
    }
    getNode() {
        const model = this.model;
        const id = model.root.toValue();
        return model.node(id) || constants_1.NULL;
    }
    find(path) {
        const finder = new Finder_1.Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    }
    val(path) {
        return this.find().val(path);
    }
    str(path) {
        return this.find().str(path);
    }
    bin(path) {
        return this.find().bin(path);
    }
    arr(path) {
        return this.find().arr(path);
    }
    obj(path) {
        return this.find().obj(path);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.batch.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    }
    flush() {
        const batch = this.batch;
        this.batch = new Batch_1.Batch([]);
        return batch;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    toView() {
        return this.getNode().toJson();
    }
}
exports.ModelApi = ModelApi;
