"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectType = void 0;
const constants_1 = require("../../../json-crdt-patch/constants");
const ObjectChunk_1 = require("./ObjectChunk");
class ObjectType {
    constructor(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
        this._toJson = {};
    }
    get(key) {
        const entry = this.latest.get(key);
        return entry ? entry.node.id : undefined;
    }
    getId(key) {
        const entry = this.latest.get(key);
        return entry ? entry.id : undefined;
    }
    getNode(key) {
        const entry = this.latest.get(key);
        return entry ? entry.node : undefined;
    }
    insert(op) {
        const { latest: last } = this;
        const clock = op.id.clock();
        for (const [key, value] of op.tuples) {
            const id = clock.tick(1);
            const lastEntry = last.get(key);
            if (lastEntry)
                if (lastEntry.id.compare(id) >= 0)
                    continue;
            this.put(key, id, value);
        }
    }
    put(key, id, value) {
        const node = this.doc.node(value);
        if (!node)
            return;
        this.putChunk(key, new ObjectChunk_1.ObjectChunk(id, node));
    }
    putChunk(key, chunk) {
        this.latest.set(key, chunk);
    }
    toJson() {
        const _toJson = this._toJson;
        const obj = {};
        let useCache = true;
        for (const [key, entry] of this.latest.entries()) {
            if (!entry.node.id.isEqual(constants_1.UNDEFINED_ID)) {
                const value = entry.node.toJson();
                if (_toJson[key] !== value)
                    useCache = false;
                obj[key] = value;
            }
            else {
                if (_toJson[key] !== undefined)
                    useCache = false;
            }
        }
        return useCache ? _toJson : (this._toJson = obj);
    }
    toString(tab = '') {
        let str = `${tab}ObjectType(${this.id.toDisplayString()})`;
        for (const [key, entry] of this.latest.entries()) {
            const node = entry.node;
            str += `\n${tab}  "${key}" :\n${node.toString(tab + '    ')}`;
        }
        return str;
    }
    clone(doc) {
        const obj = new ObjectType(doc, this.id);
        for (const [key, { id, node }] of this.latest.entries()) {
            const nodeClone = node.clone(doc);
            obj.putChunk(key, new ObjectChunk_1.ObjectChunk(id, nodeClone));
        }
        doc.nodes.index(obj);
        return obj;
    }
    *children() {
        for (const { node } of this.latest.values())
            yield node.id;
    }
}
exports.ObjectType = ObjectType;
