import type { TType, TAnyType } from '../json-type/types';
import { BooleanValidator, ObjectValidator, CustomValidator } from '../json-type-validator';
import { JsonEncoderFn, EncoderFn, PartialEncoderFn } from '../json-type-serializer';
import { JsonSchemaNode } from '../json-type-schema';
import { TsDeclaration } from '../json-type-typescript/types';
export declare type Types = {
    [ref: string]: TType;
};
export interface JsonTypeSystemOptions<T extends Types> {
    types: T;
    customValidators?: CustomValidator[];
}
export declare class JsonTypeSystem<T extends Types> {
    protected readonly options: JsonTypeSystemOptions<T>;
    constructor(options: JsonTypeSystemOptions<T>);
    readonly ref: (ref: keyof T) => TAnyType;
    hasType(ref: string): boolean;
    protected readonly fastValidatorCache: {
        [ref: string]: BooleanValidator;
    };
    protected readonly fastValidatorUsage: {
        [ref: string]: number;
    };
    getFastValidator(ref: string): BooleanValidator;
    protected readonly fullValidatorCache: {
        [ref: string]: ObjectValidator;
    };
    getFullValidator(ref: string): ObjectValidator;
    protected readonly jsonSerializerCache: {
        [ref: string]: JsonEncoderFn;
    };
    protected readonly jsonSerializerUsage: {
        [ref: string]: number;
    };
    getJsonSerializer(ref: string): JsonEncoderFn;
    protected readonly msgPackPartialEncoderCache: {
        [ref: string]: PartialEncoderFn;
    };
    protected readonly msgPackPartialEncoderUsage: {
        [ref: string]: number;
    };
    protected getMsgPackPartialEncoder(ref: string): PartialEncoderFn;
    protected readonly msgPackEncoderCache: {
        [ref: string]: EncoderFn;
    };
    getMsgPackEncoder(ref: string): EncoderFn;
    toJsonSchema(ref: string, withDefs?: boolean): JsonSchemaNode;
    toTypeScriptText(ref: string): string;
    toTsAst(ref?: string): TsDeclaration[];
}
