"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toTypeScriptText = void 0;
const wordWrap_1 = require("../util/wordWrap");
const util_1 = require("./util");
const TAB = '  ';
const formatComment = (node, __) => {
    if (node.title || node.description) {
        if (!node.description) {
            return `${__}/** ${node.title} */\n`;
        }
        let res = '';
        res += `${__}/**\n`;
        if (node.title)
            res += `${__} * # ${node.title}\n`;
        if (node.title && node.description)
            res += `${__} *\n`;
        if (node.description) {
            const lines = (0, wordWrap_1.wordWrap)(node.description);
            for (const line of lines) {
                res += `${__} * ${line.trim()}\n`;
            }
        }
        res += `${__} */\n`;
        return res;
    }
    return '';
};
const toTypeScriptText = (type, tab) => {
    if (type.const) {
        return JSON.stringify(type.const);
    }
    switch (type.__t) {
        case 'str': {
            return 'string';
        }
        case 'num': {
            return 'number';
        }
        case 'bool': {
            return 'boolean';
        }
        case 'nil': {
            return 'null';
        }
        case 'arr': {
            return `Array<${(0, exports.toTypeScriptText)(type.type, tab + TAB)}>`;
        }
        case 'enum': {
            return `[${type.values.map((t) => (0, exports.toTypeScriptText)(t, tab + TAB)).join(', ')}]`;
        }
        case 'obj': {
            const ____ = tab + TAB;
            if (type.fields.length === 0)
                return 'object';
            let res = '{\n';
            for (let i = 0; i < type.fields.length; i++) {
                const field = type.fields[i];
                const key = (0, util_1.normalizeKey)(field.key) + (field.isOptional ? '?' : '');
                if (i)
                    if (field.key || field.description)
                        res += `\n`;
                res += formatComment(field, ____);
                res += `${____}${key}: ${(0, exports.toTypeScriptText)(field.type, ____)};\n`;
            }
            res += tab + '}';
            return res;
        }
        case 'ref': {
            return type.ref;
        }
        case 'or': {
            return type.types.map((t) => (0, exports.toTypeScriptText)(t, tab + TAB)).join(' | ');
        }
        case 'bin': {
            return 'Uint8Array';
        }
        case 'any': {
            return 'any';
        }
        default: {
            return 'any';
        }
    }
};
exports.toTypeScriptText = toTypeScriptText;
