"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonTypeBuilder = void 0;
class JsonTypeBuilder {
    get str() {
        return this.String();
    }
    get num() {
        return this.Number();
    }
    get bool() {
        return this.Boolean();
    }
    get nil() {
        return this.Null();
    }
    get arr() {
        return this.Array(this.any);
    }
    get obj() {
        return this.Object({
            fields: [],
        });
    }
    get bin() {
        return this.Binary();
    }
    get any() {
        return this.Any();
    }
    Null(options = {}) {
        return Object.assign({ __t: 'nil' }, options);
    }
    Boolean(a, b) {
        if (typeof a === 'string')
            return this.Boolean(Object.assign({ id: a }, (b || {})));
        return Object.assign({ __t: 'bool' }, (a || {}));
    }
    Number(a, b) {
        if (typeof a === 'string')
            return this.Number(Object.assign({ id: a }, (b || {})));
        return Object.assign({ __t: 'num' }, (a || {}));
    }
    String(a, b) {
        if (typeof a === 'string')
            return this.String(Object.assign({ id: a }, (b || {})));
        return Object.assign({ __t: 'str' }, (a || {}));
    }
    Array(a, b, c) {
        if (typeof a === 'string')
            return this.Array(b, Object.assign({ id: a }, (c || {})));
        return Object.assign(Object.assign({ __t: 'arr' }, b), { type: a });
    }
    Object(a, b, c) {
        if (typeof a === 'string') {
            if (Array.isArray(b))
                return this.Object(Object.assign({ id: a, fields: b }, (c || {})));
            return this.Object(Object.assign({ id: a }, (b || {})));
        }
        else if (Array.isArray(a)) {
            return this.Object(Object.assign({ fields: a }, (b || {})));
        }
        return Object.assign({ __t: 'obj' }, (a || {}));
    }
    Field(key, type, options = {}) {
        return Object.assign({ key,
            type }, options);
    }
    Binary(options = {}) {
        return Object.assign({ __t: 'bin' }, options);
    }
    Any(options = {}) {
        return Object.assign({ __t: 'any' }, options);
    }
    Ref(ref) {
        return {
            __t: 'ref',
            ref,
        };
    }
    Or(...a) {
        return {
            __t: 'or',
            types: a[0] instanceof Array ? a[0] : a,
        };
    }
    Enum(a, b) {
        if (Array.isArray(a)) {
            if (!a.length)
                throw new Error('Enum must have at least one value.');
            return this.Enum(Object.assign(Object.assign({}, b), { values: a }));
        }
        return Object.assign({ __t: 'enum' }, a);
    }
}
exports.JsonTypeBuilder = JsonTypeBuilder;
