"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const header_1 = require("./header");
const binary_1 = require("../../messages/binary");
class Encoder {
    constructor() {
        this.uint8 = new Uint8Array(0);
        this.offset = 0;
    }
    encode(messages) {
        let size = 0;
        const length = messages.length;
        for (let i = 0; i < length; i++)
            size += messages[i].size();
        this.uint8 = new Uint8Array(size);
        this.offset = 0;
        for (let i = 0; i < length; i++) {
            const message = messages[i];
            if (message instanceof binary_1.BinaryNotificationMessage)
                this.notif(message);
            else if (message instanceof binary_1.BinaryRequestDataMessage)
                this.msgWithIdAndMethod(0, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestCompleteMessage)
                this.msgWithIdAndMethod(1, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestErrorMessage)
                this.msgWithIdAndMethod(2, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestUnsubscribeMessage)
                this.reqUnsub(message);
            else if (message instanceof binary_1.BinaryResponseDataMessage)
                this.resData(message);
            else if (message instanceof binary_1.BinaryResponseCompleteMessage)
                this.resComp(message);
            else if (message instanceof binary_1.BinaryResponseErrorMessage)
                this.resErr(message);
            else if (message instanceof binary_1.BinaryResponseUnsubscribeMessage)
                this.resUnsub(message);
        }
        return this.uint8;
    }
    notif(message) {
        const { method, data } = message;
        const length = data ? data.byteLength : 0;
        this.header(3, length);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    reqUnsub(message) {
        this.uint8[this.offset++] = 254;
        this.id(message.id);
    }
    resData(message) {
        this.msgWithId(4, message.id, message.data);
    }
    resComp(message) {
        this.msgWithId(5, message.id, message.data);
    }
    resErr(message) {
        this.msgWithId(6, message.id, message.data);
    }
    resUnsub(message) {
        this.uint8[this.offset++] = 255;
        this.id(message.id);
    }
    msgWithIdAndMethod(code, id, method, data) {
        const length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    msgWithId(code, id, data) {
        const length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    header(code, length) {
        this.offset = (0, header_1.writeHeader)(this.uint8, this.offset, code, length);
    }
    id(id) {
        this.uint8[this.offset++] = id >>> 8;
        this.uint8[this.offset++] = id & 0xff;
    }
    method(method) {
        const uint8 = this.uint8;
        const length = method.length;
        let offset = this.offset;
        uint8[offset++] = length;
        for (let i = 0; i < length; i++)
            uint8[offset++] = method.charCodeAt(i);
        this.offset = offset;
    }
}
exports.Encoder = Encoder;
