"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readHeader = exports.writeHeader = exports.getHeaderSize = void 0;
const getHeaderSize = (length) => {
    if (length <= 0b1111)
        return 1;
    if (length <= 2047)
        return 2;
    if (length <= 262143)
        return 3;
    return 4;
};
exports.getHeaderSize = getHeaderSize;
const writeHeader = (arr, offset, type, length) => {
    if (length <= 0b1111) {
        arr[offset++] = (type << 5) | length;
        return offset;
    }
    if (length <= 2047) {
        arr[offset++] = (type << 5) | 0b10000 | (0b1111 & length);
        arr[offset++] = length >>> 4;
        return offset;
    }
    if (length <= 262143) {
        arr[offset++] = (type << 5) | 0b10000 | (0b1111 & length);
        arr[offset++] = 0b10000000 | ((length >>> 4) & 0xff);
        arr[offset++] = length >>> 11;
        return offset;
    }
    if (length <= 33554431) {
        arr[offset++] = (type << 5) | 0b10000 | (0b1111 & length);
        arr[offset++] = 0b10000000 | ((length >>> 4) & 0xff);
        arr[offset++] = 0b10000000 | ((length >>> 11) & 0xff);
        arr[offset++] = length >>> 18;
        return offset;
    }
    throw new Error('LENGTH_TOO_BIG');
};
exports.writeHeader = writeHeader;
const readHeader = (byte1, arr, offset) => {
    let more = byte1 & 0b10000;
    let value = byte1 & 0b1111;
    if (!more)
        return [value, offset];
    let octet = arr[offset++];
    more = octet & 0b10000000;
    value = (octet << 4) | value;
    if (!more)
        return [value, offset];
    octet = arr[offset++];
    more = octet & 0b10000000;
    value = (octet << 11) | value;
    if (!more)
        return [value, offset];
    octet = arr[offset++];
    value = (octet << 18) | value;
    return [value, offset];
};
exports.readHeader = readHeader;
