"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpc2CodecJsonString = void 0;
const json_brand_1 = require("../../../../../json-brand");
const asString_1 = require("../../../../../util/asString");
const KEY_ID = '{"jsonrpc":"2.0","id":';
const KEY_RESULT = ',"result":';
const KEY_ERROR = ',"error":';
const KEY_CODE = '{"code":';
const KEY_MESSAGE = ',"message":';
const KEY_DATA = ',"data":';
const BRACKET1_CLOSE = '}';
const BRACKET1_CLOSE_TWICE = '}}';
const BRACKET2_OPEN = '[';
const BRACKET2_CLOSE = ']';
const ERROR_PARSE = '{"jsonrpc":"2.0","id":null,"error":{"code":-32700,"message":"Parse error"}}';
const ERROR_INVALID_REQUEST = ',"error":{"code":-32600,"message":"Invalid Request"}}';
const ERROR_METHOD_NOT_FOUND = ',"error":{"code":-32601,"message":"Method not found"}}';
const ERROR_INVALID_PARAMS = ',"error":{"code":-32602,"message":"Invalid params"}}';
const ERROR_INTERNAL = ',"error":{"code":-32603,"message":"Internal error"}}';
class JsonRpc2CodecJsonString {
    decode(data) {
        const messages = json_brand_1.JSON.parse(data);
        return Array.isArray(messages) ? messages : [messages];
    }
    encodeError(error, id) {
        const errorJson = KEY_CODE +
            (error.code || 0) +
            KEY_MESSAGE +
            (0, asString_1.asString)(String(error.message)) +
            (error.data !== undefined ? KEY_DATA + json_brand_1.JSON.stringify(error.data) : '');
        return (KEY_ID +
            json_brand_1.JSON.stringify(id) +
            KEY_ERROR +
            errorJson +
            BRACKET1_CLOSE_TWICE);
    }
    encodeParseError() {
        return ERROR_PARSE;
    }
    encodeInvalidRequestError(id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INVALID_REQUEST);
    }
    encodeMethodNotFoundError(id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_METHOD_NOT_FOUND);
    }
    encodeInvalidParamsError(id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INVALID_PARAMS);
    }
    encodeInternalError(id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INTERNAL);
    }
    encodeResponse(id, result) {
        return (KEY_ID +
            json_brand_1.JSON.stringify(id) +
            KEY_RESULT +
            json_brand_1.JSON.stringify(result) +
            BRACKET1_CLOSE);
    }
    encodeBatch(messages) {
        return (BRACKET2_OPEN + messages.join(',') + BRACKET2_CLOSE);
    }
}
exports.JsonRpc2CodecJsonString = JsonRpc2CodecJsonString;
