import { Observable, Observer } from 'rxjs';
import type { IRpcApiCaller, RpcMethod, RpcMethodRequest, RpcMethodResponse, RpcMethodWrapFromRpcMethod } from './types';
import { ErrorFormatter } from './error';
import type { JsonTypeSystem } from '../../../json-type-system/JsonTypeSystem';
import type { json_string } from '../../../json-brand';
import type { MsgPack } from '../../../json-pack';
export interface RpcApiCallerParams<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown, E = unknown> {
    api: Api;
    types?: JsonTypeSystem<any>;
    error?: ErrorFormatter<E>;
    preCallBufferSize?: number;
    maxActiveCalls?: number;
}
export interface Call<Request = unknown, Response = unknown> {
    req$: Observer<Request>;
    reqUnsubscribe$: Observable<null>;
    res$: Observable<Response>;
}
export declare class RpcApiCaller<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown, E = unknown> implements IRpcApiCaller<Api, Ctx> {
    readonly api: any;
    readonly error: ErrorFormatter<E>;
    protected readonly preCallBufferSize: number;
    protected readonly maxActiveCalls: number;
    protected _calls: number;
    constructor({ types, api, error, preCallBufferSize, maxActiveCalls }: RpcApiCallerParams<Api, Ctx, E>);
    get activeCalls(): number;
    exists<K extends keyof Api>(name: K): boolean;
    get<K extends keyof Api>(name: K): RpcMethodWrapFromRpcMethod<Api[K]>;
    call<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<RpcMethodResponse<Api[K]>>;
    callJson<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<json_string<RpcMethodResponse<Api[K]>>>;
    callMsgPack<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<MsgPack<RpcMethodResponse<Api[K]>>>;
    createCall<R, K extends keyof Api>(name: K, ctx: Ctx, callMethod: (name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx) => Promise<R>, callMethod$: (method: RpcMethodWrapFromRpcMethod<Api[K]>, ctx: Ctx, req$: Observable<RpcMethodRequest<Api[K]>>) => Observable<R>): Call<RpcMethodRequest<Api[K]>, R>;
    call$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<RpcMethodResponse<Api[K]>>;
    callJson$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<json_string<RpcMethodResponse<Api[K]>>>;
    callMsgPack$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<MsgPack<RpcMethodResponse<Api[K]>>>;
}
