import type { Observable } from 'rxjs';
import type { json_string } from '../../../json-brand';
import type { MsgPack } from '../../../json-pack';
import type { RpcMethodStaticWrap, RpcMethodStreamingWrap } from './methods';
export declare type RpcMethod<Context = unknown, Request = unknown, Response = unknown> = RpcMethodStatic<Context, Request, Response> | RpcMethodStreaming<Context, Request, Response>;
export declare type RpcMethodContext<T> = T extends RpcMethod<infer U, any, any> ? U : never;
export declare type RpcMethodRequest<T> = T extends RpcMethod<any, infer U, any> ? U : never;
export declare type RpcMethodResponse<T> = T extends RpcMethod<any, any, infer U> ? U : never;
export interface RpcMethodBase<Context = unknown, Request = unknown> {
    isStreaming: boolean;
    validate?: (request: Request) => void;
    onPreCall?: (ctx: Context, request: Request) => Promise<void>;
    pretty?: boolean;
    req?: string;
    res?: string;
}
export interface RpcMethodStatic<Context = unknown, Request = unknown, Response = unknown> extends RpcMethodBase<Context, Request> {
    isStreaming: false;
    call?: (ctx: Context, request: Request) => Promise<Response>;
    callJson?: (ctx: Context, request: Request) => Promise<json_string<Response>>;
    callMsgPack?: (ctx: Context, request: Request) => Promise<MsgPack<Response>>;
}
export interface RpcMethodStreaming<Context = unknown, Request = unknown, Response = unknown> extends RpcMethodBase<Context, Request> {
    isStreaming: true;
    preCallBufferSize?: number;
    call$?: (ctx: Context, request$: Observable<Request>) => Observable<Response>;
    callJson$?: (ctx: Context, request$: Observable<Request>) => Observable<json_string<Response>>;
    callMsgPack$?: (ctx: Context, request$: Observable<Request>) => Observable<MsgPack<Response>>;
}
export declare type RpcMethodWrap<Context = unknown, Request = unknown, Response = unknown> = RpcMethodStaticWrap<Context, Request, Response> | RpcMethodStreamingWrap<Context, Request, Response>;
export declare type RpcMethodWrapFromRpcMethod<T> = T extends RpcMethod<infer A, infer B, infer C> ? RpcMethodWrap<A, B, C> : never;
export declare type RpcApi<Context = unknown, T = unknown> = Record<string, RpcMethod<Context, T, T>>;
export interface IRpcApiCaller<Api extends Record<string, RpcMethod<Ctx, any, any>>, Ctx = unknown> {
    exists<K extends keyof Api>(name: K): boolean;
    get<K extends keyof Api>(name: K): RpcMethodWrapFromRpcMethod<Api[K]>;
    call<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<RpcMethodResponse<Api[K]>>;
    callJson<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<json_string<RpcMethodResponse<Api[K]>>>;
    callMsgPack<K extends keyof Api>(name: K, request: RpcMethodRequest<Api[K]>, ctx: Ctx): Promise<MsgPack<RpcMethodResponse<Api[K]>>>;
    call$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<RpcMethodResponse<Api[K]>>;
    callJson$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<json_string<RpcMethodResponse<Api[K]>>>;
    callMsgPack$<K extends keyof Api>(name: K, request$: Observable<RpcMethodRequest<Api[K]>>, ctx: Ctx): Observable<MsgPack<RpcMethodResponse<Api[K]>>>;
}
