"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpJsonRPC2Api = void 0;
const context_1 = require("../../context");
const util_1 = require("../../util");
const JsonRpc2Server_1 = require("../../../../common/json-rpc/JsonRpc2Server");
const constants_1 = require("../rpc/constants");
const JsonRpc2CodecJsonString_1 = require("../../../../common/json-rpc/codec/json-string/JsonRpc2CodecJsonString");
const STATUS_NO_RESPONSE = Buffer.from('204 No Response');
const HDR_KEY_CONTENT_TYPE = Buffer.from('Content-Type');
const HDR_VALUE_APPLICATION_JSON = Buffer.from('application/json');
function processHttpRpcRequest(res, ctx, body, server) {
    try {
        if (typeof body === 'string') {
            ctx.payloadSize = body.length;
        }
        else {
            ctx.payloadSize = body.byteLength;
            body = body.toString('utf8') || 'null';
        }
        server
            .onMessages(ctx, body)
            .then((result) => {
            if (res.aborted)
                return;
            res.cork(() => {
                if (!result)
                    res.writeStatus(STATUS_NO_RESPONSE).end();
                else
                    res.writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(result);
            });
        })
            .catch(() => {
            if (!res.aborted)
                res.writeStatus(constants_1.STATUS_400).end();
        });
    }
    catch (_a) {
        if (!res.aborted)
            res.writeStatus(constants_1.STATUS_400).end();
    }
}
const enableHttpJsonRPC2Api = (params) => {
    const { uws, route = '/json-rpc', createContext = context_1.createConnectionContext, caller, onNotification = () => { }, } = params;
    const server = new JsonRpc2Server_1.JsonRpc2Server({
        caller,
        codec: new JsonRpc2CodecJsonString_1.JsonRpc2CodecJsonString(),
        onNotification,
        strict: false,
    });
    uws.post(route, (res, req) => {
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        (0, util_1.readBody)(res, (buffer) => {
            processHttpRpcRequest(res, ctx, buffer, server);
        });
    });
    uws.get(route, (res, req) => {
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, body, server);
    });
};
exports.enableHttpJsonRPC2Api = enableHttpJsonRPC2Api;
