"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpRpcJsonGetApi = exports.enableHttpRpcJsonPostApi = void 0;
const rpc_1 = require("../../../../common/rpc");
const context_1 = require("../../context");
const util_1 = require("../../util");
const util_2 = require("../util");
const constants_1 = require("./constants");
const HDR_KEY_CONTENT_TYPE = Buffer.from('Content-Type');
const HDR_VALUE_APPLICATION_JSON = Buffer.from('application/json');
const sendError = (res, error, pretty) => {
    if (res.aborted)
        return;
    res.cork(() => {
        const errorFormatted = (0, rpc_1.formatError)(error);
        const body = JSON.stringify(errorFormatted, null, pretty ? 4 : 0);
        res.writeStatus(constants_1.STATUS_400).writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(body);
    });
};
function processHttpRpcRequest(res, ctx, name, body, caller) {
    try {
        const json = (0, util_2.parsePayload)(ctx, body);
        caller
            .call(name, json, ctx)
            .then((result) => {
            if (res.aborted)
                return;
            res.cork(() => {
                const method = caller.get(name);
                const formatted = JSON.stringify(result, null, method.pretty ? 4 : 0);
                res.writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(formatted);
            });
        })
            .catch((error) => {
            sendError(res, error, false);
        });
    }
    catch (_a) {
        const error = new Error('Could not parse payload');
        sendError(res, error, false);
    }
}
const enableHttpRpcJsonPostApi = (params) => {
    const { uws, route = '/rpc/json/*', createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        (0, util_1.readBody)(res, (buffer) => {
            processHttpRpcRequest(res, ctx, name, buffer, caller);
        });
    });
};
exports.enableHttpRpcJsonPostApi = enableHttpRpcJsonPostApi;
const enableHttpRpcJsonGetApi = (params) => {
    const { uws, route = '/rpc/json/*', createContext = context_1.createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, name, body, caller);
    });
};
exports.enableHttpRpcJsonGetApi = enableHttpRpcJsonGetApi;
