"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsBinaryReactiveRpcApi = void 0;
const context_1 = require("./context");
const binary_1 = require("../../common/codec/binary");
const binary_msgpack_1 = require("../../common/codec/binary-msgpack");
const bufferToUint8Array_1 = require("../../../util/bufferToUint8Array");
const enableWsBinaryReactiveRpcApi = (params) => {
    const encoder = new binary_1.Encoder();
    const decoder = new binary_msgpack_1.Decoder();
    const { wss, createContext, createRpcServer, onNotification } = params;
    wss.on('connection', (ws, req) => {
        let ctx;
        const rpc = createRpcServer({
            send: (messages) => {
                const uint8 = encoder.encode(messages);
                ws.send(uint8);
            },
        });
        if (onNotification) {
            rpc.onNotification = (name, data, ctx) => {
                onNotification(ws, name, data, ctx);
            };
        }
        ws.on('upgrade', (req) => {
            ctx = (createContext || context_1.createConnectionContext)(req);
        });
        ws.on('message', (buf) => {
            const uint8 = (0, bufferToUint8Array_1.bufferToUint8Array)(buf);
            const messages = decoder.decode(uint8);
            const length = messages.length;
            for (let i = 0; i < length; i++)
                rpc.onMessage(messages[i], ctx);
        });
        ws.on('close', () => {
            rpc.stop();
        });
    });
};
exports.enableWsBinaryReactiveRpcApi = enableWsBinaryReactiveRpcApi;
