"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Codegen = exports.CodegenStepExecJs = void 0;
const _1 = require(".");
class CodegenStepExecJs {
    constructor(js) {
        this.js = js;
    }
}
exports.CodegenStepExecJs = CodegenStepExecJs;
class Codegen {
    constructor(opts) {
        this.steps = [];
        this.registerCounter = 0;
        this.dependencies = [];
        this.dependencyNames = [];
        this.linked = {};
        this.constants = [];
        this.constantNames = [];
        this.options = Object.assign({ args: 'r0', name: '', prologue: '', epilogue: '', processSteps: (steps) => steps.filter((step) => step instanceof CodegenStepExecJs), linkable: {} }, opts);
    }
    js(js) {
        this.steps.push(new CodegenStepExecJs(js));
    }
    step(step) {
        this.steps.push(step);
    }
    getRegister() {
        return `r${this.registerCounter++}`;
    }
    linkDependency(dep, name = 'd' + this.dependencies.length) {
        this.dependencies.push(dep);
        this.dependencyNames.push(name);
        return name;
    }
    linkDependencies(deps) {
        return deps.map((dep) => this.linkDependency(dep));
    }
    link(name) {
        if (this.linked[name])
            return;
        this.linked[name] = 1;
        this.linkDependency(this.options.linkable[name], name);
    }
    addConstant(constant, name = 'c' + this.constants.length) {
        this.constants.push(constant);
        this.constantNames.push(name);
        return name;
    }
    addConstants(constants) {
        return constants.map((constant) => this.addConstant(constant));
    }
    generate(opts = {}) {
        const { name, args, prologue, epilogue } = Object.assign(Object.assign({}, this.options), opts);
        const steps = this.options.processSteps(this.steps);
        const js = `(function(${this.dependencyNames.join(', ')}) {
${this.constants.map((constant, index) => `var ${this.constantNames[index]} = (${constant});`).join('\n')}
return ${name ? `function ${name}` : 'function'}(${args}){
${prologue}
${steps.map((step) => step.js).join('\n')}
${epilogue}
}})`;
        return {
            deps: this.dependencies,
            js: js,
        };
    }
    compile(opts) {
        return (0, _1.compileClosure)(this.generate(opts));
    }
}
exports.Codegen = Codegen;
