import { CompleteMessage } from '../messages/CompleteMessage';
import { DataMessage } from '../messages/DataMessage';
import { ErrorMessage } from '../messages/ErrorMessage';
import { NotificationMessage } from '../messages/NotificationMessage';
import { SubscribeMessage } from '../messages/SubscribeMessage';
import { UnsubscribeMessage } from '../messages/UnsubscribeMessage';
import { readHeader } from './header';
const readMethod = (arr, offset) => {
    const size = arr[offset++];
    let str = '';
    for (let i = 0; i < size; i++)
        str += String.fromCharCode(arr[offset++]);
    return [str, offset];
};
export const decodeFullMessage = (arr, offset) => {
    const byte1 = arr[offset++];
    const code = byte1 >>> 5;
    switch (code) {
        case 0: {
            const [length, off1] = readHeader(byte1, arr, offset);
            const [method, off2] = readMethod(arr, off1);
            const data = arr.subarray(off2, off2 + length);
            return [new NotificationMessage(method, data), off2 + length];
        }
        case 1: {
            const [length, off1] = readHeader(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            const [method, off2] = readMethod(arr, off1 + 2);
            const data = arr.subarray(off2, off2 + length);
            return [new SubscribeMessage(id, method, data), off2 + length];
        }
        case 2:
        case 3:
        case 5: {
            const [length, off1] = readHeader(byte1, arr, offset);
            const o1 = arr[off1];
            const o2 = arr[off1 + 1];
            const id = (o1 << 8) | o2;
            offset = off1 + 2;
            const data = arr.subarray(offset, offset + length);
            const message = code === 2
                ? new DataMessage(id, data)
                : code === 3
                    ? new CompleteMessage(id, data)
                    : new ErrorMessage(id, data);
            return [message, offset + length];
        }
        case 4: {
            const o1 = arr[offset++];
            const o2 = arr[offset++];
            const id = (o1 << 8) | o2;
            return [new UnsubscribeMessage(id), offset];
        }
    }
    throw new Error('UNKNOWN_MESSAGE');
};
export const decodeFullMessages = (arr, offset) => {
    const messages = [];
    const length = arr.length;
    while (offset < length) {
        const [message, off] = decodeFullMessage(arr, offset);
        offset = off;
        messages.push(message);
    }
    return messages;
};
