import { ServerTimestamp } from './server';
export class LogicalTimestamp extends ServerTimestamp {
    sessionId;
    time;
    constructor(sessionId, time) {
        super(time);
        this.sessionId = sessionId;
        this.time = time;
    }
    getSessionId() {
        return this.sessionId;
    }
    stamp(sessionId, time) {
        return new LogicalTimestamp(sessionId, time);
    }
    tick(cycles) {
        return new LogicalTimestamp(this.getSessionId(), this.time + cycles);
    }
    interval(cycles, span) {
        return new LogicalTimespan(this.getSessionId(), this.time + cycles, span);
    }
    clock() {
        return new LogicalClock(this.getSessionId(), this.time);
    }
}
export class LogicalTimespan extends LogicalTimestamp {
    span;
    constructor(sessionId, time, span) {
        super(sessionId, time);
        this.span = span;
    }
}
export class LogicalClock extends LogicalTimestamp {
    tick(cycles) {
        const timestamp = new LogicalTimestamp(this.sessionId, this.time);
        this.time += cycles;
        return timestamp;
    }
}
export class LogicalVectorClock extends LogicalClock {
    clocks = new Map();
    observe(ts, span) {
        const time = ts.time + span - 1;
        const clock = this.clocks.get(ts.getSessionId());
        if (!clock)
            this.clocks.set(ts.getSessionId(), ts.tick(span - 1));
        else if (time > clock.time)
            clock.time = ts.time;
        if (time >= this.time)
            this.time = time + 1;
    }
    clone() {
        return this.fork(this.getSessionId());
    }
    fork(sessionId) {
        const clock = new LogicalVectorClock(sessionId, this.time);
        for (const ts of this.clocks.values())
            clock.observe(ts.tick(0), 1);
        return clock;
    }
}
