import { CrdtEncoder } from '../../util/binary/CrdtEncoder';
import { DeleteOperation } from '../../operations/DeleteOperation';
import { InsertArrayElementsOperation } from '../../operations/InsertArrayElementsOperation';
import { InsertBinaryDataOperation } from '../../operations/InsertBinaryDataOperation';
import { InsertStringSubstringOperation } from '../../operations/InsertStringSubstringOperation';
import { MakeArrayOperation } from '../../operations/MakeArrayOperation';
import { MakeBinaryOperation } from '../../operations/MakeBinaryOperation';
import { MakeNumberOperation } from '../../operations/MakeNumberOperation';
import { MakeObjectOperation } from '../../operations/MakeObjectOperation';
import { MakeStringOperation } from '../../operations/MakeStringOperation';
import { MakeValueOperation } from '../../operations/MakeValueOperation';
import { NoopOperation } from '../../operations/NoopOperation';
import { SetNumberOperation } from '../../operations/SetNumberOperation';
import { SetObjectKeysOperation } from '../../operations/SetObjectKeysOperation';
import { SetRootOperation } from '../../operations/SetRootOperation';
import { SetValueOperation } from '../../operations/SetValueOperation';
export class Encoder extends CrdtEncoder {
    patchId;
    encode(patch) {
        this.reset();
        const id = (this.patchId = patch.getId());
        const isServerClock = id.getSessionId() === 1;
        if (isServerClock) {
            this.b1vuint56(true, id.time);
        }
        else {
            this.b1vuint56(false, id.getSessionId());
            this.vuint57(id.time);
        }
        this.encodeOperations(patch);
        return this.flush();
    }
    encodeOperations(patch) {
        const ops = patch.ops;
        for (let i = 0; i < ops.length; i++) {
            const op = ops[i];
            this.encodeOperation(op);
        }
    }
    encodeId(id) {
        const sessionId = id.getSessionId();
        const time = id.time;
        if (sessionId === 1) {
            this.b1vuint56(true, id.time);
        }
        else {
            const patchId = this.patchId;
            if (sessionId === patchId.getSessionId() && time >= patchId.time) {
                this.b1vuint56(false, 1);
                this.vuint57(time - patchId.time);
            }
            else {
                this.b1vuint56(false, sessionId);
                this.vuint57(time);
            }
        }
    }
    encodeOperation(op) {
        if (op instanceof MakeObjectOperation) {
            this.u8(0);
            return;
        }
        else if (op instanceof MakeArrayOperation) {
            this.u8(1);
            return;
        }
        else if (op instanceof MakeStringOperation) {
            this.u8(2);
            return;
        }
        else if (op instanceof MakeValueOperation) {
            this.u8(14);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof MakeNumberOperation) {
            this.u8(3);
            return;
        }
        else if (op instanceof SetRootOperation) {
            this.u8(4);
            this.encodeId(op.value);
            return;
        }
        else if (op instanceof SetObjectKeysOperation) {
            this.u8(5);
            this.encodeId(op.object);
            this.vuint57(op.tuples.length);
            for (const [key, value] of op.tuples) {
                this.encodeId(value);
                this.encodeString(key);
            }
            return;
        }
        else if (op instanceof SetValueOperation) {
            this.u8(15);
            this.encodeId(op.obj);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof SetNumberOperation) {
            this.u8(6);
            this.encodeId(op.num);
            this.encodeNumber(op.value);
            return;
        }
        else if (op instanceof InsertStringSubstringOperation) {
            this.u8(7);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.encodeString(op.substring);
            return;
        }
        else if (op instanceof InsertArrayElementsOperation) {
            const { arr, after, elements } = op;
            const length = elements.length;
            this.u8(8);
            this.encodeId(arr);
            this.encodeId(after);
            this.vuint57(length);
            for (let i = 0; i < length; i++)
                this.encodeId(elements[i]);
            return;
        }
        else if (op instanceof DeleteOperation) {
            const { obj, after } = op;
            const length = after.span;
            if (length > 1) {
                this.u8(10);
                this.encodeId(obj);
                this.encodeId(after);
                this.vuint57(length);
                return;
            }
            this.u8(9);
            this.encodeId(obj);
            this.encodeId(after);
            return;
        }
        else if (op instanceof NoopOperation) {
            const { length } = op;
            if (length > 1) {
                this.u8(12);
                this.vuint57(length);
                return;
            }
            this.u8(11);
            return;
        }
        else if (op instanceof MakeBinaryOperation) {
            this.u8(16);
            return;
        }
        else if (op instanceof InsertBinaryDataOperation) {
            const buf = op.data;
            const length = buf.length;
            this.u8(17);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.vuint57(length);
            this.buf(buf, length);
            return;
        }
        throw new Error('UNKNOWN_OP');
    }
}
