import { LogicalTimestamp, LogicalVectorClock } from '../../clock';
import { FALSE_ID, NULL_ID, TRUE_ID, UNDEFINED_ID } from '../../constants';
export class ClockDecoder {
    table = new Map();
    index;
    clock;
    static fromArr(arr) {
        const decoder = new ClockDecoder(arr[0], 0);
        const length = arr.length;
        for (let i = 0; i < length; i += 2)
            decoder.pushTuple(arr[i], arr[i + 1]);
        return decoder;
    }
    constructor(sessionId, time) {
        this.index = 1;
        this.clock = new LogicalVectorClock(sessionId, time);
        if (time)
            this.clock.observe(new LogicalTimestamp(sessionId, time - 1), 1);
    }
    pushTuple(sessionId, time) {
        const ts = new LogicalTimestamp(sessionId, time);
        this.clock.observe(ts, 1);
        this.table.set(this.index++, ts);
    }
    decodeId(sessionIndex, timeDiff) {
        if (sessionIndex === 0) {
            switch (timeDiff) {
                case 1:
                    return NULL_ID;
                case 2:
                    return TRUE_ID;
                case 3:
                    return FALSE_ID;
                case 4:
                    return UNDEFINED_ID;
            }
            return new LogicalTimestamp(0, timeDiff);
        }
        const ts = this.table.get(sessionIndex);
        if (!ts)
            throw new Error('INVALID_CLOCK_TABLE');
        return new LogicalTimestamp(ts.getSessionId(), ts.time - timeDiff);
    }
}
