export declare type JsonCodecTimestamp = [sessionId: number, time: number] | number;
export interface JsonCodecPatch {
    id: JsonCodecTimestamp;
    ops: JsonCodecOperation[];
}
export declare type JsonCodecOperation = JsonCodecMakeObjectOperation | JsonCodecMakeArrayOperation | JsonCodecMakeStringOperation | JsonCodecMakeBinaryOperation | JsonCodecMakeValueOperation | JsonCodecMakeNumberOperation | JsonCodecMakeConstantOperation | JsonCodecSetRootOperation | JsonCodecSetObjectKeysOperation | JsonCodecSetNumberOperation | JsonCodecSetValueOperation | JsonCodecInsertStringSubstringOperation | JsonCodecBinaryInsertDataOperation | JsonCodecInsertArrayElementsOperation | JsonCodecDeleteOperation | JsonCodecNoopOperation;
export interface JsonCodecMakeObjectOperation {
    op: 'obj';
}
export interface JsonCodecMakeArrayOperation {
    op: 'arr';
}
export interface JsonCodecMakeStringOperation {
    op: 'str';
}
export interface JsonCodecMakeBinaryOperation {
    op: 'bin';
}
export interface JsonCodecMakeNumberOperation {
    op: 'num';
}
export interface JsonCodecMakeValueOperation {
    op: 'val';
    value: unknown;
}
export interface JsonCodecMakeConstantOperation {
    op: 'const';
    value: unknown;
}
export interface JsonCodecSetRootOperation {
    op: 'root';
    value: JsonCodecTimestamp;
}
export interface JsonCodecSetObjectKeysOperation {
    op: 'obj_set';
    obj: JsonCodecTimestamp;
    tuples: [key: string, value: JsonCodecTimestamp][];
}
export interface JsonCodecSetNumberOperation {
    op: 'num_set';
    after: JsonCodecTimestamp;
    value: number;
}
export interface JsonCodecSetValueOperation {
    op: 'val_set';
    obj: JsonCodecTimestamp;
    value: unknown;
}
export interface JsonCodecInsertStringSubstringOperation {
    op: 'str_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecBinaryInsertDataOperation {
    op: 'bin_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecInsertArrayElementsOperation {
    op: 'arr_ins';
    obj: JsonCodecTimestamp;
    after?: JsonCodecTimestamp;
    values: JsonCodecTimestamp[];
}
export interface JsonCodecDeleteOperation {
    op: 'del';
    obj: JsonCodecTimestamp;
    after: JsonCodecTimestamp;
    len?: number;
}
export interface JsonCodecNoopOperation {
    op: 'noop';
    len?: number;
}
