import { ArrayApi } from './ArrayApi';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { BinaryApi } from './BinaryApi';
import { BinaryType } from '../../types/rga-binary/BinaryType';
import { NULL, UNDEFINED } from '../../constants';
import { ObjectApi } from './ObjectApi';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { StringApi } from './StringApi';
import { StringType } from '../../types/rga-string/StringType';
import { ValueApi } from './ValueApi';
import { ValueType } from '../../types/lww-value/ValueType';
export class Finder {
    node;
    api;
    constructor(node, api) {
        this.node = node;
        this.api = api;
    }
    find(steps) {
        let node = this.node || NULL;
        const length = steps.length;
        if (!length)
            return node;
        let i = 0;
        while (i < length) {
            const step = steps[i++];
            if (node instanceof ObjectType) {
                const id = node.get(String(step));
                if (!id)
                    return UNDEFINED;
                const nextNode = this.api.model.node(id);
                if (!nextNode)
                    return UNDEFINED;
                node = nextNode;
            }
            else if (node instanceof ArrayType) {
                const id = node.findValue(Number(step));
                const nextNode = this.api.model.node(id);
                if (!nextNode)
                    return UNDEFINED;
                node = nextNode;
                continue;
            }
        }
        return node;
    }
    val(path) {
        const node = this.find(path);
        if (node instanceof ValueType)
            return new ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    }
    str(path) {
        const node = this.find(path);
        if (node instanceof StringType)
            return new StringApi(this.api, node);
        throw new Error('NOT_STR');
    }
    bin(path) {
        const node = this.find(path);
        if (node instanceof BinaryType)
            return new BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    }
    arr(path) {
        const node = this.find(path);
        if (node instanceof ArrayType)
            return new ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    }
    obj(path) {
        const obj = this.find(path);
        if (obj instanceof ObjectType)
            return new ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    }
}
