import { Batch } from '../../../json-crdt-patch/Batch';
import { Finder } from './Finder';
import { NULL } from '../../constants';
import { PatchBuilder } from '../../../json-crdt-patch/PatchBuilder';
export class ModelApi {
    model;
    batch = new Batch([]);
    builder;
    getApi() {
        return this;
    }
    constructor(model) {
        this.model = model;
        this.builder = new PatchBuilder(model.clock);
    }
    getNode() {
        const model = this.model;
        const id = model.root.toValue();
        return model.node(id) || NULL;
    }
    find(path) {
        const finder = new Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    }
    val(path) {
        return this.find().val(path);
    }
    str(path) {
        return this.find().str(path);
    }
    bin(path) {
        return this.find().bin(path);
    }
    arr(path) {
        return this.find().arr(path);
    }
    obj(path) {
        return this.find().obj(path);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.batch.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    }
    flush() {
        const batch = this.batch;
        this.batch = new Batch([]);
        return batch;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    toView() {
        return this.getNode().toJson();
    }
}
