import { LogicalTimestamp } from '../../../json-crdt-patch/clock';
export class ArrayChunk {
    id;
    nodes;
    left = null;
    right = null;
    deleted;
    constructor(id, nodes) {
        this.id = id;
        this.nodes = nodes;
    }
    span() {
        return this.deleted || this.nodes.length;
    }
    split(time) {
        const newSpan = 1 + time - this.id.time;
        const newId = new LogicalTimestamp(this.id.getSessionId(), time + 1);
        if (this.nodes) {
            const newChunkValues = this.nodes.splice(newSpan);
            return new ArrayChunk(newId, newChunkValues);
        }
        else {
            const chunk = new ArrayChunk(newId, undefined);
            chunk.deleted = this.span() - newSpan;
            this.deleted = newSpan;
            return chunk;
        }
    }
    delete() {
        this.deleted = this.span();
        delete this.nodes;
    }
    merge(values) {
        this.nodes.push(...values);
    }
    clone(model) {
        if (this.nodes) {
            const nodes = [];
            for (const node of this.nodes)
                nodes.push(node.clone(model));
            return new ArrayChunk(this.id, nodes);
        }
        else {
            const chunk = new ArrayChunk(this.id, undefined);
            chunk.deleted = this.deleted;
            delete this.nodes;
            return chunk;
        }
    }
    toString(tab = '') {
        const str = `${tab}ArrayChunk(${this.id.toDisplayString()}) { ${!this.nodes ? `[${this.deleted || 0}]` : this.nodes.map((val) => val.toString()).join(', ')} }`;
        return str;
    }
}
