import { LogicalTimestamp } from '../../../json-crdt-patch/clock';
export class BinaryChunk {
    id;
    buf;
    left = null;
    right = null;
    deleted;
    constructor(id, buf) {
        this.id = id;
        this.buf = buf;
    }
    span() {
        return this.deleted || this.buf.byteLength;
    }
    split(time) {
        const newSpan = 1 + time - this.id.time;
        const newId = new LogicalTimestamp(this.id.getSessionId(), time + 1);
        if (this.buf) {
            const newChunkStr = this.buf.subarray(newSpan);
            this.buf = this.buf.subarray(0, newSpan);
            return new BinaryChunk(newId, newChunkStr);
        }
        else {
            const chunk = new BinaryChunk(newId, undefined);
            chunk.deleted = this.span() - newSpan;
            this.deleted = newSpan;
            return chunk;
        }
    }
    delete() {
        this.deleted = this.span();
        delete this.buf;
    }
    merge(buf) {
        if (this.deleted) {
            this.deleted += buf.length;
            return;
        }
        const length = this.buf.length;
        const combined = new Uint8Array(length + buf.length);
        combined.set(this.buf);
        combined.set(buf, length);
        this.buf = combined;
    }
    clone() {
        const chunk = new BinaryChunk(this.id, this.buf);
        if (this.deleted) {
            chunk.deleted = this.deleted;
            delete chunk.buf;
        }
        return chunk;
    }
    toString(tab = '') {
        const str = `${tab}${this.constructor.name}(${this.id.toDisplayString()}) { ${!this.buf
            ? `[${this.deleted || this.span()}]`
            : this.buf.length > 10
                ? this.buf.subarray(0, 10).toString() + ',…'
                : this.buf.toString()} }`;
        return str;
    }
}
