import type { Display, Identifiable } from './common';
export interface TType extends Display, Partial<Identifiable> {
    __t: string;
    examples?: TExample[];
}
export interface TExample extends Display {
    value: unknown;
}
export interface Validatable {
    validator?: string | string[];
}
export interface TObject extends TType, Validatable {
    __t: 'obj';
    fields: TObjectField[];
    unknownFields?: boolean;
}
export interface TObjectField extends Display {
    key: string;
    type: TType;
    isOptional?: boolean;
}
export interface TArray extends TType, Validatable {
    __t: 'arr';
    type: TType;
    const?: unknown[];
}
export interface TNumber extends TType, Validatable {
    __t: 'num';
    const?: number;
    format?: 'i' | 'u' | 'f' | 'i8' | 'i16' | 'i32' | 'i64' | 'u8' | 'u16' | 'u32' | 'u64' | 'f32' | 'f64';
}
export interface TString extends TType, Validatable {
    __t: 'str';
    const?: string;
    format?: 'ascii';
    noJsonEscape?: boolean;
    minLength?: number;
    maxLength?: number;
}
export interface TBoolean extends TType {
    __t: 'bool';
    const?: boolean;
}
export interface TNull extends TType {
    __t: 'nil';
}
export interface TBinary extends TType, Validatable {
    __t: 'bin';
}
export interface TAny extends TType, Validatable {
    __t: 'any';
}
export interface TRef extends TType {
    __t: 'ref';
    ref: string;
}
export interface TOr extends TType {
    __t: 'or';
    types: TType[];
}
export interface TEnum extends TType {
    __t: 'enum';
    values: unknown[];
}
export declare type TJson = TObject | TArray | TNumber | TString | TBoolean | TNull;
export declare type TMessagePack = TJson | TBinary;
export declare type TAnyType = TMessagePack | TRef | TOr | TEnum | TAny;
export declare type NoT<T extends TType> = Omit<T, '__t'>;
