import { writeHeader } from './header';
import { BinaryNotificationMessage, BinaryRequestDataMessage, BinaryRequestCompleteMessage, BinaryRequestErrorMessage, BinaryRequestUnsubscribeMessage, BinaryResponseDataMessage, BinaryResponseCompleteMessage, BinaryResponseErrorMessage, BinaryResponseUnsubscribeMessage, } from '../../messages/binary';
export class Encoder {
    uint8 = new Uint8Array(0);
    offset = 0;
    encode(messages) {
        let size = 0;
        const length = messages.length;
        for (let i = 0; i < length; i++)
            size += messages[i].size();
        this.uint8 = new Uint8Array(size);
        this.offset = 0;
        for (let i = 0; i < length; i++) {
            const message = messages[i];
            if (message instanceof BinaryNotificationMessage)
                this.notif(message);
            else if (message instanceof BinaryRequestDataMessage)
                this.msgWithIdAndMethod(0, message.id, message.method, message.data);
            else if (message instanceof BinaryRequestCompleteMessage)
                this.msgWithIdAndMethod(1, message.id, message.method, message.data);
            else if (message instanceof BinaryRequestErrorMessage)
                this.msgWithIdAndMethod(2, message.id, message.method, message.data);
            else if (message instanceof BinaryRequestUnsubscribeMessage)
                this.reqUnsub(message);
            else if (message instanceof BinaryResponseDataMessage)
                this.resData(message);
            else if (message instanceof BinaryResponseCompleteMessage)
                this.resComp(message);
            else if (message instanceof BinaryResponseErrorMessage)
                this.resErr(message);
            else if (message instanceof BinaryResponseUnsubscribeMessage)
                this.resUnsub(message);
        }
        return this.uint8;
    }
    notif(message) {
        const { method, data } = message;
        const length = data ? data.byteLength : 0;
        this.header(3, length);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    reqUnsub(message) {
        this.uint8[this.offset++] = 254;
        this.id(message.id);
    }
    resData(message) {
        this.msgWithId(4, message.id, message.data);
    }
    resComp(message) {
        this.msgWithId(5, message.id, message.data);
    }
    resErr(message) {
        this.msgWithId(6, message.id, message.data);
    }
    resUnsub(message) {
        this.uint8[this.offset++] = 255;
        this.id(message.id);
    }
    msgWithIdAndMethod(code, id, method, data) {
        const length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    msgWithId(code, id, data) {
        const length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    }
    header(code, length) {
        this.offset = writeHeader(this.uint8, this.offset, code, length);
    }
    id(id) {
        this.uint8[this.offset++] = id >>> 8;
        this.uint8[this.offset++] = id & 0xff;
    }
    method(method) {
        const uint8 = this.uint8;
        const length = method.length;
        let offset = this.offset;
        uint8[offset++] = length;
        for (let i = 0; i < length; i++)
            uint8[offset++] = method.charCodeAt(i);
        this.offset = offset;
    }
}
