export class JsonRpc2CodecJson {
    decode(data) {
        return Array.isArray(data) ? data : [data];
    }
    encodeError(error, id) {
        const message = {
            jsonrpc: '2.0',
            id,
            error,
        };
        return message;
    }
    encodeParseError() {
        return this.encodeError({
            code: -32700,
            message: 'Parse error',
        }, null);
    }
    encodeInvalidRequestError(id) {
        return this.encodeError({
            code: -32600,
            message: 'Invalid Request',
        }, id);
    }
    encodeMethodNotFoundError(id) {
        return this.encodeError({
            code: -32601,
            message: 'Method not found',
        }, id);
    }
    encodeInvalidParamsError(id) {
        return this.encodeError({
            code: -32602,
            message: 'Invalid params',
        }, id);
    }
    encodeInternalError(id) {
        return this.encodeError({
            code: -32603,
            message: 'Internal error',
        }, id);
    }
    encodeResponse(id, result) {
        const responseMessage = {
            jsonrpc: '2.0',
            id,
            result,
        };
        return responseMessage;
    }
    encodeBatch(messages) {
        return messages;
    }
}
