import { Observable } from 'rxjs';
import { ReactiveRpcRequestMessage, ReactiveRpcResponseMessage, RequestUnsubscribeMessage, ResponseCompleteMessage, ResponseDataMessage, ResponseErrorMessage } from '../messages/nominal';
import { TimedQueue } from '../util/TimedQueue';
export interface RpcClientParams<T = unknown> {
    send: (messages: ReactiveRpcRequestMessage<T>[]) => void;
    bufferSize?: number;
    bufferTime?: number;
}
export declare class RpcClient<T = unknown> {
    private id;
    readonly buffer: TimedQueue<ReactiveRpcRequestMessage<T>>;
    private readonly calls;
    constructor({ send, bufferSize, bufferTime }: RpcClientParams<T>);
    getInflightCallCount(): number;
    onMessages(messages: ReactiveRpcResponseMessage<T>[]): void;
    onMessage(message: ReactiveRpcResponseMessage<T>): void;
    onResponseComplete(message: ResponseCompleteMessage<T>): void;
    onResponseData(message: ResponseDataMessage<T>): void;
    onResponseError(message: ResponseErrorMessage<T>): void;
    onRequestUnsubscribe(message: RequestUnsubscribeMessage): void;
    call$(method: string, data: T): Observable<T>;
    call$(method: string, data: Observable<T>): Observable<T>;
    call(method: string, request: T): Promise<T>;
    notify(method: string, data: undefined | T): void;
    stop(reason?: string): void;
    disconnect(): void;
}
