import { RpcServerError } from './constants';
export const formatErrorLike = (error) => {
    const out = { message: error.message };
    if (typeof error.status === 'number')
        out.status = error.status;
    if (typeof error.code === 'string')
        out.code = error.code;
    if (typeof error.errno === 'number')
        out.errno = error.errno;
    if (typeof error.errorId === 'number')
        out.errorId = error.errorId;
    return out;
};
export const isErrorLike = (error) => {
    if (error instanceof Error)
        return true;
    if (typeof error === 'object')
        if (typeof error.message === 'string')
            return true;
    return false;
};
export const formatError = (error) => {
    if (isErrorLike(error))
        return formatErrorLike(error);
    return {
        message: String(error),
    };
};
export const formatErrorCode = (errno) => {
    return {
        message: 'PROTOCOL',
        errno,
        code: RpcServerError[errno],
    };
};
export class ErrorLikeErrorFormatter {
    format(error) {
        return formatError(error);
    }
    formatValidation(error) {
        return this.format(error);
    }
    formatCode(code) {
        return formatErrorCode(code);
    }
}
export class RpcError extends Error {
    errno;
    code;
    constructor(errno, message = 'PROTOCOL') {
        super(message);
        this.errno = errno;
        this.code = RpcServerError[errno];
    }
}
