import { JSON } from '../../../json-brand';
import { decode, encodeFull } from '../../../json-pack/util';
import { of, firstValueFrom, from } from 'rxjs';
import { map } from 'rxjs/operators';
export class RpcMethodStaticWrap {
    isStreaming = false;
    req;
    res;
    validate;
    onPreCall;
    pretty;
    call;
    callJson;
    callMsgPack;
    call$;
    callJson$;
    callMsgPack$;
    constructor(method, types) {
        this.req = method.req || '';
        this.res = method.res || '';
        this.validate = method.validate ? method.validate.bind(method) : undefined;
        this.onPreCall = method.onPreCall ? method.onPreCall.bind(method) : undefined;
        this.pretty = !!method.pretty;
        const call = method.call ? method.call.bind(method) : undefined;
        const callJson = method.callJson ? method.callJson.bind(method) : undefined;
        const callMsgPack = method.callMsgPack ? method.callMsgPack.bind(method) : undefined;
        if (call)
            this.call = call;
        else if (callJson) {
            this.call = async (ctx, request) => JSON.parse(await callJson(ctx, request));
        }
        else if (callMsgPack) {
            this.call = async (ctx, request) => decode(await callMsgPack(ctx, request));
        }
        else
            throw new Error('No call method defined.');
        if (callJson) {
            this.callJson = callJson;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getJsonSerializer(this.res) : JSON.stringify;
            this.callJson = async (ctx, request) => serializer(await this.call(ctx, request));
        }
        if (callMsgPack) {
            this.callMsgPack = callMsgPack;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getMsgPackEncoder(this.res) : encodeFull;
            this.callMsgPack = async (ctx, request) => serializer(await this.call(ctx, request));
        }
        this.call$ = (ctx, request$) => from((async () => {
            const request = await firstValueFrom(request$);
            return await this.call(ctx, request);
        })());
        this.callJson$ = (ctx, request$) => from((async () => {
            const request = await firstValueFrom(request$);
            return await this.callJson(ctx, request);
        })());
        this.callMsgPack$ = (ctx, request$) => from((async () => {
            const request = await firstValueFrom(request$);
            return await this.callMsgPack(ctx, request);
        })());
    }
}
export class RpcMethodStreamingWrap {
    isStreaming = true;
    req;
    res;
    validate;
    onPreCall;
    pretty;
    preCallBufferSize;
    timeout;
    call;
    callJson;
    callMsgPack;
    call$;
    callJson$;
    callMsgPack$;
    constructor(method, types) {
        this.req = method.req || '';
        this.res = method.res || '';
        this.validate = method.validate ? method.validate.bind(method) : undefined;
        this.onPreCall = method.onPreCall ? method.onPreCall.bind(method) : undefined;
        this.preCallBufferSize = method.preCallBufferSize;
        this.pretty = !!method.pretty;
        const call$ = method.call$ ? method.call$.bind(method) : undefined;
        const callJson$ = method.callJson$ ? method.callJson$.bind(method) : undefined;
        const callMsgPack$ = method.callMsgPack$ ? method.callMsgPack$.bind(method) : undefined;
        if (call$)
            this.call$ = call$;
        else if (callJson$) {
            this.call$ = (ctx, request) => callJson$(ctx, request).pipe(map((json) => JSON.parse(json)));
        }
        else if (callMsgPack$) {
            this.call$ = (ctx, request) => callMsgPack$(ctx, request).pipe(map((blob) => decode(blob)));
        }
        else
            throw new Error('No call method defined.');
        if (callJson$) {
            this.callJson$ = callJson$;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getJsonSerializer(this.res) : JSON.stringify;
            this.callJson$ = (ctx, request) => this.call$(ctx, request).pipe(map((res) => serializer(res)));
        }
        if (callMsgPack$) {
            this.callMsgPack$ = callMsgPack$;
        }
        else {
            const serializer = this.res && types && types.hasType(this.res) ? types.getMsgPackEncoder(this.res) : encodeFull;
            this.callMsgPack$ = (ctx, request) => this.call$(ctx, request).pipe(map((res) => serializer(res)));
        }
        this.call = (ctx, request) => firstValueFrom(this.call$(ctx, of(request)));
        this.callJson = (ctx, request) => firstValueFrom(this.callJson$(ctx, of(request)));
        this.callMsgPack = (ctx, request) => firstValueFrom(this.callMsgPack$(ctx, of(request)));
    }
}
export const wrapMethod = (method, types) => {
    return method.isStreaming
        ? new RpcMethodStreamingWrap(method, types)
        : new RpcMethodStaticWrap(method, types);
};
