import { createConnectionContext } from '../../context';
import { readBody } from '../../util';
import { JsonRpc2Server } from '../../../../common/json-rpc/JsonRpc2Server';
import { STATUS_400 } from '../rpc/constants';
import { JsonRpc2CodecJsonString } from '../../../../common/json-rpc/codec/json-string/JsonRpc2CodecJsonString';
const STATUS_NO_RESPONSE = Buffer.from('204 No Response');
const HDR_KEY_CONTENT_TYPE = Buffer.from('Content-Type');
const HDR_VALUE_APPLICATION_JSON = Buffer.from('application/json');
function processHttpRpcRequest(res, ctx, body, server) {
    try {
        if (typeof body === 'string') {
            ctx.payloadSize = body.length;
        }
        else {
            ctx.payloadSize = body.byteLength;
            body = body.toString('utf8') || 'null';
        }
        server
            .onMessages(ctx, body)
            .then((result) => {
            if (res.aborted)
                return;
            res.cork(() => {
                if (!result)
                    res.writeStatus(STATUS_NO_RESPONSE).end();
                else
                    res.writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(result);
            });
        })
            .catch(() => {
            if (!res.aborted)
                res.writeStatus(STATUS_400).end();
        });
    }
    catch {
        if (!res.aborted)
            res.writeStatus(STATUS_400).end();
    }
}
export const enableHttpJsonRPC2Api = (params) => {
    const { uws, route = '/json-rpc', createContext = createConnectionContext, caller, onNotification = () => { }, } = params;
    const server = new JsonRpc2Server({
        caller,
        codec: new JsonRpc2CodecJsonString(),
        onNotification,
        strict: false,
    });
    uws.post(route, (res, req) => {
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        readBody(res, (buffer) => {
            processHttpRpcRequest(res, ctx, buffer, server);
        });
    });
    uws.get(route, (res, req) => {
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, body, server);
    });
};
