import { formatError } from '../../../../common/rpc';
import { encode, decode } from '../../../../../json-pack/util';
import { createConnectionContext } from '../../context';
import { readBody } from '../../util';
import { STATUS_400 } from './constants';
import { bufferToUint8Array } from '../../../../../util/bufferToUint8Array';
import { parsePayload } from '../util';
const DEFAULT_ROUTE = '/rpc/msgpack/*';
function sendError(res, error) {
    if (res.aborted)
        return;
    res.cork(() => {
        const body = encode(formatError(error));
        res.writeStatus(STATUS_400).end(body);
    });
}
function processRequest(res, ctx, name, json, caller) {
    try {
        caller
            .call(name, json, ctx)
            .then((result) => {
            if (res.aborted)
                return;
            res.cork(() => {
                res.end(encode(result));
            });
        })
            .catch((error) => {
            sendError(res, error);
        });
    }
    catch {
        const error = new Error('Could not execute request');
        sendError(res, error);
    }
}
export const enableHttpRpcMsgPackPostApi = (params) => {
    const { uws, route = DEFAULT_ROUTE, createContext = createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        readBody(res, (buffer) => {
            try {
                const length = (ctx.payloadSize = buffer.length);
                const json = length ? decode(bufferToUint8Array(buffer)) : null;
                processRequest(res, ctx, name, json, caller);
            }
            catch {
                const error = new Error('Could not parse payload');
                sendError(res, error);
            }
        });
    });
};
export const enableHttpRpcMsgPackGetApi = (params) => {
    const { uws, route = DEFAULT_ROUTE, createContext = createConnectionContext, caller } = params;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, (res, req) => {
        const url = req.getUrl();
        const name = url.substr(route.length - 1);
        const query = req.getQuery();
        const params = new URLSearchParams(query);
        const body = String(params.get('a') || 'null');
        const ctx = createContext(req, res);
        res.onAborted(() => {
            res.aborted = true;
        });
        try {
            const json = parsePayload(ctx, body);
            processRequest(res, ctx, name, json, caller);
        }
        catch {
            const error = new Error('Could not parse payload');
            sendError(res, error);
        }
    });
};
