import { BinaryNotificationMessage, } from '../../../common';
import { Decoder as DecoderJson } from '../../../common/codec/compact-json/Decoder';
import { Decoder as DecoderMsgPack } from '../../../common/codec/compact-msgpack/Decoder';
import { Encoder as EncoderCompactMsgPackBinary } from '../../../common/codec/compact-msgpack-binary';
import { Encoder as EncoderCompactJsonString } from '../../../common/codec/compact-json_string';
import { createConnectionContext } from '../context';
import { JsonNotificationMessage } from '../../../common/messages/json';
import { JSON } from '../../../../json-brand';
import { encode } from '../../../../json-pack/util';
export const enableWsCompactReactiveRpcApi = (params) => {
    const decoderJson = new DecoderJson();
    const decoderMsgPack = new DecoderMsgPack();
    const encoderCompactJsonString = new EncoderCompactJsonString();
    const encoderCompactMsgPackBinary = new EncoderCompactMsgPackBinary();
    const invalidPayloadJson = encoderCompactJsonString.encode([
        new JsonNotificationMessage('.err', JSON.stringify('CODING')),
    ]);
    const invalidPayloadMsgPack = encoderCompactMsgPackBinary.encode([
        new BinaryNotificationMessage('.err', encode('CODING')),
    ]);
    const { route = '/rpc/compact', uws, createRpcServerJson, createRpcServerMsgPack, onNotification, createContext = createConnectionContext, compression, idleTimeout = 0, maxBackpressure = 3145728, maxPayloadLength = 1048576, } = params;
    uws.ws(route, {
        idleTimeout,
        compression,
        maxBackpressure,
        maxPayloadLength,
        upgrade: (res, req, context) => {
            const secWebSocketKey = req.getHeader('sec-websocket-key');
            const secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            const secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            const ctx = createContext(req, res);
            const isBinary = secWebSocketProtocol.indexOf('MsgPack') > -1;
            res.upgrade({ ctx, isBinary }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: (ws) => {
            const { isBinary } = ws;
            let rpc;
            if (isBinary) {
                rpc = createRpcServerMsgPack({
                    send: (messages) => {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        const encoded = encoderCompactMsgPackBinary.encode(messages);
                        ws.send(encoded, true);
                    },
                });
            }
            else {
                rpc = createRpcServerJson({
                    send: (messages) => {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        const encoded = encoderCompactJsonString.encode(messages);
                        ws.send(encoded, false);
                    },
                });
            }
            if (onNotification) {
                rpc.onNotification = (name, data, ctx) => {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: (ws, buf, isBinary) => {
            const { ctx, rpc } = ws;
            let messages;
            try {
                messages = isBinary
                    ? decoderMsgPack.decode(new Uint8Array(buf))
                    : decoderJson.decode(Buffer.from(buf).toString('utf8'));
            }
            catch (error) {
                if (ws.getBufferedAmount() > maxBackpressure)
                    return;
                ws.send(isBinary ? invalidPayloadMsgPack : invalidPayloadJson, isBinary);
                return;
            }
            if (messages instanceof Array)
                rpc.onMessages(messages, ctx);
            else
                rpc.onMessage(messages, ctx);
        },
        close: (ws, code, message) => {
            ws.rpc.stop();
        },
    });
};
