import { createConnectionContext } from './context';
import { Encoder } from '../../common/codec/binary';
import { Decoder } from '../../common/codec/binary-msgpack';
import { bufferToUint8Array } from '../../../util/bufferToUint8Array';
export const enableWsBinaryReactiveRpcApi = (params) => {
    const encoder = new Encoder();
    const decoder = new Decoder();
    const { wss, createContext, createRpcServer, onNotification } = params;
    wss.on('connection', (ws, req) => {
        let ctx;
        const rpc = createRpcServer({
            send: (messages) => {
                const uint8 = encoder.encode(messages);
                ws.send(uint8);
            },
        });
        if (onNotification) {
            rpc.onNotification = (name, data, ctx) => {
                onNotification(ws, name, data, ctx);
            };
        }
        ws.on('upgrade', (req) => {
            ctx = (createContext || createConnectionContext)(req);
        });
        ws.on('message', (buf) => {
            const uint8 = bufferToUint8Array(buf);
            const messages = decoder.decode(uint8);
            const length = messages.length;
            for (let i = 0; i < length; i++)
                rpc.onMessage(messages[i], ctx);
        });
        ws.on('close', () => {
            rpc.stop();
        });
    });
};
