import { URL } from 'url';
export const createConnectionContext = (req) => {
    const ip = (typeof req.headers['x-forwarded-for'] === 'string' ? req.headers['x-forwarded-for'] : '') ||
        (typeof req.headers['x-real-ip'] === 'string' ? req.headers['x-real-ip'] : '') ||
        req.socket.remoteAddress ||
        '';
    let token = req.headers['authorization'] || '';
    if (!token && req.url) {
        const url = new URL(req.url);
        const params = new URLSearchParams(url.search);
        token = params.get('access_token') || '';
        if (!token)
            token = params.get('token') || '';
    }
    if (!token && typeof req.headers['sec-websocket-protocol'] === 'string') {
        const secWebSocketProtocol = req.headers['sec-websocket-protocol'];
        if (secWebSocketProtocol) {
            const protocols = secWebSocketProtocol.split(',');
            const length = protocols.length;
            for (let i = 0; i < length; i++) {
                let protocol = protocols[i].trim();
                if (protocol.indexOf('X-Authorization=') === 0) {
                    protocol = protocol.substring('X-Authorization='.length);
                    if (protocol) {
                        token = Buffer.from(protocol).toString('base64');
                        break;
                    }
                }
            }
        }
    }
    return { ip, token };
};
