"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryRxClient = void 0;
var tslib_1 = require("tslib");
var messages_1 = require("./messages");
var rxjs_1 = require("rxjs");
var TimedQueue_1 = require("../json-rx/TimedQueue");
var codec_1 = require("./codec");
var BinaryRxClient = (function () {
    function BinaryRxClient(_a) {
        var _this = this;
        var send = _a.send, _b = _a.bufferSize, bufferSize = _b === void 0 ? 100 : _b, _c = _a.bufferTime, bufferTime = _c === void 0 ? 10 : _c;
        this.id = 1;
        this.encoder = new codec_1.Encoder();
        this.observers = new Map();
        this.buffer = new TimedQueue_1.TimedQueue();
        this.buffer.itemLimit = bufferSize;
        this.buffer.timeLimit = bufferTime;
        this.buffer.onFlush = function (messages) {
            send(_this.encoder.encode(messages));
        };
    }
    BinaryRxClient.prototype.sendSubscribe = function (id, method, data) {
        var message = new messages_1.SubscribeMessage(id, method, data);
        this.buffer.push(message);
    };
    BinaryRxClient.prototype.sendUnsubscribe = function (id) {
        this.buffer.push(new messages_1.UnsubscribeMessage(id));
    };
    BinaryRxClient.prototype.onData = function (message) {
        var id = message.id, data = message.data;
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.next(data);
    };
    BinaryRxClient.prototype.onComplete = function (message) {
        var id = message.id, data = message.data;
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        if (data && data.byteLength)
            observer.observer.next(data);
        observer.observer.complete();
    };
    BinaryRxClient.prototype.onError = function (message) {
        var id = message.id, data = message.data;
        var observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        observer.observer.error(data);
    };
    BinaryRxClient.prototype.onArray = function (arr) {
        var messages = (0, codec_1.decodeFullMessages)(arr, 0);
        this.onMessages(messages);
    };
    BinaryRxClient.prototype.onMessages = function (messages) {
        var length = messages.length;
        for (var i = 0; i < length; i++) {
            var message = messages[i];
            if (message instanceof messages_1.CompleteMessage)
                this.onComplete(message);
            else if (message instanceof messages_1.DataMessage)
                this.onData(message);
            else if (message instanceof messages_1.ErrorMessage)
                this.onError(message);
        }
    };
    BinaryRxClient.prototype.call = function (method, data) {
        var _this = this;
        var id = this.id++;
        if (this.id >= 0xffff)
            this.id = 1;
        if (this.observers.has(id))
            return this.call(method, data);
        var observable = new rxjs_1.Observable(function (observer) {
            var entry = { observer: observer };
            _this.observers.set(id, entry);
            return function () {
                _this.observers.delete(id);
                if (!entry.completed)
                    _this.sendUnsubscribe(id);
            };
        });
        this.sendSubscribe(id, method, data);
        return observable;
    };
    BinaryRxClient.prototype.notify = function (method, data) {
        var message = new messages_1.NotificationMessage(method, data);
        this.buffer.push(message);
    };
    BinaryRxClient.prototype.stop = function () {
        var e_1, _a;
        this.buffer.onFlush = function (message) { };
        try {
            for (var _b = (0, tslib_1.__values)(this.observers.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var observer = _c.value.observer;
                observer.complete();
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    return BinaryRxClient;
}());
exports.BinaryRxClient = BinaryRxClient;
