"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringify = exports.parse = void 0;
var json_pack_1 = require("../json-pack");
var base64_1 = require("../util/base64");
var isUint8Array_1 = require("../util/isUint8Array");
var constants_1 = require("./constants");
var binUriStartLength = constants_1.binUriStart.length;
var msgPackUriStartLength = constants_1.msgPackUriStart.length;
var msgPackExtStartLength = constants_1.msgPackExtStart.length;
var minDataUri = Math.min(binUriStartLength, msgPackUriStartLength);
var parseExtDataUri = function (uri) {
    uri = uri.substring(msgPackExtStartLength);
    var commaIndex = uri.indexOf(',');
    if (commaIndex === -1)
        throw new Error('INVALID_EXT_DATA_URI');
    var typeString = uri.substring(0, commaIndex);
    var buf = (0, base64_1.fromBase64)(uri.substring(commaIndex + 1));
    return new json_pack_1.JsonPackExtension(Number(typeString), buf);
};
var unwrapBinary = function (value) {
    if (!value)
        return value;
    if (value instanceof Array) {
        var len = value.length;
        for (var i = 0; i < len; i++) {
            var item = value[i];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < minDataUri)
                        continue;
                    if (item.substring(0, binUriStartLength) === constants_1.binUriStart)
                        value[i] = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                    else if (item.substring(0, msgPackUriStartLength) === constants_1.msgPackUriStart)
                        value[i] = new json_pack_1.JsonPackValue((0, base64_1.fromBase64)(item.substring(msgPackUriStartLength)));
                    else if (item.substring(0, msgPackExtStartLength) === constants_1.msgPackExtStart)
                        value[i] = parseExtDataUri(item);
                }
            }
        }
        return value;
    }
    if (typeof value === 'object') {
        for (var key in value) {
            var item = value[key];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < minDataUri)
                        continue;
                    if (item.substring(0, binUriStartLength) === constants_1.binUriStart) {
                        var buf = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                        value[key] = buf;
                    }
                    else if (item.substring(0, msgPackUriStartLength) === constants_1.msgPackUriStart) {
                        value[key] = new json_pack_1.JsonPackValue((0, base64_1.fromBase64)(item.substring(msgPackUriStartLength)));
                    }
                    else if (item.substring(0, msgPackExtStartLength) === constants_1.msgPackExtStart)
                        value[key] = parseExtDataUri(item);
                }
            }
        }
        return value;
    }
    if (typeof value === 'string') {
        if (value.length < minDataUri)
            return value;
        if (value.substring(0, binUriStartLength) === constants_1.binUriStart)
            return (0, base64_1.fromBase64)(value.substring(binUriStartLength));
        if (value.substring(0, msgPackUriStartLength) === constants_1.msgPackUriStart)
            return new json_pack_1.JsonPackValue((0, base64_1.fromBase64)(value.substring(msgPackUriStartLength)));
        if (value.substring(0, msgPackExtStartLength) === constants_1.msgPackExtStart)
            return parseExtDataUri(value);
        else
            return value;
    }
    return value;
};
var parse = function (json) {
    var parsed = JSON.parse(json);
    return unwrapBinary(parsed);
};
exports.parse = parse;
var wrapBinary = function (value) {
    if (!value)
        return value;
    if ((0, isUint8Array_1.isUint8Array)(value))
        return constants_1.binUriStart + (0, base64_1.toBase64)(value);
    if (value instanceof Array) {
        var out = [];
        var len = value.length;
        for (var i = 0; i < len; i++) {
            var item = value[i];
            out.push(!item || typeof item !== 'object' ? item : wrapBinary(item));
        }
        return out;
    }
    if (value instanceof json_pack_1.JsonPackValue)
        return constants_1.msgPackUriStart + (0, base64_1.toBase64)(value.buf);
    if (value instanceof json_pack_1.JsonPackExtension)
        return constants_1.msgPackExtStart + value.type + ',' + (0, base64_1.toBase64)(value.buf);
    if (typeof value === 'object') {
        var out = {};
        for (var key in value) {
            var item = value[key];
            out[key] = !item || typeof item !== 'object' ? item : wrapBinary(item);
        }
        return out;
    }
    return value;
};
var stringify = function (value, replacer, space) {
    var wrapped = wrapBinary(value);
    return JSON.stringify(wrapped, replacer, space);
};
exports.stringify = stringify;
