"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Batch = void 0;
var Batch = (function () {
    function Batch(patches) {
        this.patches = patches;
    }
    Batch.prototype.getId = function () {
        if (!this.patches.length)
            return undefined;
        return this.patches[0].getId();
    };
    Batch.prototype.rebase = function (serverTime) {
        var id = this.getId();
        if (!id)
            throw new Error('BATCH_EMPTY');
        var transformHorizon = id.time;
        var patches = this.patches;
        var length = patches.length;
        var newPatches = [];
        for (var i = 0; i < length; i++) {
            var patch = patches[i];
            newPatches.push(patch.rebase(serverTime, transformHorizon));
            serverTime += patch.span();
        }
        return new Batch(newPatches);
    };
    Batch.prototype.clone = function () {
        return new Batch(this.patches.map(function (patch) { return patch.clone(); }));
    };
    return Batch;
}());
exports.Batch = Batch;
