"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PatchBuilder = void 0;
var tslib_1 = require("tslib");
var DeleteOperation_1 = require("./operations/DeleteOperation");
var constants_1 = require("./constants");
var InsertArrayElementsOperation_1 = require("./operations/InsertArrayElementsOperation");
var InsertBinaryDataOperation_1 = require("./operations/InsertBinaryDataOperation");
var InsertStringSubstringOperation_1 = require("./operations/InsertStringSubstringOperation");
var isUint8Array_1 = require("../util/isUint8Array");
var MakeArrayOperation_1 = require("./operations/MakeArrayOperation");
var MakeBinaryOperation_1 = require("./operations/MakeBinaryOperation");
var MakeConstantOperation_1 = require("./operations/MakeConstantOperation");
var MakeNumberOperation_1 = require("./operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("./operations/MakeObjectOperation");
var MakeStringOperation_1 = require("./operations/MakeStringOperation");
var MakeValueOperation_1 = require("./operations/MakeValueOperation");
var NoopOperation_1 = require("./operations/NoopOperation");
var Patch_1 = require("./Patch");
var SetNumberOperation_1 = require("./operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("./operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("./operations/SetRootOperation");
var SetValueOperation_1 = require("./operations/SetValueOperation");
var PatchBuilder = (function () {
    function PatchBuilder(clock) {
        this.clock = clock;
        this.patch = new Patch_1.Patch();
    }
    PatchBuilder.prototype.obj = function () {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeObjectOperation_1.MakeObjectOperation(id);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.arr = function () {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeArrayOperation_1.MakeArrayOperation(id);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.str = function () {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeStringOperation_1.MakeStringOperation(id);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.bin = function () {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeBinaryOperation_1.MakeBinaryOperation(id);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.num = function () {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeNumberOperation_1.MakeNumberOperation(id);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.const = function (value) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeConstantOperation_1.MakeConstantOperation(id, value);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.val = function (value) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new MakeValueOperation_1.MakeValueOperation(id, value);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.root = function (value) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new SetRootOperation_1.SetRootOperation(id, value);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.setKeys = function (obj, tuples) {
        this.pad();
        if (!tuples.length)
            throw new Error('EMPTY_TUPLES');
        var id = this.clock.tick(1);
        var op = new SetObjectKeysOperation_1.SetObjectKeysOperation(id, obj, tuples);
        var span = op.span();
        if (span > 1)
            this.clock.tick(span - 1);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.setNum = function (obj, value) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new SetNumberOperation_1.SetNumberOperation(id, obj, value);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.setVal = function (obj, value) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new SetValueOperation_1.SetValueOperation(id, obj, value);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.insStr = function (obj, after, substring) {
        this.pad();
        if (!substring.length)
            throw new Error('EMPTY_STRING');
        var id = this.clock.tick(1);
        var op = new InsertStringSubstringOperation_1.InsertStringSubstringOperation(id, obj, after, substring);
        var span = op.span();
        if (span > 1)
            this.clock.tick(span - 1);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.insBin = function (obj, after, data) {
        this.pad();
        if (!data.length)
            throw new Error('EMPTY_BINARY');
        var id = this.clock.tick(1);
        var op = new InsertBinaryDataOperation_1.InsertBinaryDataOperation(id, obj, after, data);
        var span = op.span();
        if (span > 1)
            this.clock.tick(span - 1);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.insArr = function (arr, after, elements) {
        this.pad();
        var id = this.clock.tick(1);
        var op = new InsertArrayElementsOperation_1.InsertArrayElementsOperation(id, arr, after, elements);
        var span = op.span();
        if (span > 1)
            this.clock.tick(span - 1);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.del = function (obj, start, span) {
        this.pad();
        var id = this.clock.tick(span);
        var op = new DeleteOperation_1.DeleteOperation(id, obj, start.interval(0, span));
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.noop = function (span) {
        this.pad();
        var id = this.clock.tick(span);
        var op = new NoopOperation_1.NoopOperation(id, span);
        this.patch.ops.push(op);
        return id;
    };
    PatchBuilder.prototype.jsonObj = function (json) {
        var e_1, _a;
        var obj = this.obj();
        var keys = Object.keys(json);
        if (keys.length) {
            var tuples = [];
            try {
                for (var keys_1 = (0, tslib_1.__values)(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                    var k = keys_1_1.value;
                    tuples.push([k, this.json(json[k])]);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            this.setKeys(obj, tuples);
        }
        return obj;
    };
    PatchBuilder.prototype.jsonArr = function (json) {
        var e_2, _a;
        var arr = this.arr();
        if (json.length) {
            var values = [];
            try {
                for (var json_1 = (0, tslib_1.__values)(json), json_1_1 = json_1.next(); !json_1_1.done; json_1_1 = json_1.next()) {
                    var el = json_1_1.value;
                    values.push(this.json(el));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (json_1_1 && !json_1_1.done && (_a = json_1.return)) _a.call(json_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
            this.insArr(arr, arr, values);
        }
        return arr;
    };
    PatchBuilder.prototype.jsonStr = function (json) {
        var str = this.str();
        if (json)
            this.insStr(str, str, json);
        return str;
    };
    PatchBuilder.prototype.jsonBin = function (json) {
        var bin = this.bin();
        if (json.length)
            this.insBin(bin, bin, json);
        return bin;
    };
    PatchBuilder.prototype.jsonVal = function (json) {
        return this.val(json);
    };
    PatchBuilder.prototype.json = function (json) {
        switch (json) {
            case null:
                return constants_1.NULL_ID;
            case true:
                return constants_1.TRUE_ID;
            case false:
                return constants_1.FALSE_ID;
        }
        if (Array.isArray(json))
            return this.jsonArr(json);
        if ((0, isUint8Array_1.isUint8Array)(json))
            return this.jsonBin(json);
        switch (typeof json) {
            case 'object':
                return this.jsonObj(json);
            case 'string':
                return this.jsonStr(json);
            case 'number':
                return this.jsonVal(json);
        }
        return constants_1.UNDEFINED_ID;
    };
    PatchBuilder.prototype.pad = function () {
        var nextTime = this.patch.nextTime();
        if (!nextTime)
            return;
        var drift = this.clock.time - nextTime;
        if (drift > 0) {
            var id = this.clock.stamp(this.clock.getSessionId(), nextTime);
            var padding = new NoopOperation_1.NoopOperation(id, drift);
            this.patch.ops.push(padding);
        }
    };
    return PatchBuilder;
}());
exports.PatchBuilder = PatchBuilder;
