"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicalVectorClock = exports.LogicalClock = exports.LogicalTimespan = exports.LogicalTimestamp = void 0;
var tslib_1 = require("tslib");
var server_1 = require("./server");
var LogicalTimestamp = (function (_super) {
    (0, tslib_1.__extends)(LogicalTimestamp, _super);
    function LogicalTimestamp(sessionId, time) {
        var _this = _super.call(this, time) || this;
        _this.sessionId = sessionId;
        _this.time = time;
        return _this;
    }
    LogicalTimestamp.prototype.getSessionId = function () {
        return this.sessionId;
    };
    LogicalTimestamp.prototype.stamp = function (sessionId, time) {
        return new LogicalTimestamp(sessionId, time);
    };
    LogicalTimestamp.prototype.tick = function (cycles) {
        return new LogicalTimestamp(this.getSessionId(), this.time + cycles);
    };
    LogicalTimestamp.prototype.interval = function (cycles, span) {
        return new LogicalTimespan(this.getSessionId(), this.time + cycles, span);
    };
    LogicalTimestamp.prototype.clock = function () {
        return new LogicalClock(this.getSessionId(), this.time);
    };
    return LogicalTimestamp;
}(server_1.ServerTimestamp));
exports.LogicalTimestamp = LogicalTimestamp;
var LogicalTimespan = (function (_super) {
    (0, tslib_1.__extends)(LogicalTimespan, _super);
    function LogicalTimespan(sessionId, time, span) {
        var _this = _super.call(this, sessionId, time) || this;
        _this.span = span;
        return _this;
    }
    return LogicalTimespan;
}(LogicalTimestamp));
exports.LogicalTimespan = LogicalTimespan;
var LogicalClock = (function (_super) {
    (0, tslib_1.__extends)(LogicalClock, _super);
    function LogicalClock() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LogicalClock.prototype.tick = function (cycles) {
        var timestamp = new LogicalTimestamp(this.sessionId, this.time);
        this.time += cycles;
        return timestamp;
    };
    return LogicalClock;
}(LogicalTimestamp));
exports.LogicalClock = LogicalClock;
var LogicalVectorClock = (function (_super) {
    (0, tslib_1.__extends)(LogicalVectorClock, _super);
    function LogicalVectorClock() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.clocks = new Map();
        return _this;
    }
    LogicalVectorClock.prototype.observe = function (ts, span) {
        var time = ts.time + span - 1;
        var clock = this.clocks.get(ts.getSessionId());
        if (!clock)
            this.clocks.set(ts.getSessionId(), ts.tick(span - 1));
        else if (time > clock.time)
            clock.time = ts.time;
        if (time >= this.time)
            this.time = time + 1;
    };
    LogicalVectorClock.prototype.clone = function () {
        return this.fork(this.getSessionId());
    };
    LogicalVectorClock.prototype.fork = function (sessionId) {
        var e_1, _a;
        var clock = new LogicalVectorClock(sessionId, this.time);
        try {
            for (var _b = (0, tslib_1.__values)(this.clocks.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var ts = _c.value;
                clock.observe(ts.tick(0), 1);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return clock;
    };
    return LogicalVectorClock;
}(LogicalClock));
exports.LogicalVectorClock = LogicalVectorClock;
