"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var CrdtEncoder_1 = require("../../util/binary/CrdtEncoder");
var DeleteOperation_1 = require("../../operations/DeleteOperation");
var InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
var InsertBinaryDataOperation_1 = require("../../operations/InsertBinaryDataOperation");
var InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
var MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
var MakeBinaryOperation_1 = require("../../operations/MakeBinaryOperation");
var MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
var MakeStringOperation_1 = require("../../operations/MakeStringOperation");
var MakeValueOperation_1 = require("../../operations/MakeValueOperation");
var NoopOperation_1 = require("../../operations/NoopOperation");
var SetNumberOperation_1 = require("../../operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("../../operations/SetRootOperation");
var SetValueOperation_1 = require("../../operations/SetValueOperation");
var Encoder = (function (_super) {
    (0, tslib_1.__extends)(Encoder, _super);
    function Encoder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Encoder.prototype.encode = function (patch) {
        this.reset();
        var id = (this.patchId = patch.getId());
        var isServerClock = id.getSessionId() === 1;
        if (isServerClock) {
            this.b1vuint56(true, id.time);
        }
        else {
            this.b1vuint56(false, id.getSessionId());
            this.vuint57(id.time);
        }
        this.encodeOperations(patch);
        return this.flush();
    };
    Encoder.prototype.encodeOperations = function (patch) {
        var ops = patch.ops;
        for (var i = 0; i < ops.length; i++) {
            var op = ops[i];
            this.encodeOperation(op);
        }
    };
    Encoder.prototype.encodeId = function (id) {
        var sessionId = id.getSessionId();
        var time = id.time;
        if (sessionId === 1) {
            this.b1vuint56(true, id.time);
        }
        else {
            var patchId = this.patchId;
            if (sessionId === patchId.getSessionId() && time >= patchId.time) {
                this.b1vuint56(false, 1);
                this.vuint57(time - patchId.time);
            }
            else {
                this.b1vuint56(false, sessionId);
                this.vuint57(time);
            }
        }
    };
    Encoder.prototype.encodeOperation = function (op) {
        var e_1, _a;
        if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
            this.u8(0);
            return;
        }
        else if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
            this.u8(1);
            return;
        }
        else if (op instanceof MakeStringOperation_1.MakeStringOperation) {
            this.u8(2);
            return;
        }
        else if (op instanceof MakeValueOperation_1.MakeValueOperation) {
            this.u8(14);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
            this.u8(3);
            return;
        }
        else if (op instanceof SetRootOperation_1.SetRootOperation) {
            this.u8(4);
            this.encodeId(op.value);
            return;
        }
        else if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
            this.u8(5);
            this.encodeId(op.object);
            this.vuint57(op.tuples.length);
            try {
                for (var _b = (0, tslib_1.__values)(op.tuples), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = (0, tslib_1.__read)(_c.value, 2), key = _d[0], value = _d[1];
                    this.encodeId(value);
                    this.encodeString(key);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return;
        }
        else if (op instanceof SetValueOperation_1.SetValueOperation) {
            this.u8(15);
            this.encodeId(op.obj);
            this.encodeAny(op.value);
            return;
        }
        else if (op instanceof SetNumberOperation_1.SetNumberOperation) {
            this.u8(6);
            this.encodeId(op.num);
            this.encodeNumber(op.value);
            return;
        }
        else if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
            this.u8(7);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.encodeString(op.substring);
            return;
        }
        else if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
            var arr = op.arr, after = op.after, elements = op.elements;
            var length_1 = elements.length;
            this.u8(8);
            this.encodeId(arr);
            this.encodeId(after);
            this.vuint57(length_1);
            for (var i = 0; i < length_1; i++)
                this.encodeId(elements[i]);
            return;
        }
        else if (op instanceof DeleteOperation_1.DeleteOperation) {
            var obj = op.obj, after = op.after;
            var length_2 = after.span;
            if (length_2 > 1) {
                this.u8(10);
                this.encodeId(obj);
                this.encodeId(after);
                this.vuint57(length_2);
                return;
            }
            this.u8(9);
            this.encodeId(obj);
            this.encodeId(after);
            return;
        }
        else if (op instanceof NoopOperation_1.NoopOperation) {
            var length_3 = op.length;
            if (length_3 > 1) {
                this.u8(12);
                this.vuint57(length_3);
                return;
            }
            this.u8(11);
            return;
        }
        else if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation) {
            this.u8(16);
            return;
        }
        else if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation) {
            var buf = op.data;
            var length_4 = buf.length;
            this.u8(17);
            this.encodeId(op.obj);
            this.encodeId(op.after);
            this.vuint57(length_4);
            this.buf(buf, length_4);
            return;
        }
        throw new Error('UNKNOWN_OP');
    };
    return Encoder;
}(CrdtEncoder_1.CrdtEncoder));
exports.Encoder = Encoder;
