"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = void 0;
var decode_1 = require("../../../util/base64/decode");
var clock_1 = require("../../clock");
var PatchBuilder_1 = require("../../PatchBuilder");
var decode = function (data) {
    var x = data[0];
    var clock = Array.isArray(x) ? new clock_1.LogicalVectorClock(x[0], x[1]) : new clock_1.ServerVectorClock(x);
    var sessionId = clock.getSessionId();
    var time = clock.time;
    var builder = new PatchBuilder_1.PatchBuilder(clock);
    var length = data.length;
    var i = 1;
    var decodeTimestamp = function () {
        var x = data[i++];
        if (Array.isArray(x))
            return new clock_1.LogicalTimestamp(x[0], x[1]);
        else if (x < 0)
            return new clock_1.LogicalTimestamp(sessionId, time - x - 1);
        else
            return new clock_1.ServerTimestamp(x);
    };
    while (i < length) {
        switch (data[i++]) {
            case 0: {
                builder.obj();
                break;
            }
            case 1: {
                builder.arr();
                break;
            }
            case 2: {
                builder.str();
                break;
            }
            case 16: {
                builder.bin();
                break;
            }
            case 3: {
                builder.num();
                break;
            }
            case 4: {
                builder.root(decodeTimestamp());
                break;
            }
            case 5: {
                var length_1 = data[i++];
                var obj = decodeTimestamp();
                var tuples = [];
                for (var j = 0; j < length_1; j++) {
                    var key = data[i++];
                    tuples.push([key, decodeTimestamp()]);
                }
                builder.setKeys(obj, tuples);
                break;
            }
            case 6: {
                var value = data[i++];
                builder.setNum(decodeTimestamp(), value);
                break;
            }
            case 7: {
                var value = data[i++];
                builder.insStr(decodeTimestamp(), decodeTimestamp(), value);
                break;
            }
            case 17: {
                var value = data[i++];
                builder.insBin(decodeTimestamp(), decodeTimestamp(), (0, decode_1.fromBase64)(value));
                break;
            }
            case 8: {
                var length_2 = data[i++];
                var arr = decodeTimestamp();
                var after = decodeTimestamp();
                var values = [];
                for (var j = 0; j < length_2; j++)
                    values.push(decodeTimestamp());
                builder.insArr(arr, after, values);
                break;
            }
            case 9: {
                var obj = decodeTimestamp();
                var after = decodeTimestamp();
                builder.del(obj, after, 1);
                break;
            }
            case 10: {
                var span = data[i++];
                var obj = decodeTimestamp();
                var after = decodeTimestamp();
                builder.del(obj, after, span);
                break;
            }
            case 11: {
                builder.noop(1);
                break;
            }
            case 12: {
                builder.noop(data[i++]);
                break;
            }
            case 13: {
                builder.const(data[i++]);
                break;
            }
            case 14: {
                builder.val(data[i++]);
                break;
            }
            case 15: {
                var obj = decodeTimestamp();
                var value = data[i++];
                builder.setVal(obj, value);
                break;
            }
        }
    }
    return builder.patch;
};
exports.decode = decode;
