"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = void 0;
var tslib_1 = require("tslib");
var DeleteOperation_1 = require("../../operations/DeleteOperation");
var InsertArrayElementsOperation_1 = require("../../operations/InsertArrayElementsOperation");
var InsertBinaryDataOperation_1 = require("../../operations/InsertBinaryDataOperation");
var InsertStringSubstringOperation_1 = require("../../operations/InsertStringSubstringOperation");
var MakeArrayOperation_1 = require("../../operations/MakeArrayOperation");
var MakeBinaryOperation_1 = require("../../operations/MakeBinaryOperation");
var MakeConstantOperation_1 = require("../../operations/MakeConstantOperation");
var MakeNumberOperation_1 = require("../../operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("../../operations/MakeObjectOperation");
var MakeStringOperation_1 = require("../../operations/MakeStringOperation");
var MakeValueOperation_1 = require("../../operations/MakeValueOperation");
var NoopOperation_1 = require("../../operations/NoopOperation");
var SetNumberOperation_1 = require("../../operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("../../operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("../../operations/SetRootOperation");
var SetValueOperation_1 = require("../../operations/SetValueOperation");
var encode_1 = require("../../../util/base64/encode");
var encodeTimestamp = function (ts) {
    return ts.getSessionId() === 1 ? ts.time : [ts.getSessionId(), ts.time];
};
var encode = function (patch) {
    var e_1, _a;
    var id = patch.getId();
    if (!id)
        throw new Error('PATCH_EMPTY');
    var ops = [];
    var res = {
        id: encodeTimestamp(id),
        ops: ops,
    };
    try {
        for (var _b = (0, tslib_1.__values)(patch.ops), _c = _b.next(); !_c.done; _c = _b.next()) {
            var op = _c.value;
            if (op instanceof MakeObjectOperation_1.MakeObjectOperation) {
                ops.push({ op: 'obj' });
                continue;
            }
            if (op instanceof MakeArrayOperation_1.MakeArrayOperation) {
                ops.push({ op: 'arr' });
                continue;
            }
            if (op instanceof MakeStringOperation_1.MakeStringOperation) {
                ops.push({ op: 'str' });
                continue;
            }
            if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation) {
                ops.push({ op: 'bin' });
                continue;
            }
            if (op instanceof MakeNumberOperation_1.MakeNumberOperation) {
                ops.push({ op: 'num' });
                continue;
            }
            if (op instanceof MakeValueOperation_1.MakeValueOperation) {
                ops.push({ op: 'val', value: op.value });
                continue;
            }
            if (op instanceof MakeConstantOperation_1.MakeConstantOperation) {
                ops.push({ op: 'const', value: op.value });
                continue;
            }
            if (op instanceof SetRootOperation_1.SetRootOperation) {
                ops.push({
                    op: 'root',
                    value: encodeTimestamp(op.value),
                });
                continue;
            }
            if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation) {
                ops.push({
                    op: 'obj_set',
                    obj: encodeTimestamp(op.object),
                    tuples: op.tuples.map(function (_a) {
                        var _b = (0, tslib_1.__read)(_a, 2), key = _b[0], value = _b[1];
                        return [key, encodeTimestamp(value)];
                    }),
                });
                continue;
            }
            if (op instanceof SetNumberOperation_1.SetNumberOperation) {
                ops.push({
                    op: 'num_set',
                    after: encodeTimestamp(op.num),
                    value: op.value,
                });
                continue;
            }
            if (op instanceof SetValueOperation_1.SetValueOperation) {
                ops.push({
                    op: 'val_set',
                    obj: encodeTimestamp(op.obj),
                    value: op.value,
                });
                continue;
            }
            if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation) {
                ops.push({
                    op: 'str_ins',
                    obj: encodeTimestamp(op.obj),
                    after: encodeTimestamp(op.after),
                    value: op.substring,
                });
                continue;
            }
            if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation) {
                ops.push({
                    op: 'bin_ins',
                    obj: encodeTimestamp(op.obj),
                    after: encodeTimestamp(op.after),
                    value: (0, encode_1.toBase64)(op.data),
                });
                continue;
            }
            if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation) {
                ops.push({
                    op: 'arr_ins',
                    obj: encodeTimestamp(op.arr),
                    after: encodeTimestamp(op.after),
                    values: op.elements.map(encodeTimestamp),
                });
                continue;
            }
            if (op instanceof DeleteOperation_1.DeleteOperation) {
                var encoded = {
                    op: 'del',
                    obj: encodeTimestamp(op.obj),
                    after: encodeTimestamp(op.after),
                };
                var span = op.span();
                if (span > 1)
                    encoded.len = span;
                ops.push(encoded);
                continue;
            }
            if (op instanceof NoopOperation_1.NoopOperation) {
                var encoded = {
                    op: 'noop',
                };
                var length_1 = op.length;
                if (length_1 > 1)
                    encoded.len = length_1;
                ops.push(encoded);
                continue;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return res;
};
exports.encode = encode;
