"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
var tslib_1 = require("tslib");
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryChunk_1 = require("../../types/rga-binary/BinaryChunk");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ClockDecoder_1 = require("../../../json-crdt-patch/codec/clock/ClockDecoder");
var ConstantType_1 = require("../../types/const/ConstantType");
var CrdtDecoder_1 = require("../../../json-crdt-patch/util/binary/CrdtDecoder");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var constants_1 = require("../../../json-crdt-patch/constants");
var constants_2 = require("../../constants");
var clock_1 = require("../../../json-crdt-patch/clock");
var model_1 = require("../../model");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Decoder = (function (_super) {
    (0, tslib_1.__extends)(Decoder, _super);
    function Decoder() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.time = -1;
        return _this;
    }
    Decoder.prototype.decode = function (data) {
        delete this.clockDecoder;
        this.time = -1;
        this.reset(data);
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), isServerTime = _a[0], x = _a[1];
        if (isServerTime) {
            this.time = x;
            this.doc = model_1.Model.withServerClock(x);
        }
        else {
            this.decodeClockTable(x);
            this.doc = model_1.Model.withLogicalClock(this.clockDecoder.clock);
        }
        this.literals = this.val();
        if (!(this.literals instanceof Array))
            throw new Error('INVALID_LITERALS');
        this.decodeRoot();
        delete this.clockDecoder;
        delete this.literals;
        return this.doc;
    };
    Decoder.prototype.decodeClockTable = function (length) {
        var _a = (0, tslib_1.__read)(this.uint53vuint39(), 2), sessionId = _a[0], time = _a[1];
        this.clockDecoder = new ClockDecoder_1.ClockDecoder(sessionId, 0);
        this.clockDecoder.pushTuple(sessionId, time);
        for (var i = 1; i < length; i++) {
            var ts = this.uint53vuint39();
            this.clockDecoder.pushTuple(ts[0], ts[1]);
        }
    };
    Decoder.prototype.ts = function () {
        if (this.time < 0) {
            var _a = (0, tslib_1.__read)(this.id(), 2), sessionIndex = _a[0], timeDiff = _a[1];
            return this.clockDecoder.decodeId(sessionIndex, timeDiff);
        }
        else {
            var _b = (0, tslib_1.__read)(this.b1vuint56(), 2), isServerTime = _b[0], x = _b[1];
            if (isServerTime) {
                return new clock_1.ServerTimestamp(this.time - x);
            }
            else {
                var sessionId = x;
                var time = this.vuint57();
                if (sessionId === 0) {
                    switch (time) {
                        case 1:
                            return constants_1.NULL_ID;
                        case 2:
                            return constants_1.TRUE_ID;
                        case 3:
                            return constants_1.FALSE_ID;
                        case 4:
                            return constants_1.UNDEFINED_ID;
                    }
                }
                return new clock_1.LogicalTimestamp(sessionId, time);
            }
        }
    };
    Decoder.prototype.decodeRoot = function () {
        var doc = this.doc;
        var id = this.ts();
        var node = this.x >= this.uint8.byteLength ? null : this.decodeNode();
        doc.root = new DocRootType_1.DocRootType(doc, id, node);
    };
    Decoder.prototype.decodeNode = function () {
        var byte = this.u8();
        if (byte < 128)
            return this.createConst(byte);
        else if (byte <= 143)
            return this.decodeObj(byte & 15);
        else if (byte <= 159)
            return this.decodeArr(byte & 15);
        else if (byte <= 191)
            return this.decodeStr(byte & 31);
        else if (byte >= 224)
            return this.createConst(byte - 0x100);
        else {
            switch (byte) {
                case 0xc0:
                    return constants_2.NULL;
                case 0xc1:
                    return constants_2.UNDEFINED;
                case 0xc2:
                    return constants_2.FALSE;
                case 0xc3:
                    return constants_2.TRUE;
                case 0xc4:
                    return this.decodeBin(this.u8());
                case 0xc5:
                    return this.decodeBin(this.u16());
                case 0xc6:
                    return this.decodeBin(this.u32());
                case 0xca:
                    return this.createConst(this.f32());
                case 0xcb:
                    return this.createConst(this.f64());
                case 0xcc:
                    return this.createConst(this.u8());
                case 0xcd:
                    return this.createConst(this.u16());
                case 0xce:
                    return this.createConst(this.u32());
                case 0xcf:
                    return this.createConst(this.u32() * 4294967296 + this.u32());
                case 0xd0:
                    return this.createConst(this.i8());
                case 0xd1:
                    return this.createConst(this.i16());
                case 0xd2:
                    return this.createConst(this.i32());
                case 0xd3:
                    return this.createConst(this.i32() * 4294967296 + this.i32());
                case 0xd4:
                    return this.createConst(this.val());
                case 0xd5:
                    return this.decodeVal();
                case 0xd6:
                    return this.decodeValLiteral();
                case 0xde:
                    return this.decodeObj(this.u16());
                case 0xdf:
                    return this.decodeObj(this.u32());
                case 0xdc:
                    return this.decodeArr(this.u16());
                case 0xdd:
                    return this.decodeArr(this.u32());
                case 0xd9:
                    return this.decodeStr(this.u8());
                case 0xda:
                    return this.decodeStr(this.u16());
                case 0xdb:
                    return this.decodeStr(this.u32());
            }
        }
        return constants_2.NULL;
    };
    Decoder.prototype.createConst = function (value) {
        return new ConstantType_1.ConstantType(constants_1.ORIGIN, value);
    };
    Decoder.prototype.decodeObj = function (length) {
        var id = this.ts();
        var obj = new ObjectType_1.ObjectType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeObjChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeObjChunk = function (obj) {
        var id = this.ts();
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), isLiteralIndex = _a[0], x = _a[1];
        var key;
        if (isLiteralIndex) {
            key = this.literals[x];
        }
        else {
            key = this.str(x);
        }
        var node = this.decodeNode();
        var chunk = new ObjectChunk_1.ObjectChunk(id, node);
        obj.putChunk(key, chunk);
    };
    Decoder.prototype.decodeArr = function (length) {
        var id = this.ts();
        var obj = new ArrayType_1.ArrayType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeArrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeArrChunk = function (obj) {
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        var id = this.ts();
        if (deleted) {
            var chunk = new ArrayChunk_1.ArrayChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            var nodes = [];
            for (var i = 0; i < length; i++)
                nodes.push(this.decodeNode());
            var chunk = new ArrayChunk_1.ArrayChunk(id, nodes);
            obj.append(chunk);
        }
    };
    Decoder.prototype.decodeStr = function (length) {
        var id = this.ts();
        var obj = new StringType_1.StringType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeStrChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeStrChunk = function (obj) {
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        var id = this.ts();
        if (deleted) {
            var chunk = new StringChunk_1.StringChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            var text = void 0;
            if (length === 0) {
                var literalIndex = this.vuint57();
                text = this.literals[literalIndex];
            }
            else {
                text = this.str(length);
            }
            var chunk = new StringChunk_1.StringChunk(id, text);
            obj.append(chunk);
        }
    };
    Decoder.prototype.decodeBin = function (length) {
        var id = this.ts();
        var obj = new BinaryType_1.BinaryType(this.doc, id);
        for (var i = 0; i < length; i++)
            this.decodeBinChunk(obj);
        this.doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeBinChunk = function (obj) {
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        var id = this.ts();
        if (deleted) {
            var chunk = new BinaryChunk_1.BinaryChunk(id, undefined);
            chunk.deleted = length;
            obj.append(chunk);
        }
        else {
            var text = this.bin(length);
            var chunk = new BinaryChunk_1.BinaryChunk(id, text);
            obj.append(chunk);
        }
    };
    Decoder.prototype.decodeVal = function () {
        var id = this.ts();
        var writeId = this.ts();
        var value = this.val();
        var obj = new ValueType_1.ValueType(id, writeId, value);
        this.doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeValLiteral = function () {
        var id = this.ts();
        var writeId = this.ts();
        var literalIndex = this.vuint57();
        var value = this.literals[literalIndex];
        var obj = new ValueType_1.ValueType(id, writeId, value);
        this.doc.nodes.index(obj);
        return obj;
    };
    return Decoder;
}(CrdtDecoder_1.CrdtDecoder));
exports.Decoder = Decoder;
