"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ClockEncoder_1 = require("../../../json-crdt-patch/codec/clock/ClockEncoder");
var ConstantType_1 = require("../../types/const/ConstantType");
var CrdtEncoder_1 = require("../../../json-crdt-patch/util/binary/CrdtEncoder");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var StringType_1 = require("../../types/rga-string/StringType");
var utf8_1 = require("../../../util/utf8");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Encoder = (function (_super) {
    (0, tslib_1.__extends)(Encoder, _super);
    function Encoder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Encoder.prototype.encode = function (model) {
        delete this.clockEncoder;
        delete this.time;
        this.literals = new Map();
        this.reset();
        var clock = model.clock;
        var isServerClock = clock.getSessionId() === 1;
        if (isServerClock) {
            this.time = clock.time;
            this.b1vuint56(true, clock.time);
        }
        else {
            model.advanceClocks();
            this.clockEncoder = new ClockEncoder_1.ClockEncoder(model.clock);
        }
        this.encodeLiteralsTable(model);
        this.encodeRoot(model.root);
        var data = this.flush();
        if (isServerClock)
            return data;
        this.encodeClockTable(data);
        delete this.clockEncoder;
        delete this.literals;
        return this.flush();
    };
    Encoder.prototype.encodeLiteralsTable = function (model) {
        var e_1, _a, e_2, _b, e_3, _c, e_4, _d;
        var literalFrequencies = new Map();
        try {
            for (var _e = (0, tslib_1.__values)(model.nodes.iterate()), _f = _e.next(); !_f.done; _f = _e.next()) {
                var node = _f.value;
                if (node instanceof ObjectType_1.ObjectType) {
                    try {
                        for (var _g = (e_2 = void 0, (0, tslib_1.__values)(node.latest.keys())), _h = _g.next(); !_h.done; _h = _g.next()) {
                            var key = _h.value;
                            var count = literalFrequencies.get(key) || 0;
                            literalFrequencies.set(key, count + 1);
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_h && !_h.done && (_b = _g.return)) _b.call(_g);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                }
                else if (node instanceof StringType_1.StringType) {
                    try {
                        for (var _j = (e_3 = void 0, (0, tslib_1.__values)(node.chunks())), _k = _j.next(); !_k.done; _k = _j.next()) {
                            var chunk = _k.value;
                            var str = chunk.str;
                            if (str) {
                                var count = literalFrequencies.get(str) || 0;
                                literalFrequencies.set(str, count + 1);
                            }
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (_k && !_k.done && (_c = _j.return)) _c.call(_j);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                }
                else if (node instanceof ValueType_1.ValueType) {
                    var value = node.value;
                    switch (typeof value) {
                        case 'number':
                        case 'string': {
                            var count = literalFrequencies.get(value) || 0;
                            literalFrequencies.set(value, count + 1);
                        }
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_a = _e.return)) _a.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var literals = [];
        try {
            for (var _l = (0, tslib_1.__values)(literalFrequencies.entries()), _m = _l.next(); !_m.done; _m = _l.next()) {
                var _o = (0, tslib_1.__read)(_m.value, 2), literal = _o[0], frequency = _o[1];
                if (frequency > 1)
                    literals.push(literal);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_m && !_m.done && (_d = _l.return)) _d.call(_l);
            }
            finally { if (e_4) throw e_4.error; }
        }
        this.encodeArray(literals);
        for (var i = 0; i < literals.length; i++)
            this.literals.set(literals[i], i);
    };
    Encoder.prototype.encodeClockTable = function (data) {
        var e_5, _a;
        var clockEncoder = this.clockEncoder;
        var length = clockEncoder.table.size;
        var dataSize = data.byteLength;
        this.uint8 = new Uint8Array(8 + 12 * length + dataSize);
        this.view = new DataView(this.uint8.buffer, this.uint8.byteOffset, this.uint8.byteLength);
        this.offset = 0;
        this.b1vuint56(false, length);
        try {
            for (var _b = (0, tslib_1.__values)(clockEncoder.table.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sid = _c.value;
                var ts = clockEncoder.clock.clocks.get(sid);
                if (ts)
                    this.uint53vuint39(sid, ts.time);
                else if (sid === clockEncoder.clock.getSessionId())
                    this.uint53vuint39(sid, 0);
                else {
                }
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
        this.buf(data, dataSize);
    };
    Encoder.prototype.ts = function (ts) {
        if (this.clockEncoder) {
            var relativeId = this.clockEncoder.append(ts);
            this.id(relativeId.sessionIndex, relativeId.timeDiff);
        }
        else {
            var sessionId = ts.getSessionId();
            if (sessionId === 1) {
                this.b1vuint56(true, this.time - ts.time);
            }
            else {
                this.b1vuint56(false, sessionId);
                this.vuint57(ts.time);
            }
        }
    };
    Encoder.prototype.encodeRoot = function (root) {
        this.ts(root.id);
        if (root.node)
            this.encodeNode(root.node);
    };
    Encoder.prototype.encodeNode = function (node) {
        if (node instanceof ObjectType_1.ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType_1.ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType_1.StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType_1.ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType_1.ConstantType)
            return this.encodeConst(node);
        else if (node instanceof BinaryType_1.BinaryType)
            return this.encodeBin(node);
        throw new Error('UNKNOWN_NODE');
    };
    Encoder.prototype.encodeObj = function (obj) {
        var e_6, _a;
        var length = obj.latest.size;
        this.encodeObjectHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = (0, tslib_1.__read)(_c.value, 2), key = _d[0], chunk = _d[1];
                this.encodeObjChunk(key, chunk);
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
    };
    Encoder.prototype.encodeObjChunk = function (key, chunk) {
        this.ts(chunk.id);
        var length = (0, utf8_1.utf8Count)(key);
        var indexInLiteralsTable = this.literals.get(key);
        if (indexInLiteralsTable !== undefined) {
            this.b1vuint56(true, this.literals.get(key));
        }
        else {
            this.b1vuint56(false, length);
            this.encodeUtf8(key, length);
        }
        this.encodeNode(chunk.node);
    };
    Encoder.prototype.encodeArr = function (obj) {
        var e_7, _a;
        var length = obj.size();
        this.encodeArrayHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeArrChunk(chunk);
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_7) throw e_7.error; }
        }
    };
    Encoder.prototype.encodeArrChunk = function (chunk) {
        if (chunk.deleted) {
            this.b1vuint56(true, chunk.deleted);
            this.ts(chunk.id);
        }
        else {
            var nodes = chunk.nodes;
            var length_1 = nodes.length;
            this.b1vuint56(false, length_1);
            this.ts(chunk.id);
            for (var i = 0; i < length_1; i++)
                this.encodeNode(nodes[i]);
        }
    };
    Encoder.prototype.encodeStr = function (obj) {
        var e_8, _a;
        var length = obj.size();
        this.encodeStringHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeStrChunk(chunk);
            }
        }
        catch (e_8_1) { e_8 = { error: e_8_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_8) throw e_8.error; }
        }
    };
    Encoder.prototype.encodeStrChunk = function (chunk) {
        if (chunk.deleted) {
            this.b1vuint56(true, chunk.deleted);
            this.ts(chunk.id);
        }
        else {
            var text = chunk.str;
            var indexInLiteralsTable = this.literals.get(text);
            var useLiteralsTable = indexInLiteralsTable !== undefined;
            var length_2;
            this.b1vuint56(false, useLiteralsTable ? 0 : (length_2 = (0, utf8_1.utf8Count)(text)));
            this.ts(chunk.id);
            if (useLiteralsTable) {
                this.vuint57(indexInLiteralsTable);
            }
            else {
                this.encodeUtf8(text, length_2);
            }
        }
    };
    Encoder.prototype.encodeBin = function (obj) {
        var e_9, _a;
        var length = obj.size();
        this.encodeBinaryHeader(length);
        this.ts(obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeBinChunk(chunk);
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_9) throw e_9.error; }
        }
    };
    Encoder.prototype.encodeBinChunk = function (chunk) {
        if (chunk.deleted) {
            this.b1vuint56(true, chunk.deleted);
            this.ts(chunk.id);
        }
        else {
            var data = chunk.buf;
            var length_3 = data.byteLength;
            this.b1vuint56(false, length_3);
            this.ts(chunk.id);
            this.buf(data, length_3);
        }
    };
    Encoder.prototype.encodeConst = function (obj) {
        var value = obj.value;
        switch (value) {
            case null:
            case false:
            case true:
                this.encodeAny(value);
                break;
            case undefined:
                this.u8(0xc1);
                break;
            default: {
                if (typeof value === 'number')
                    this.encodeNumber(value);
                else {
                    this.u8(0xd4);
                    this.encodeAny(value);
                }
            }
        }
    };
    Encoder.prototype.encodeVal = function (obj) {
        var value = obj.value;
        var indexInLiteralsTable = this.literals.get(obj.value);
        if (indexInLiteralsTable !== undefined && (typeof value === 'number' || typeof value === 'string')) {
            this.u8(0xd6);
            this.ts(obj.id);
            this.ts(obj.writeId);
            this.vuint57(indexInLiteralsTable);
        }
        else {
            this.u8(0xd5);
            this.ts(obj.id);
            this.ts(obj.writeId);
            this.encodeAny(obj.value);
        }
    };
    Encoder.prototype.encodeUtf8 = function (str, byteLength) {
        this.ensureCapacity(byteLength);
        var strLength = str.length;
        var uint8 = this.uint8;
        var offset = this.offset;
        var pos = 0;
        while (pos < strLength) {
            var value = str.charCodeAt(pos++);
            if ((value & 0xffffff80) === 0) {
                uint8[offset++] = value;
                continue;
            }
            else if ((value & 0xfffff800) === 0) {
                uint8[offset++] = ((value >> 6) & 0x1f) | 0xc0;
            }
            else {
                if (value >= 0xd800 && value <= 0xdbff) {
                    if (pos < strLength) {
                        var extra = str.charCodeAt(pos);
                        if ((extra & 0xfc00) === 0xdc00) {
                            pos++;
                            value = ((value & 0x3ff) << 10) + (extra & 0x3ff) + 0x10000;
                        }
                    }
                }
                if ((value & 0xffff0000) === 0) {
                    uint8[offset++] = ((value >> 12) & 0x0f) | 0xe0;
                    uint8[offset++] = ((value >> 6) & 0x3f) | 0x80;
                }
                else {
                    uint8[offset++] = ((value >> 18) & 0x07) | 0xf0;
                    uint8[offset++] = ((value >> 12) & 0x3f) | 0x80;
                    uint8[offset++] = ((value >> 6) & 0x3f) | 0x80;
                }
            }
            uint8[offset++] = (value & 0x3f) | 0x80;
        }
        this.offset = offset;
    };
    return Encoder;
}(CrdtEncoder_1.CrdtEncoder));
exports.Encoder = Encoder;
