"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
var tslib_1 = require("tslib");
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryChunk_1 = require("../../types/rga-binary/BinaryChunk");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ClockDecoder_1 = require("../../../json-crdt-patch/codec/clock/ClockDecoder");
var ConstantType_1 = require("../../types/const/ConstantType");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var constants_1 = require("../../constants");
var decode_1 = require("../../../util/base64/decode");
var clock_1 = require("../../../json-crdt-patch/clock");
var model_1 = require("../../model");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var constants_2 = require("../../../json-crdt-patch/constants");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Decoder = (function () {
    function Decoder() {
    }
    Decoder.prototype.decode = function (data) {
        var x = data[0];
        var isServerTime = typeof x === 'number';
        if (isServerTime) {
            this.time = x;
        }
        else {
            this.clockDecoder = ClockDecoder_1.ClockDecoder.fromArr(x);
        }
        var doc = isServerTime ? model_1.Model.withServerClock(x) : model_1.Model.withLogicalClock(this.clockDecoder.clock);
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), rootId = _a[0], index = _a[1];
        var rootNode = data[index] ? this.decodeNode(doc, data[index]) : null;
        doc.root = new DocRootType_1.DocRootType(doc, rootId, rootNode);
        return doc;
    };
    Decoder.prototype.ts = function (arr, index) {
        var x = arr[index];
        if (typeof x === 'number') {
            if (x < 0) {
                var sessionIndex = -x;
                var timeDiff = arr[index + 1];
                return [this.clockDecoder.decodeId(sessionIndex, timeDiff), index + 2];
            }
            else {
                return [new clock_1.ServerTimestamp(this.time - x), index + 1];
            }
        }
        else {
            var time = x[0];
            switch (time) {
                case constants_2.ORIGIN.time:
                    return [constants_2.ORIGIN, index + 1];
                case constants_2.NULL_ID.time:
                    return [constants_2.NULL_ID, index + 1];
                case constants_2.TRUE_ID.time:
                    return [constants_2.TRUE_ID, index + 1];
                case constants_2.FALSE_ID.time:
                    return [constants_2.FALSE_ID, index + 1];
                case constants_2.UNDEFINED_ID.time:
                    return [constants_2.UNDEFINED_ID, index + 1];
                default:
                    return [new clock_1.LogicalTimestamp(0, time), index + 1];
            }
        }
    };
    Decoder.prototype.decodeNode = function (doc, data) {
        switch (data) {
            case 1:
                return constants_1.NULL;
            case 2:
                return constants_1.FALSE;
            case 3:
                return constants_1.TRUE;
            case 4:
                return constants_1.UNDEFINED;
        }
        if (data instanceof Array) {
            switch (data[0]) {
                case 0:
                    return this.decodeObj(doc, data);
                case 1:
                    return this.decodeArr(doc, data);
                case 2:
                    return this.decodeStr(doc, data);
                case 3:
                    return this.decodeVal(doc, data);
                case 4:
                    return this.decodeConst(doc, data);
                case 5:
                    return this.decodeBin(doc, data);
            }
        }
        throw new Error('UNKNOWN_NODE');
    };
    Decoder.prototype.decodeObj = function (doc, data) {
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var obj = new ObjectType_1.ObjectType(doc, id);
        var length = data.length;
        for (var i = index; i < length;) {
            var key = data[i];
            var _b = (0, tslib_1.__read)(this.ts(data, i + 1), 2), keyId = _b[0], idx = _b[1];
            var node = this.decodeNode(doc, data[idx]);
            var chunk = new ObjectChunk_1.ObjectChunk(keyId, node);
            obj.putChunk(key, chunk);
            i = idx + 1;
        }
        doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeArr = function (doc, data) {
        var _this = this;
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var obj = new ArrayType_1.ArrayType(doc, id);
        var length = data.length;
        for (var i = index; i < length;) {
            var _b = (0, tslib_1.__read)(this.ts(data, i), 2), chunkId = _b[0], idx = _b[1];
            var content = data[idx];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                var nodes = content.map(function (c) { return _this.decodeNode(doc, c); });
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, nodes);
            }
            obj.append(chunk);
            i = idx + 1;
        }
        doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeStr = function (doc, data) {
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var obj = new StringType_1.StringType(doc, id);
        var length = data.length;
        for (var i = index; i < length;) {
            var _b = (0, tslib_1.__read)(this.ts(data, i), 2), chunkId = _b[0], idx = _b[1];
            var content = data[idx];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new StringChunk_1.StringChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new StringChunk_1.StringChunk(chunkId, content);
            }
            obj.append(chunk);
            i = idx + 1;
        }
        doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeBin = function (doc, data) {
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), id = _a[0], index = _a[1];
        var obj = new BinaryType_1.BinaryType(doc, id);
        var length = data.length;
        for (var i = index; i < length;) {
            var _b = (0, tslib_1.__read)(this.ts(data, i), 2), chunkId = _b[0], idx = _b[1];
            var content = data[idx];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new BinaryChunk_1.BinaryChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new BinaryChunk_1.BinaryChunk(chunkId, (0, decode_1.fromBase64)(content));
            }
            obj.append(chunk);
            i = idx + 1;
        }
        doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeVal = function (doc, data) {
        var _a = (0, tslib_1.__read)(this.ts(data, 1), 2), id = _a[0], index1 = _a[1];
        var _b = (0, tslib_1.__read)(this.ts(data, index1), 2), writeId = _b[0], index2 = _b[1];
        var value = data[index2];
        var obj = new ValueType_1.ValueType(id, writeId, value);
        doc.nodes.index(obj);
        return obj;
    };
    Decoder.prototype.decodeConst = function (doc, data) {
        var value = data[1];
        var obj = new ConstantType_1.ConstantType(constants_2.ORIGIN, value);
        return obj;
    };
    return Decoder;
}());
exports.Decoder = Decoder;
