"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ClockEncoder_1 = require("../../../json-crdt-patch/codec/clock/ClockEncoder");
var ConstantType_1 = require("../../types/const/ConstantType");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var StringType_1 = require("../../types/rga-string/StringType");
var encode_1 = require("../../../util/base64/encode");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Encoder = (function () {
    function Encoder() {
    }
    Encoder.prototype.encode = function (model) {
        var isServerTime = model.clock.getSessionId() === 1;
        var clock = model.clock;
        var snapshot = isServerTime ? [clock.time] : [null];
        if (isServerTime) {
            this.time = clock.time;
        }
        else {
            model.advanceClocks();
            this.clock = new ClockEncoder_1.ClockEncoder(model.clock);
        }
        this.encodeRoot(snapshot, model.root);
        if (!isServerTime)
            snapshot[0] = this.clock.toJson();
        return snapshot;
    };
    Encoder.prototype.ts = function (arr, ts) {
        switch (ts.getSessionId()) {
            case 0: {
                arr.push([ts.time]);
                break;
            }
            case 1: {
                arr.push(this.time - ts.time);
                break;
            }
            default: {
                var relativeId = this.clock.append(ts);
                arr.push(-relativeId.sessionIndex, relativeId.timeDiff);
            }
        }
    };
    Encoder.prototype.encodeRoot = function (arr, root) {
        this.ts(arr, root.id);
        if (!root.node)
            arr.push(0);
        else
            this.encodeNode(arr, root.node);
    };
    Encoder.prototype.encodeNode = function (arr, node) {
        if (node instanceof ObjectType_1.ObjectType)
            return this.encodeObj(arr, node);
        else if (node instanceof ArrayType_1.ArrayType)
            return this.encodeArr(arr, node);
        else if (node instanceof StringType_1.StringType)
            return this.encodeStr(arr, node);
        else if (node instanceof ValueType_1.ValueType)
            return this.encodeVal(arr, node);
        else if (node instanceof ConstantType_1.ConstantType)
            return this.encodeConst(arr, node);
        else if (node instanceof BinaryType_1.BinaryType)
            return this.encodeBin(arr, node);
        throw new Error('UNKNOWN_NODE');
    };
    Encoder.prototype.encodeObj = function (arr, obj) {
        var e_1, _a;
        var res = [0];
        arr.push(res);
        this.ts(res, obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = (0, tslib_1.__read)(_c.value, 2), key = _d[0], entry = _d[1];
                res.push(key);
                this.ts(res, entry.id);
                this.encodeNode(res, entry.node);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    Encoder.prototype.encodeArr = function (arr, obj) {
        var e_2, _a;
        var res = [1];
        arr.push(res);
        this.ts(res, obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeArrChunk(res, chunk);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    Encoder.prototype.encodeArrChunk = function (arr, chunk) {
        var e_3, _a;
        this.ts(arr, chunk.id);
        if (chunk.deleted)
            arr.push(chunk.deleted);
        else {
            var nodes = [];
            try {
                for (var _b = (0, tslib_1.__values)(chunk.nodes), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var n = _c.value;
                    this.encodeNode(nodes, n);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
            arr.push(nodes);
        }
    };
    Encoder.prototype.encodeStr = function (arr, obj) {
        var e_4, _a;
        var res = [2];
        arr.push(res);
        this.ts(res, obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeStrChunk(res, chunk);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
    };
    Encoder.prototype.encodeStrChunk = function (arr, chunk) {
        this.ts(arr, chunk.id);
        if (chunk.deleted)
            arr.push(chunk.deleted);
        else
            arr.push(chunk.str);
    };
    Encoder.prototype.encodeBin = function (arr, obj) {
        var e_5, _a;
        var res = [5];
        arr.push(res);
        this.ts(res, obj.id);
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                this.encodeBinChunk(res, chunk);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    Encoder.prototype.encodeBinChunk = function (arr, chunk) {
        this.ts(arr, chunk.id);
        if (chunk.deleted)
            arr.push(chunk.deleted);
        else
            arr.push((0, encode_1.toBase64)(chunk.buf));
    };
    Encoder.prototype.encodeVal = function (arr, obj) {
        var res = [3];
        arr.push(res);
        this.ts(res, obj.id);
        this.ts(res, obj.writeId);
        res.push(obj.value);
    };
    Encoder.prototype.encodeConst = function (arr, obj) {
        switch (obj.value) {
            case null: {
                arr.push(1);
                return;
            }
            case false: {
                arr.push(2);
                return;
            }
            case true: {
                arr.push(3);
                return;
            }
            case undefined: {
                arr.push(4);
                return;
            }
        }
        var res = [4];
        arr.push(res);
        res.push(obj.value);
    };
    return Encoder;
}());
exports.Encoder = Encoder;
