"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var tslib_1 = require("tslib");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ConstantType_1 = require("../../types/const/ConstantType");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var StringType_1 = require("../../types/rga-string/StringType");
var encode_1 = require("../../../util/base64/encode");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Encoder = (function () {
    function Encoder() {
    }
    Encoder.prototype.encode = function (model) {
        var clock = model.clock;
        var isServerClock = clock.getSessionId() === 1;
        return {
            time: isServerClock ? clock.time : this.encodeClock(model.clock),
            root: this.encodeRoot(model.root),
        };
    };
    Encoder.prototype.encodeClock = function (clock) {
        var e_1, _a;
        var data = [];
        var sessionId = clock.getSessionId();
        var localTs = clock.clocks.get(sessionId);
        if (!localTs)
            data.push([sessionId, clock.time]);
        try {
            for (var _b = (0, tslib_1.__values)(clock.clocks.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var c = _c.value;
                data.push([c.getSessionId(), c.time]);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return data;
    };
    Encoder.prototype.encodeTimestamp = function (ts) {
        return ts.getSessionId() === 1 ? ts.time : [ts.getSessionId(), ts.time];
    };
    Encoder.prototype.encodeRoot = function (root) {
        return {
            type: 'root',
            id: this.encodeTimestamp(root.id),
            node: root.node ? this.encodeNode(root.node) : null,
        };
    };
    Encoder.prototype.encodeNode = function (node) {
        if (node instanceof ObjectType_1.ObjectType)
            return this.encodeObj(node);
        else if (node instanceof ArrayType_1.ArrayType)
            return this.encodeArr(node);
        else if (node instanceof StringType_1.StringType)
            return this.encodeStr(node);
        else if (node instanceof ValueType_1.ValueType)
            return this.encodeVal(node);
        else if (node instanceof ConstantType_1.ConstantType)
            return this.encodeConst(node);
        else if (node instanceof BinaryType_1.BinaryType)
            return this.encodeBin(node);
        throw new Error('UNKNOWN_NODE');
    };
    Encoder.prototype.encodeObj = function (obj) {
        var e_2, _a;
        var chunks = {};
        try {
            for (var _b = (0, tslib_1.__values)(obj.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = (0, tslib_1.__read)(_c.value, 2), key = _d[0], entry = _d[1];
                chunks[key] = {
                    id: this.encodeTimestamp(entry.id),
                    node: this.encodeNode(entry.node),
                };
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return {
            type: 'obj',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    Encoder.prototype.encodeArr = function (obj) {
        var e_3, _a;
        var chunks = [];
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                chunks.push(this.encodeArrChunk(chunk));
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return {
            type: 'arr',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    Encoder.prototype.encodeArrChunk = function (chunk) {
        var _this = this;
        if (chunk.deleted) {
            var tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        var res = {
            id: this.encodeTimestamp(chunk.id),
            nodes: chunk.nodes.map(function (n) { return _this.encodeNode(n); }),
        };
        return res;
    };
    Encoder.prototype.encodeStr = function (obj) {
        var e_4, _a;
        var chunks = [];
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                chunks.push(this.encodeStrChunk(chunk));
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return {
            type: 'str',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    Encoder.prototype.encodeStrChunk = function (chunk) {
        if (chunk.deleted) {
            var tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        var res = {
            id: this.encodeTimestamp(chunk.id),
            value: chunk.str,
        };
        return res;
    };
    Encoder.prototype.encodeBin = function (obj) {
        var e_5, _a;
        var chunks = [];
        try {
            for (var _b = (0, tslib_1.__values)(obj.chunks()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var chunk = _c.value;
                chunks.push(this.encodeBinChunk(chunk));
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
        return {
            type: 'bin',
            id: this.encodeTimestamp(obj.id),
            chunks: chunks,
        };
    };
    Encoder.prototype.encodeBinChunk = function (chunk) {
        if (chunk.deleted) {
            var tombstone = {
                id: this.encodeTimestamp(chunk.id),
                span: chunk.deleted,
            };
            return tombstone;
        }
        var res = {
            id: this.encodeTimestamp(chunk.id),
            value: (0, encode_1.toBase64)(chunk.buf),
        };
        return res;
    };
    Encoder.prototype.encodeVal = function (obj) {
        return {
            type: 'val',
            id: this.encodeTimestamp(obj.id),
            writeId: this.encodeTimestamp(obj.writeId),
            value: obj.value,
        };
    };
    Encoder.prototype.encodeConst = function (obj) {
        return {
            type: 'const',
            value: obj.value,
        };
    };
    return Encoder;
}());
exports.Encoder = Encoder;
