"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Finder = void 0;
var ArrayApi_1 = require("./ArrayApi");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryApi_1 = require("./BinaryApi");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var constants_1 = require("../../constants");
var ObjectApi_1 = require("./ObjectApi");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var StringApi_1 = require("./StringApi");
var StringType_1 = require("../../types/rga-string/StringType");
var ValueApi_1 = require("./ValueApi");
var ValueType_1 = require("../../types/lww-value/ValueType");
var Finder = (function () {
    function Finder(node, api) {
        this.node = node;
        this.api = api;
    }
    Finder.prototype.find = function (steps) {
        var node = this.node || constants_1.NULL;
        var length = steps.length;
        if (!length)
            return node;
        var i = 0;
        while (i < length) {
            var step = steps[i++];
            if (node instanceof ObjectType_1.ObjectType) {
                var id = node.get(String(step));
                if (!id)
                    return constants_1.UNDEFINED;
                var nextNode = this.api.model.node(id);
                if (!nextNode)
                    return constants_1.UNDEFINED;
                node = nextNode;
            }
            else if (node instanceof ArrayType_1.ArrayType) {
                var id = node.findValue(Number(step));
                var nextNode = this.api.model.node(id);
                if (!nextNode)
                    return constants_1.UNDEFINED;
                node = nextNode;
                continue;
            }
        }
        return node;
    };
    Finder.prototype.val = function (path) {
        var node = this.find(path);
        if (node instanceof ValueType_1.ValueType)
            return new ValueApi_1.ValueApi(this.api, node);
        throw new Error('NOT_VAL');
    };
    Finder.prototype.str = function (path) {
        var node = this.find(path);
        if (node instanceof StringType_1.StringType)
            return new StringApi_1.StringApi(this.api, node);
        throw new Error('NOT_STR');
    };
    Finder.prototype.bin = function (path) {
        var node = this.find(path);
        if (node instanceof BinaryType_1.BinaryType)
            return new BinaryApi_1.BinaryApi(this.api, node);
        throw new Error('NOT_BIN');
    };
    Finder.prototype.arr = function (path) {
        var node = this.find(path);
        if (node instanceof ArrayType_1.ArrayType)
            return new ArrayApi_1.ArrayApi(this.api, node);
        throw new Error('NOT_ARR');
    };
    Finder.prototype.obj = function (path) {
        var obj = this.find(path);
        if (obj instanceof ObjectType_1.ObjectType)
            return new ObjectApi_1.ObjectApi(this.api, obj);
        throw new Error('NOT_OBJ');
    };
    return Finder;
}());
exports.Finder = Finder;
