"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
var Batch_1 = require("../../../json-crdt-patch/Batch");
var Finder_1 = require("./Finder");
var constants_1 = require("../../constants");
var PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
var ModelApi = (function () {
    function ModelApi(model) {
        this.model = model;
        this.batch = new Batch_1.Batch([]);
        this.builder = new PatchBuilder_1.PatchBuilder(model.clock);
    }
    ModelApi.prototype.getApi = function () {
        return this;
    };
    ModelApi.prototype.getNode = function () {
        var model = this.model;
        var id = model.root.toValue();
        return model.node(id) || constants_1.NULL;
    };
    ModelApi.prototype.find = function (path) {
        var finder = new Finder_1.Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    };
    ModelApi.prototype.val = function (path) {
        return this.find().val(path);
    };
    ModelApi.prototype.str = function (path) {
        return this.find().str(path);
    };
    ModelApi.prototype.bin = function (path) {
        return this.find().bin(path);
    };
    ModelApi.prototype.arr = function (path) {
        return this.find().arr(path);
    };
    ModelApi.prototype.obj = function (path) {
        return this.find().obj(path);
    };
    ModelApi.prototype.commit = function () {
        var patch = this.builder.patch;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.batch.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    };
    ModelApi.prototype.flush = function () {
        var batch = this.batch;
        this.batch = new Batch_1.Batch([]);
        return batch;
    };
    ModelApi.prototype.patch = function (cb) {
        this.commit();
        cb(this);
        this.commit();
    };
    ModelApi.prototype.root = function (json) {
        var value = this.builder.json(json);
        this.builder.root(value);
        return this;
    };
    ModelApi.prototype.toView = function () {
        return this.getNode().toJson();
    };
    return ModelApi;
}());
exports.ModelApi = ModelApi;
