"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerNodeIndex = exports.LogicalNodeIndex = void 0;
var tslib_1 = require("tslib");
var LogicalNodeIndex = (function () {
    function LogicalNodeIndex() {
        this.entries = new Map();
    }
    LogicalNodeIndex.prototype.get = function (id) {
        var time = id.time;
        var sessionId = id.getSessionId();
        var map1 = this.entries;
        var map2 = map1.get(sessionId);
        if (!map2)
            return undefined;
        var operation = map2.get(time);
        if (operation)
            return operation;
        return undefined;
    };
    LogicalNodeIndex.prototype.index = function (operation) {
        var id = operation.id;
        var sessionId = id.getSessionId();
        var time = id.time;
        var map = this.entries.get(sessionId);
        if (!map) {
            map = new Map();
            this.entries.set(sessionId, map);
        }
        map.set(time, operation);
    };
    LogicalNodeIndex.prototype.delete = function (id) {
        var time = id.time;
        var timeMap = this.entries.get(id.getSessionId());
        if (!timeMap)
            return;
        timeMap.delete(time);
    };
    LogicalNodeIndex.prototype.iterate = function () {
        var _a, _b, map, e_1_1;
        var e_1, _c;
        return (0, tslib_1.__generator)(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _d.trys.push([0, 5, 6, 7]);
                    _a = (0, tslib_1.__values)(this.entries.values()), _b = _a.next();
                    _d.label = 1;
                case 1:
                    if (!!_b.done) return [3, 4];
                    map = _b.value;
                    return [5, (0, tslib_1.__values)(map.values())];
                case 2:
                    _d.sent();
                    _d.label = 3;
                case 3:
                    _b = _a.next();
                    return [3, 1];
                case 4: return [3, 7];
                case 5:
                    e_1_1 = _d.sent();
                    e_1 = { error: e_1_1 };
                    return [3, 7];
                case 6:
                    try {
                        if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                    }
                    finally { if (e_1) throw e_1.error; }
                    return [7];
                case 7: return [2];
            }
        });
    };
    LogicalNodeIndex.prototype.random = function () {
        var e_2, _a;
        var sessions = (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(this.entries.keys()), false);
        if (!sessions.length)
            return null;
        var sessionId = sessions[Math.floor(Math.random() * sessions.length)];
        var map = this.entries.get(sessionId);
        var nodeCount = map.size;
        var index = Math.floor(Math.random() * nodeCount);
        var pos = 0;
        try {
            for (var _b = (0, tslib_1.__values)(map.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var node = _c.value;
                if (pos === index)
                    return node;
                pos++;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return null;
    };
    return LogicalNodeIndex;
}());
exports.LogicalNodeIndex = LogicalNodeIndex;
var ServerNodeIndex = (function () {
    function ServerNodeIndex() {
        this.entries = new Map();
    }
    ServerNodeIndex.prototype.get = function (id) {
        return this.entries.get(id.time);
    };
    ServerNodeIndex.prototype.index = function (node) {
        var id = node.id;
        this.entries.set(id.time, node);
    };
    ServerNodeIndex.prototype.delete = function (id) {
        this.entries.delete(id.time);
    };
    ServerNodeIndex.prototype.iterate = function () {
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [5, (0, tslib_1.__values)(this.entries.values())];
                case 1:
                    _a.sent();
                    return [2];
            }
        });
    };
    ServerNodeIndex.prototype.random = function () {
        var e_3, _a;
        var nodeCount = this.entries.size;
        var index = Math.floor(Math.random() * nodeCount);
        var pos = 0;
        try {
            for (var _b = (0, tslib_1.__values)(this.entries.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var node = _c.value;
                if (pos === index)
                    return node;
                pos++;
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return null;
    };
    return ServerNodeIndex;
}());
exports.ServerNodeIndex = ServerNodeIndex;
