"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArrayType = void 0;
var tslib_1 = require("tslib");
var ArrayChunk_1 = require("./ArrayChunk");
var ArrayOriginChunk_1 = require("./ArrayOriginChunk");
var ArrayType = (function () {
    function ArrayType(doc, id) {
        this.doc = doc;
        this.id = id;
        this._toJson = [];
        this.start = this.end = new ArrayOriginChunk_1.ArrayOriginChunk(id);
    }
    ArrayType.prototype.insert = function (op) {
        var e_1, _a;
        var curr = this.end;
        while (curr) {
            if (curr.id.overlap(curr.span(), op.id, op.span()))
                return;
            if (curr.id.inSpan(curr.span(), op.after, 1))
                break;
            curr = curr.left;
        }
        if (!curr)
            return;
        var nodes = [];
        try {
            for (var _b = (0, tslib_1.__values)(op.elements), _c = _b.next(); !_c.done; _c = _b.next()) {
                var el = _c.value;
                var node = this.doc.node(el);
                if (node)
                    nodes.push(node);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (!nodes.length)
            return;
        var isOriginChunk = curr instanceof ArrayOriginChunk_1.ArrayOriginChunk;
        if (!curr.deleted && !isOriginChunk) {
            var doesAfterMatch = curr.id.getSessionId() === op.after.getSessionId() && curr.id.time + curr.span() - 1 === op.after.time;
            var isIdSameSession = curr.id.getSessionId() === op.id.getSessionId();
            var isIdIncreasingWithoutAGap = curr.id.time + curr.span() === op.id.time;
            if (doesAfterMatch && isIdSameSession && isIdIncreasingWithoutAGap) {
                curr.merge(nodes);
                return;
            }
        }
        var chunk = new ArrayChunk_1.ArrayChunk(op.id, nodes);
        var targetsLastElementInChunk = op.after.time === curr.id.time + curr.span() - 1;
        if (targetsLastElementInChunk) {
            while (curr.right && curr.right.id.compare(op.id) > 0)
                curr = curr.right;
            this.insertChunk(chunk, curr);
            return;
        }
        this.splitChunk(curr, op.after.time);
        this.insertChunk(chunk, curr);
    };
    ArrayType.prototype.insertChunk = function (chunk, after) {
        chunk.left = after;
        chunk.right = after.right;
        if (after.right)
            after.right.left = chunk;
        else
            this.end = chunk;
        after.right = chunk;
    };
    ArrayType.prototype.splitChunk = function (chunk, time) {
        var newChunk = chunk.split(time);
        this.insertChunk(newChunk, chunk);
        return newChunk;
    };
    ArrayType.prototype.delete = function (op) {
        var e_2, _a;
        var after = op.after;
        var length = after.span;
        var chunk = this.end;
        var chunks = [];
        while (chunk) {
            if (chunk.id.overlap(chunk.span(), after, length))
                chunks.push(chunk);
            if (chunk.id.inSpan(chunk.span(), after, 1))
                break;
            chunk = chunk.left;
        }
        try {
            for (var chunks_1 = (0, tslib_1.__values)(chunks), chunks_1_1 = chunks_1.next(); !chunks_1_1.done; chunks_1_1 = chunks_1.next()) {
                var c = chunks_1_1.value;
                this.deleteInChunk(c, after.time, after.time + length - 1);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (chunks_1_1 && !chunks_1_1.done && (_a = chunks_1.return)) _a.call(chunks_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    ArrayType.prototype.deleteInChunk = function (chunk, t1, t2) {
        var c1 = chunk.id.time;
        var c2 = c1 + chunk.span() - 1;
        if (t1 <= c1) {
            if (t2 >= c2)
                chunk.delete();
            else {
                this.splitChunk(chunk, t2);
                chunk.delete();
            }
        }
        else {
            if (t2 >= c2)
                this.splitChunk(chunk, t1 - 1).delete();
            else {
                this.splitChunk(chunk, t2);
                this.splitChunk(chunk, t1 - 1).delete();
            }
        }
    };
    ArrayType.prototype.findId = function (index) {
        var chunk = this.start;
        var cnt = 0;
        var next = index + 1;
        while (chunk) {
            if (chunk.nodes) {
                cnt += chunk.nodes.length;
                if (cnt >= next)
                    return chunk.id.tick(chunk.nodes.length - (cnt - index));
            }
            chunk = chunk.right;
        }
        throw new Error('OUT_OF_BOUNDS');
    };
    ArrayType.prototype.findValue = function (index) {
        var chunk = this.start;
        var cnt = 0;
        var next = index + 1;
        while (chunk) {
            if (chunk.nodes) {
                cnt += chunk.nodes.length;
                if (cnt >= next)
                    return chunk.nodes[chunk.nodes.length - (cnt - index)].id;
            }
            chunk = chunk.right;
        }
        throw new Error('OUT_OF_BOUNDS');
    };
    ArrayType.prototype.findIdSpans = function (index, length) {
        var chunk = this.start;
        var cnt = 0;
        var next = index + 1;
        while (chunk) {
            if (chunk.nodes) {
                cnt += chunk.nodes.length;
                if (cnt >= next) {
                    var remaining = cnt - index;
                    if (remaining >= length)
                        return [chunk.id.interval(chunk.span() - remaining, length)];
                    length -= remaining;
                    var result = [chunk.id.interval(chunk.span() - remaining, remaining)];
                    while (chunk.right) {
                        chunk = chunk.right;
                        if (chunk.deleted)
                            continue;
                        var span = chunk.span();
                        if (span >= length) {
                            result.push(chunk.id.interval(0, length));
                            break;
                        }
                        result.push(chunk.id.interval(0, span));
                        length -= span;
                    }
                    return result;
                }
            }
            chunk = chunk.right;
        }
        throw new Error('OUT_OF_BOUNDS');
    };
    ArrayType.prototype.append = function (chunk) {
        var last = this.end;
        last.right = chunk;
        chunk.left = last;
        this.end = chunk;
    };
    ArrayType.prototype.toJson = function () {
        var arr = [];
        var curr = this.start;
        while (curr) {
            if (curr.nodes)
                arr.push.apply(arr, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(curr.nodes.map(function (node) { return node.toJson(); })), false));
            curr = curr.right;
        }
        var _toJson = this._toJson;
        if (arr.length !== _toJson.length)
            return this._toJson = arr;
        for (var i = 0; i < arr.length; i++)
            if (arr[i] !== _toJson[i])
                return this._toJson = arr;
        return _toJson;
    };
    ArrayType.prototype.clone = function (model) {
        var copy = new ArrayType(model, this.id);
        var i = this.start;
        var j = copy.start;
        while (i.right) {
            var cloned = i.right.clone(model);
            j.right = cloned;
            cloned.left = j;
            j = cloned;
            i = i.right;
        }
        copy.end = j;
        model.nodes.index(copy);
        return copy;
    };
    ArrayType.prototype.children = function () {
        var chunk, _a, _b, node, e_3_1;
        var e_3, _c;
        return (0, tslib_1.__generator)(this, function (_d) {
            switch (_d.label) {
                case 0:
                    chunk = this.start;
                    _d.label = 1;
                case 1:
                    if (!(chunk = chunk.right)) return [3, 10];
                    if (!chunk.nodes) return [3, 9];
                    _d.label = 2;
                case 2:
                    _d.trys.push([2, 7, 8, 9]);
                    _a = (e_3 = void 0, (0, tslib_1.__values)(chunk.nodes)), _b = _a.next();
                    _d.label = 3;
                case 3:
                    if (!!_b.done) return [3, 6];
                    node = _b.value;
                    return [4, node.id];
                case 4:
                    _d.sent();
                    _d.label = 5;
                case 5:
                    _b = _a.next();
                    return [3, 3];
                case 6: return [3, 9];
                case 7:
                    e_3_1 = _d.sent();
                    e_3 = { error: e_3_1 };
                    return [3, 9];
                case 8:
                    try {
                        if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                    }
                    finally { if (e_3) throw e_3.error; }
                    return [7];
                case 9: return [3, 1];
                case 10: return [2];
            }
        });
    };
    ArrayType.prototype.chunks = function () {
        var curr;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0:
                    curr = this.start;
                    _a.label = 1;
                case 1:
                    if (!(curr = curr.right)) return [3, 3];
                    return [4, curr];
                case 2:
                    _a.sent();
                    return [3, 1];
                case 3: return [2];
            }
        });
    };
    ArrayType.prototype.size = function () {
        var curr = this.start;
        var size = 0;
        while ((curr = curr.right))
            size++;
        return size;
    };
    ArrayType.prototype.length = function () {
        var curr = this.start;
        var size = 0;
        while ((curr = curr.right))
            if (curr.nodes)
                size += curr.nodes.length;
        return size;
    };
    ArrayType.prototype.toString = function (tab) {
        if (tab === void 0) { tab = ''; }
        var str = "".concat(tab, "ArrayType(").concat(this.id.toDisplayString(), ")");
        var curr = this.start;
        while (curr) {
            str += "\n".concat(curr.toString(tab + '  '));
            curr = curr.right;
        }
        return str;
    };
    return ArrayType;
}());
exports.ArrayType = ArrayType;
