"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryChunk = void 0;
var clock_1 = require("../../../json-crdt-patch/clock");
var BinaryChunk = (function () {
    function BinaryChunk(id, buf) {
        this.id = id;
        this.buf = buf;
        this.left = null;
        this.right = null;
    }
    BinaryChunk.prototype.span = function () {
        return this.deleted || this.buf.byteLength;
    };
    BinaryChunk.prototype.split = function (time) {
        var newSpan = 1 + time - this.id.time;
        var newId = new clock_1.LogicalTimestamp(this.id.getSessionId(), time + 1);
        if (this.buf) {
            var newChunkStr = this.buf.subarray(newSpan);
            this.buf = this.buf.subarray(0, newSpan);
            return new BinaryChunk(newId, newChunkStr);
        }
        else {
            var chunk = new BinaryChunk(newId, undefined);
            chunk.deleted = this.span() - newSpan;
            this.deleted = newSpan;
            return chunk;
        }
    };
    BinaryChunk.prototype.delete = function () {
        this.deleted = this.span();
        delete this.buf;
    };
    BinaryChunk.prototype.merge = function (buf) {
        if (this.deleted) {
            this.deleted += buf.length;
            return;
        }
        var length = this.buf.length;
        var combined = new Uint8Array(length + buf.length);
        combined.set(this.buf);
        combined.set(buf, length);
        this.buf = combined;
    };
    BinaryChunk.prototype.clone = function () {
        var chunk = new BinaryChunk(this.id, this.buf);
        if (this.deleted) {
            chunk.deleted = this.deleted;
            delete chunk.buf;
        }
        return chunk;
    };
    BinaryChunk.prototype.toString = function (tab) {
        if (tab === void 0) { tab = ''; }
        var str = "".concat(tab).concat(this.constructor.name, "(").concat(this.id.toDisplayString(), ") { ").concat(!this.buf
            ? "[".concat(this.deleted || this.span(), "]")
            : this.buf.length > 10
                ? this.buf.subarray(0, 10).toString() + ',…'
                : this.buf.toString(), " }");
        return str;
    };
    return BinaryChunk;
}());
exports.BinaryChunk = BinaryChunk;
