"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyPatch = exports.applyOperation = void 0;
var clone_1 = require("../../json-clone/clone");
var json_pointer_1 = require("../../json-pointer");
var deepEqual_1 = require("../../json-equal/deepEqual");
var hasOwnProperty_1 = require("../../util/hasOwnProperty");
var isArray = Array.isArray;
function applyOperation(doc, operation) {
    var path = operation.path;
    var isRoot = !path;
    if (isRoot) {
        switch (operation.op) {
            case 'add':
            case 'replace':
                doc = operation.value;
                return { doc: operation.value, old: doc };
            case 'remove':
                return { doc: null, old: doc };
            case 'move': {
                var val = (0, json_pointer_1.findByPointer)(operation.from, doc).val;
                return { doc: val, old: doc };
            }
            case 'copy': {
                var val = (0, json_pointer_1.findByPointer)(operation.from, doc).val;
                return { doc: val, old: doc };
            }
            case 'test': {
                if (!(0, deepEqual_1.deepEqual)(operation.value, doc))
                    throw new Error('TEST');
                return { doc: doc };
            }
        }
        return { doc: doc };
    }
    var indexOfSlash = 0;
    var indexAfterSlash = 1;
    var obj = doc;
    var key = '';
    while (indexOfSlash > -1) {
        indexOfSlash = path.indexOf('/', indexAfterSlash);
        key = indexOfSlash > -1 ? path.substring(indexAfterSlash, indexOfSlash) : path.substring(indexAfterSlash);
        indexAfterSlash = indexOfSlash + 1;
        if (isArray(obj)) {
            var length_1 = obj.length;
            if (key === '-')
                key = length_1;
            else {
                var key2 = ~~key;
                if ('' + key2 !== key)
                    throw new Error('INVALID_INDEX');
                key = key2;
                if (key < 0 || key > length_1)
                    throw new Error('INVALID_INDEX');
            }
            if (indexOfSlash === -1) {
                switch (operation.op) {
                    case 'add': {
                        var old = obj[key];
                        if (key < obj.length)
                            obj.splice(key, 0, operation.value);
                        else
                            obj.push(operation.value);
                        return { doc: doc, old: old };
                    }
                    case 'replace': {
                        var old = obj[key];
                        obj[key] = operation.value;
                        return { doc: doc, old: old };
                    }
                    case 'remove': {
                        var old = obj[key];
                        obj.splice(key, 1);
                        return { doc: doc, old: old };
                    }
                    case 'move': {
                        var removeResult = applyOperation(doc, { op: 'remove', path: operation.from });
                        return applyOperation(removeResult.doc, { op: 'add', path: operation.path, value: removeResult.old });
                    }
                    case 'copy': {
                        var old = obj[key];
                        var val = (0, json_pointer_1.findByPointer)(operation.from, doc).val;
                        var value = (0, clone_1.clone)(val);
                        if (key < obj.length)
                            obj.splice(key, 0, value);
                        else
                            obj.push(value);
                        return { doc: doc, old: old };
                    }
                    case 'test': {
                        if (!(0, deepEqual_1.deepEqual)(operation.value, obj[key]))
                            throw new Error('TEST');
                        return { doc: doc };
                    }
                }
                break;
            }
            obj = obj[key];
        }
        else if (typeof obj === 'object' && !!obj) {
            key = (0, json_pointer_1.unescapeComponent)(key);
            if (indexOfSlash === -1) {
                switch (operation.op) {
                    case 'add': {
                        var old = obj[key];
                        obj[key] = operation.value;
                        return { doc: doc, old: old };
                    }
                    case 'replace': {
                        var old = obj[key];
                        obj[key] = operation.value;
                        return { doc: doc, old: old };
                    }
                    case 'remove': {
                        var old = obj[key];
                        delete obj[key];
                        return { doc: doc, old: old };
                    }
                    case 'move': {
                        var removeResult = applyOperation(doc, { op: 'remove', path: operation.from });
                        var addResult = applyOperation(doc, { op: 'add', path: operation.path, value: removeResult.old });
                        return addResult;
                    }
                    case 'copy': {
                        var val = (0, json_pointer_1.findByPointer)(operation.from, doc).val;
                        var value = (0, clone_1.clone)(val);
                        var old = obj[key];
                        obj[key] = value;
                        return { doc: doc, old: old };
                    }
                    case 'test': {
                        if (!(0, deepEqual_1.deepEqual)(operation.value, obj[key]))
                            throw new Error('TEST');
                        return { doc: doc };
                    }
                }
                break;
            }
            obj = (0, hasOwnProperty_1.hasOwnProperty)(obj, key) ? obj[key] : undefined;
        }
        else
            throw new Error('NOT_FOUND');
    }
    return { doc: doc };
}
exports.applyOperation = applyOperation;
function applyPatch(doc, patch, options) {
    if (!options.mutate)
        doc = (0, clone_1.clone)(doc);
    var res = [];
    for (var i = 0; i < patch.length; i++) {
        var operation = patch[i];
        var opResult = applyOperation(doc, operation);
        res.push(opResult);
        doc = opResult.doc;
    }
    return { doc: doc, res: res };
}
exports.applyPatch = applyPatch;
