"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.randomType = void 0;
var tslib_1 = require("tslib");
var _1 = require(".");
var clone_1 = require("../json-clone/clone");
var randomType = function (type, options) {
    var e_1, _a;
    switch (type.__t) {
        case 'nil':
            return null;
        case 'bool': {
            if (typeof type.const === 'boolean')
                return type.const;
            return _1.RandomJson.genBoolean();
        }
        case 'num': {
            if (typeof type.const === 'number')
                return type.const;
            if (type.format) {
                var num = Math.random();
                switch (type.format) {
                    case 'i8':
                        return Math.round(num * 0xff) - 0x80;
                    case 'i16':
                        return Math.round(num * 0xffff) - 0x8000;
                    case 'i32':
                        return Math.round(num * 0xffffffff) - 0x80000000;
                    case 'i64':
                    case 'i':
                        return Math.round(num * 0xffffffffff) - 0x8000000000;
                    case 'u8':
                        return Math.round(num * 0xff);
                    case 'u16':
                        return Math.round(num * 0xffff);
                    case 'u32':
                        return Math.round(num * 0xffffffff);
                    case 'u64':
                    case 'u':
                        return Math.round(num * 0xffffffffff);
                }
            }
            return _1.RandomJson.genNumber();
        }
        case 'str': {
            if (typeof type.const === 'string')
                return type.const;
            return _1.RandomJson.genString();
        }
        case 'arr': {
            var length_1 = Math.round(Math.random() * 10);
            var arr = [];
            for (var i = 0; i < length_1; i++) {
                arr.push((0, exports.randomType)(type.type, options));
            }
            return arr;
        }
        case 'obj': {
            var obj = type.unknownFields
                ? _1.RandomJson.genObject()
                : {};
            try {
                for (var _b = (0, tslib_1.__values)(type.fields), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var field = _c.value;
                    if (field.isOptional)
                        if (Math.random() > 0.5)
                            continue;
                    obj[field.key] = (0, exports.randomType)(field.type, options);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return obj;
        }
        case 'enum': {
            var index = Math.floor(Math.random() * type.values.length);
            var value = (0, clone_1.clone)(type.values[index]);
            return value;
        }
        case 'ref': {
            if (!options.ref) {
                throw new Error('Reference resolution callback not provided.');
            }
            return (0, exports.randomType)(options.ref(type.ref), options);
        }
        case 'any': {
            return _1.RandomJson.generate({ nodeCount: 5 });
        }
        case 'bin': {
            var octets = _1.RandomJson.genString()
                .split('')
                .map(function (c) { return c.charCodeAt(0); });
            return new Uint8Array(octets);
        }
        case 'or': {
            var index = Math.floor(Math.random() * type.types.length);
            return (0, exports.randomType)(type.types[index], options);
        }
    }
};
exports.randomType = randomType;
