"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxServerJson = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var util_1 = require("./util");
var TimedQueue_1 = require("./TimedQueue");
var asString_1 = require("../util/asString");
var formatError = function (error) {
    var json = '{"message":' + (0, asString_1.asString)(error.message);
    if (typeof error.status === 'number')
        json += ',"status":' + error.status;
    if (typeof error.code === 'string')
        json += ',"code":' + (0, asString_1.asString)(error.code);
    if (typeof error.errno === 'number')
        json += ',"errno":' + error.errno;
    if (typeof error.errorId === 'string')
        json += ',"errorId":' + (0, asString_1.asString)(error.errorId);
    return (json + '}');
};
var JsonRxServerJson = (function () {
    function JsonRxServerJson(_a) {
        var send = _a.send, call = _a.call, notify = _a.notify, _b = _a.maxActiveSubscriptions, maxActiveSubscriptions = _b === void 0 ? 30 : _b, _c = _a.bufferSize, bufferSize = _c === void 0 ? 10 : _c, _d = _a.bufferTime, bufferTime = _d === void 0 ? 1 : _d;
        this.active = new Map();
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
        if (bufferTime) {
            var buffer_1 = new TimedQueue_1.TimedQueue();
            buffer_1.itemLimit = bufferSize;
            buffer_1.timeLimit = bufferTime;
            buffer_1.onFlush = function (messages) {
                send(messages.length === 1 ? messages[0] : ('[' + messages.join(',') + ']'));
            };
            this.send = function (message) { return buffer_1.push(message); };
        }
        else {
            this.send = send;
        }
    }
    JsonRxServerJson.prototype.sendError = function (id, error) {
        var message = ('[-1,' + id + ',' + JSON.stringify(error) + ']');
        this.send(message);
    };
    JsonRxServerJson.prototype.onSubscribe = function (message, ctx) {
        var _this = this;
        var _a = (0, tslib_1.__read)(message, 3), id = _a[0], name = _a[1], payload = _a[2];
        (0, util_1.assertId)(id);
        (0, util_1.assertName)(name);
        try {
            var active = this.active.get(id);
            if (active) {
                this.active.delete(id);
                active.unsubscribe();
                this.send(('[-1,' + id + ',{"message":"ID already active."}]'));
                return;
            }
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            var callResult = this.call(name, payload, ctx);
            var observable = (0, rxjs_1.isObservable)(callResult)
                ? callResult
                : (0, rxjs_1.from)(callResult).pipe((0, operators_1.mergeMap)(function (value) { return ((0, rxjs_1.isObservable)(value) ? value : (0, rxjs_1.of)(value)); }));
            var ref_1 = { buffer: [] };
            var done_1 = false;
            var subscription = observable.subscribe(function (data) {
                if (data === undefined)
                    return;
                ref_1.buffer.push(data);
                (0, util_1.microtask)(function () {
                    var e_1, _a;
                    if (!ref_1.buffer.length)
                        return;
                    try {
                        try {
                            for (var _b = (0, tslib_1.__values)(ref_1.buffer), _c = _b.next(); !_c.done; _c = _b.next()) {
                                var payload_1 = _c.value;
                                _this.send(('[-2,' + id + ',' + payload_1 + ']'));
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                    }
                    finally {
                        ref_1.buffer = [];
                    }
                });
            }, function (error) {
                done_1 = true;
                _this.active.delete(id);
                var message;
                if (error instanceof Error) {
                    message = ('[-1,' + id + ',' + formatError(error) + ']');
                }
                else {
                    message = ('[-1,' + id + ',' + JSON.stringify(error) + ']');
                }
                _this.send(message);
            }, function () {
                done_1 = true;
                _this.active.delete(id);
                try {
                    if (!ref_1.buffer.length)
                        return _this.send("[0,".concat(id, "]"));
                    var last = ref_1.buffer.length - 1;
                    for (var i = 0; i <= last; i++) {
                        var isLast = i === last;
                        var message_1 = void 0;
                        if (isLast) {
                            message_1 = ('[0,' + id + ',' + ref_1.buffer[i] + ']');
                        }
                        else {
                            message_1 = ('[-2,' + id + ',' + ref_1.buffer[i] + ']');
                        }
                        _this.send(message_1);
                    }
                }
                finally {
                    ref_1.buffer = [];
                }
            });
            if (!done_1)
                this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    };
    JsonRxServerJson.prototype.onUnsubscribe = function (message) {
        var _a = (0, tslib_1.__read)(message, 2), id = _a[1];
        (0, util_1.assertId)(id);
        var subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    };
    JsonRxServerJson.prototype.onNotification = function (message, ctx) {
        var _a = (0, tslib_1.__read)(message, 2), name = _a[0], payload = _a[1];
        (0, util_1.assertName)(name);
        this.notify(name, payload, ctx);
    };
    JsonRxServerJson.prototype.onMessage = function (message, ctx) {
        if (!(0, util_1.isArray)(message))
            throw new Error('Invalid message');
        var _a = (0, tslib_1.__read)(message, 1), one = _a[0];
        if ((0, util_1.isArray)(one)) {
            for (var i = 0; i < message.length; i++)
                this.onMessage(message[i], ctx);
            return;
        }
        if (typeof one === 'string')
            return this.onNotification(message, ctx);
        if (one > 0)
            return this.onSubscribe(message, ctx);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    };
    JsonRxServerJson.prototype.stop = function () {
        var e_2, _a;
        this.send = function (message) { };
        try {
            for (var _b = (0, tslib_1.__values)(this.active.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sub = _c.value;
                sub.unsubscribe();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    return JsonRxServerJson;
}());
exports.JsonRxServerJson = JsonRxServerJson;
