"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonSizeApprox = exports.jsonSize = void 0;
var utf8_1 = require("../util/utf8");
var numberSize = function (num) {
    var isInteger = num === Math.round(num);
    if (isInteger)
        return Math.max(Math.floor(Math.log10(Math.abs(num))), 0) + 1 + (num < 0 ? 1 : 0);
    return JSON.stringify(num).length;
};
var stringSize = function (str) {
    var strLength = str.length;
    var byteLength = strLength;
    var pos = 0;
    while (pos < strLength) {
        var value = str.charCodeAt(pos++);
        if (value < 128) {
            switch (value) {
                case 8:
                case 9:
                case 10:
                case 12:
                case 13:
                case 34:
                case 92:
                    byteLength += 1;
                    break;
            }
            continue;
        }
        else
            return (0, utf8_1.utf8Count)(JSON.stringify(str));
    }
    return byteLength + 2;
};
var booleanSize = function (bool) { return (bool ? 4 : 5); };
var arraySize = function (arr) {
    var size = 0;
    var length = arr.length;
    for (var i = 0; i < length; i++)
        size += (0, exports.jsonSize)(arr[i]);
    return size + 2 + (length > 1 ? length - 1 : 0);
};
var objectSize = function (obj) {
    var size = 2;
    var length = 0;
    for (var key in obj)
        if (obj.hasOwnProperty(key)) {
            length++;
            size += stringSize(key) + (0, exports.jsonSize)(obj[key]);
        }
    var colonSize = length;
    var commaSize = length > 1 ? length - 1 : 0;
    return size + colonSize + commaSize;
};
var jsonSize = function (value) {
    if (value === null)
        return 4;
    switch (typeof value) {
        case 'number':
            return numberSize(value);
        case 'string':
            return stringSize(value);
        case 'boolean':
            return booleanSize(value);
    }
    if (value instanceof Array)
        return arraySize(value);
    return objectSize(value);
};
exports.jsonSize = jsonSize;
var jsonSizeApprox = function (value) {
    if (value === null)
        return 4;
    switch (typeof value) {
        case 'number':
            return numberSize(value);
        case 'string':
            return value.length;
        case 'boolean':
            return booleanSize(value);
    }
    if (value instanceof Array)
        return arraySize(value);
    return objectSize(value);
};
exports.jsonSizeApprox = jsonSizeApprox;
