"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.msgpackSizeFast = void 0;
var json_pack_1 = require("../json-pack");
var isUint8Array_1 = require("../util/isUint8Array");
var arraySize = function (arr) {
    var size = 2;
    for (var i = arr.length - 1; i >= 0; i--)
        size += (0, exports.msgpackSizeFast)(arr[i]);
    return size;
};
var objectSize = function (obj) {
    var size = 2;
    for (var key in obj)
        if (obj.hasOwnProperty(key))
            size += 2 + key.length + (0, exports.msgpackSizeFast)(obj[key]);
    return size;
};
var msgpackSizeFast = function (value) {
    if (value === null)
        return 1;
    switch (typeof value) {
        case 'number':
            return 9;
        case 'string':
            return 4 + value.length;
        case 'boolean':
            return 1;
    }
    if (value instanceof Array)
        return arraySize(value);
    if ((0, isUint8Array_1.isUint8Array)(value))
        return 5 + value.byteLength;
    if (value instanceof json_pack_1.JsonPackValue)
        return value.buf.byteLength;
    if (value instanceof json_pack_1.JsonPackExtension)
        return 6 + value.buf.byteLength;
    return objectSize(value);
};
exports.msgpackSizeFast = msgpackSizeFast;
