"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toJsonSchema = void 0;
var tslib_1 = require("tslib");
var util_1 = require("../json-type/util");
var augmentNode = function (type, node) {
    if (type.title)
        node.title = type.title;
    if (type.description)
        node.description = type.description;
    if (type.examples)
        node.examples = type.examples.map(function (example) { return example.value; });
};
var toJsonSchema = function (type, ctx) {
    var e_1, _a;
    switch (type.__t) {
        case 'str': {
            var node = { type: 'string' };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'num': {
            var node = {
                type: 'number',
            };
            if (type.format)
                if (util_1.INTS.indexOf(type.format) > -1)
                    node.type = 'integer';
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'bool': {
            var node = {
                type: 'boolean',
            };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'nil': {
            var node = {
                type: 'null',
            };
            augmentNode(type, node);
            return node;
        }
        case 'arr': {
            var node = {
                type: 'array',
                items: (0, exports.toJsonSchema)(type.type, ctx),
            };
            if (type.const)
                node.const = type.const;
            augmentNode(type, node);
            return node;
        }
        case 'obj': {
            var node = {
                type: 'object',
                properties: {},
            };
            var required = [];
            try {
                for (var _b = (0, tslib_1.__values)(type.fields), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var field = _c.value;
                    node.properties[field.key] = (0, exports.toJsonSchema)(field.type, ctx);
                    if (field.isOptional === undefined || !field.isOptional)
                        required.push(field.key);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (required.length)
                node.required = required;
            augmentNode(type, node);
            return node;
        }
        case 'ref': {
            if (!ctx.ref) {
                var node = { $ref: '/' + type.ref };
                return node;
            }
            else if (ctx.defs) {
                var node = { $ref: '#/$defs/' + type.ref };
                if (!ctx.defs[type.ref])
                    ctx.defs[type.ref] = (0, exports.toJsonSchema)(ctx.ref(type.ref), ctx);
                return node;
            }
            return (0, exports.toJsonSchema)(ctx.ref(type.ref), ctx);
        }
        case 'any':
        default: {
            var node = {
                type: ['string', 'number', 'boolean', 'null', 'array', 'object'],
            };
            augmentNode(type, node);
            return node;
        }
    }
};
exports.toJsonSchema = toJsonSchema;
