"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonSerializerCodegen = void 0;
var tslib_1 = require("tslib");
var codegen_1 = require("../util/codegen");
var JsExpression_1 = require("../util/codegen/util/JsExpression");
var asString_1 = require("../util/asString");
var normalizeAccessor_1 = require("../util/codegen/util/normalizeAccessor");
var encode_1 = require("../util/base64/encode");
var WriteTextStep = (function () {
    function WriteTextStep(str) {
        this.str = str;
    }
    return WriteTextStep;
}());
var JsonSerializerCodegen = (function () {
    function JsonSerializerCodegen(opts) {
        this.base64Linked = false;
        this.options = (0, tslib_1.__assign)({ ref: function (id) { return undefined; } }, opts);
        var typeNamePart = this.options.type.id && /^[a-z][a-z0-9_]*$/i.test(this.options.type.id)
            ? this.options.type.id[0].toUpperCase() + this.options.type.id.substr(1)
            : '';
        this.codegen = new codegen_1.Codegen({
            name: 'toJson' + typeNamePart,
            prologue: "var s = '';",
            epilogue: "return s;",
            processSteps: function (steps) {
                var e_1, _a;
                var stepsJoined = [];
                for (var i = 0; i < steps.length; i++) {
                    var step = steps[i];
                    if (step instanceof codegen_1.CodegenStepExecJs)
                        stepsJoined.push(step);
                    else if (step instanceof WriteTextStep) {
                        var last = stepsJoined[stepsJoined.length - 1];
                        if (last instanceof WriteTextStep)
                            last.str += step.str;
                        else
                            stepsJoined.push(step);
                    }
                }
                var execSteps = [];
                try {
                    for (var stepsJoined_1 = (0, tslib_1.__values)(stepsJoined), stepsJoined_1_1 = stepsJoined_1.next(); !stepsJoined_1_1.done; stepsJoined_1_1 = stepsJoined_1.next()) {
                        var step = stepsJoined_1_1.value;
                        if (step instanceof codegen_1.CodegenStepExecJs) {
                            execSteps.push(step);
                        }
                        else if (step instanceof WriteTextStep) {
                            var js = "s += ".concat(JSON.stringify(step.str), ";");
                            execSteps.push(new codegen_1.CodegenStepExecJs(js));
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (stepsJoined_1_1 && !stepsJoined_1_1.done && (_a = stepsJoined_1.return)) _a.call(stepsJoined_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return execSteps;
            },
        });
        this.codegen.linkDependency(asString_1.asString, 'asString');
        this.codegen.linkDependency(JSON.stringify, 'stringify');
    }
    JsonSerializerCodegen.prototype.linkBase64 = function () {
        if (this.base64Linked)
            return;
        this.codegen.linkDependency(encode_1.toBase64, 'toBase64');
    };
    JsonSerializerCodegen.prototype.js = function (js) {
        this.codegen.js(js);
    };
    JsonSerializerCodegen.prototype.writeText = function (str) {
        this.codegen.step(new WriteTextStep(str));
    };
    JsonSerializerCodegen.prototype.onString = function (str, value) {
        if (str.const !== undefined) {
            this.writeText(JSON.stringify(str.const));
            return;
        }
        if (str.noJsonEscape) {
            this.js("s += ".concat(value.use(), ";"));
            return;
        }
        this.js("s += asString(".concat(value.use(), ");"));
    };
    JsonSerializerCodegen.prototype.onNumber = function (num, value) {
        if (num.const !== undefined) {
            this.writeText(JSON.stringify(num.const));
            return;
        }
        this.js("s += ".concat(value.use(), ";"));
    };
    JsonSerializerCodegen.prototype.onBoolean = function (bool, value) {
        if (bool.const !== undefined) {
            this.writeText(JSON.stringify(bool.const));
            return;
        }
        this.js("s += ".concat(value.use(), " ? 'true' : 'false';"));
    };
    JsonSerializerCodegen.prototype.onNull = function () {
        this.writeText('null');
    };
    JsonSerializerCodegen.prototype.onArray = function (arr, value) {
        if (arr.const !== undefined) {
            this.writeText(JSON.stringify(arr.const));
            return;
        }
        this.writeText('[');
        var r = this.codegen.getRegister();
        var rl = this.codegen.getRegister();
        var ri = this.codegen.getRegister();
        var rItem = this.codegen.getRegister();
        this.js("var ".concat(r, " = ").concat(value.use(), ", ").concat(rl, " = ").concat(r, ".length, ").concat(ri, " = 0, ").concat(rItem, ";"));
        this.js("for(; ".concat(ri, " < ").concat(rl, "; ").concat(ri, "++) ") + '{');
        this.js("".concat(rItem, " = ").concat(r, "[").concat(ri, "];"));
        this.js("if (".concat(ri, ") s += ',';"));
        this.onType(arr.type, new JsExpression_1.JsExpression(function () { return "".concat(rItem); }));
        this.js("}");
        this.writeText(']');
    };
    JsonSerializerCodegen.prototype.onObject = function (obj, value) {
        if (obj.unknownFields) {
            this.js("s += stringify(".concat(value.use(), ");"));
            return;
        }
        var requiredFields = obj.fields.filter(function (field) { return !field.isOptional; });
        var optionalFields = obj.fields.filter(function (field) { return field.isOptional; });
        this.writeText('{');
        var _loop_1 = function (i) {
            var field = requiredFields[i];
            if (i)
                this_1.writeText(',');
            this_1.writeText(JSON.stringify(field.key) + ':');
            var accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            this_1.onType(field.type, new JsExpression_1.JsExpression(function () { return "".concat(value.use()).concat(accessor); }));
        };
        var this_1 = this;
        for (var i = 0; i < requiredFields.length; i++) {
            _loop_1(i);
        }
        var rHasFields = this.codegen.getRegister();
        if (!requiredFields.length) {
            this.js("var ".concat(rHasFields, " = false;"));
        }
        var _loop_2 = function (i) {
            var field = optionalFields[i];
            var accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            var rValue = this_2.codegen.getRegister();
            this_2.js("var ".concat(rValue, " = ").concat(value.use()).concat(accessor, ";"));
            this_2.js("if (".concat(rValue, " !== undefined) {"));
            if (requiredFields.length) {
                this_2.writeText(',');
            }
            else {
                this_2.js("if (".concat(rHasFields, ") s += ',';"));
                this_2.js("".concat(rHasFields, " = true;"));
            }
            this_2.writeText(JSON.stringify(field.key) + ':');
            this_2.onType(field.type, new JsExpression_1.JsExpression(function () { return "".concat(rValue); }));
            this_2.js("}");
        };
        var this_2 = this;
        for (var i = 0; i < optionalFields.length; i++) {
            _loop_2(i);
        }
        this.writeText('}');
    };
    JsonSerializerCodegen.prototype.onBinary = function (value) {
        this.linkBase64();
        this.writeText('"data:application/octet-stream;base64,');
        this.js("s += toBase64(".concat(value.use(), ");"));
        this.writeText('"');
    };
    JsonSerializerCodegen.prototype.onRef = function (ref, value) {
        var type = this.options.ref(ref.ref);
        switch (typeof type) {
            case 'function': {
                var d = this.codegen.linkDependency(type);
                this.js("s += ".concat(d, "(").concat(value.use(), ");"));
                break;
            }
            case 'object': {
                this.onType(type, value);
                break;
            }
            default:
                throw new Error("Unknown [ref = ".concat(ref.ref, "]."));
        }
    };
    JsonSerializerCodegen.prototype.onAny = function (value) {
        this.js("s += stringify(".concat(value.use(), ");"));
    };
    JsonSerializerCodegen.prototype.onType = function (type, value) {
        switch (type.__t) {
            case 'str': {
                this.onString(type, value);
                break;
            }
            case 'num': {
                this.onNumber(type, value);
                break;
            }
            case 'bool': {
                this.onBoolean(type, value);
                break;
            }
            case 'nil': {
                this.onNull();
                break;
            }
            case 'arr': {
                this.onArray(type, value);
                break;
            }
            case 'obj': {
                this.onObject(type, value);
                break;
            }
            case 'bin': {
                this.onBinary(value);
                break;
            }
            case 'ref': {
                this.onRef(type, value);
                break;
            }
            default: {
                this.onAny(value);
                break;
            }
        }
    };
    JsonSerializerCodegen.prototype.run = function () {
        var r = this.codegen.getRegister();
        var value = new JsExpression_1.JsExpression(function () { return r; });
        this.onType(this.options.type, value);
        return this;
    };
    JsonSerializerCodegen.prototype.generate = function () {
        return this.codegen.generate();
    };
    JsonSerializerCodegen.prototype.compile = function () {
        return this.codegen.compile();
    };
    return JsonSerializerCodegen;
}());
exports.JsonSerializerCodegen = JsonSerializerCodegen;
