"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MsgPackSerializerCodegen = void 0;
var tslib_1 = require("tslib");
var util_1 = require("../json-pack/util");
var JsExpression_1 = require("../util/codegen/util/JsExpression");
var normalizeAccessor_1 = require("../util/codegen/util/normalizeAccessor");
var codegen_1 = require("../util/codegen");
var UINTS = ['u', 'u8', 'u16', 'u32', 'u64'];
var INTS = (0, tslib_1.__spreadArray)(['i', 'i8', 'i16', 'i32', 'i64'], (0, tslib_1.__read)(UINTS), false);
var join = function (a, b) {
    var res = new Uint8Array(a.length + b.length);
    res.set(a);
    res.set(b, a.length);
    return res;
};
var WriteBlobStep = (function () {
    function WriteBlobStep(arr) {
        this.arr = arr;
    }
    return WriteBlobStep;
}());
var MsgPackSerializerCodegen = (function () {
    function MsgPackSerializerCodegen(opts) {
        var _this = this;
        this.registerCounter = 0;
        this.options = (0, tslib_1.__assign)({ ref: function (id) { return undefined; } }, opts);
        this.codegen = new codegen_1.Codegen({
            name: 'toMsgPack',
            args: 'r0',
            prologue: 'e.reset();',
            epilogue: 'return e.flush();',
            processSteps: function (steps) {
                var e_1, _a;
                var stepsJoined = [];
                for (var i = 0; i < steps.length; i++) {
                    var step = steps[i];
                    if (step instanceof codegen_1.CodegenStepExecJs)
                        stepsJoined.push(step);
                    else if (step instanceof WriteBlobStep) {
                        var last = stepsJoined[stepsJoined.length - 1];
                        if (last instanceof WriteBlobStep)
                            last.arr = join(last.arr, step.arr);
                        else
                            stepsJoined.push(step);
                    }
                }
                var execSteps = [];
                try {
                    for (var stepsJoined_1 = (0, tslib_1.__values)(stepsJoined), stepsJoined_1_1 = stepsJoined_1.next(); !stepsJoined_1_1.done; stepsJoined_1_1 = stepsJoined_1.next()) {
                        var step = stepsJoined_1_1.value;
                        if (step instanceof codegen_1.CodegenStepExecJs) {
                            execSteps.push(step);
                        }
                        else if (step instanceof WriteBlobStep) {
                            execSteps.push(_this.codegenBlob(step));
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (stepsJoined_1_1 && !stepsJoined_1_1.done && (_a = stepsJoined_1.return)) _a.call(stepsJoined_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return execSteps;
            },
        });
        this.codegen.linkDependency(this.options.encoder, 'e');
    }
    MsgPackSerializerCodegen.prototype.execJs = function (js) {
        this.codegen.js(js);
    };
    MsgPackSerializerCodegen.prototype.writeBlob = function (arr) {
        this.codegen.step(new WriteBlobStep(arr));
    };
    MsgPackSerializerCodegen.prototype.getBlob = function (callback) {
        util_1.encoder.reset();
        callback(util_1.encoder);
        return util_1.encoder.flush();
    };
    MsgPackSerializerCodegen.prototype.getRegister = function () {
        return "r".concat(this.registerCounter++);
    };
    MsgPackSerializerCodegen.prototype.genAndWriteBlob = function (callback) {
        this.writeBlob(this.getBlob(callback));
    };
    MsgPackSerializerCodegen.prototype.onString = function (str, value) {
        if (str.const !== undefined) {
            this.writeBlob(util_1.encoder.encode(str.const));
            return;
        }
        if (str.format === 'ascii') {
            this.execJs("e.encodeAsciiString(".concat(value.use(), ");"));
            return;
        }
        this.execJs("e.encodeString(".concat(value.use(), ");"));
    };
    MsgPackSerializerCodegen.prototype.onNumber = function (num, value) {
        if (num.const !== undefined) {
            this.writeBlob(util_1.encoder.encode(num.const));
            return;
        }
        var isInteger = INTS.indexOf(num.format) > -1;
        if (isInteger) {
            var isUnsignedInteger = UINTS.indexOf(num.format) > -1;
            if (isUnsignedInteger) {
                this.execJs("e.encodeUnsignedInteger(".concat(value.use(), ");"));
            }
            else {
                this.execJs("e.encodeInteger(".concat(value.use(), ");"));
            }
            return;
        }
        this.execJs("e.encodeNumber(".concat(value.use(), ");"));
    };
    MsgPackSerializerCodegen.prototype.onBoolean = function (bool, value) {
        if (bool.const !== undefined) {
            this.writeBlob(util_1.encoder.encode(bool.const));
            return;
        }
        this.execJs("e.encodeBoolean(".concat(value.use(), ");"));
    };
    MsgPackSerializerCodegen.prototype.onNull = function () {
        this.execJs("e.encodeNull();");
    };
    MsgPackSerializerCodegen.prototype.onArray = function (arr, value) {
        if (arr.const !== undefined) {
            this.genAndWriteBlob(function (encoder) { return encoder.encodeArray(arr.const); });
            return;
        }
        var r = this.getRegister();
        var rl = this.getRegister();
        var ri = this.getRegister();
        var rItem = this.getRegister();
        this.execJs("var ".concat(r, " = ").concat(value.use(), ", ").concat(rl, " = ").concat(r, ".length, ").concat(ri, " = 0, ").concat(rItem, ";"));
        this.execJs("e.encodeArrayHeader(".concat(rl, ");"));
        this.execJs("for(; ".concat(ri, " < ").concat(rl, "; ").concat(ri, "++) ") + '{');
        this.execJs("".concat(rItem, " = ").concat(r, "[").concat(ri, "];"));
        this.onType(arr.type, new JsExpression_1.JsExpression(function () { return "".concat(rItem); }));
        this.execJs("}");
    };
    MsgPackSerializerCodegen.prototype.onObject = function (obj, value) {
        if (obj.unknownFields) {
            this.execJs("e.encodeObject(".concat(value.use(), ");"));
            return;
        }
        var hasOptionalFields = false;
        for (var i = 0; i < obj.fields.length; i++) {
            if (obj.fields[i].isOptional) {
                hasOptionalFields = true;
                break;
            }
        }
        if (hasOptionalFields)
            this.onObjectVariableLength(obj, value);
        else
            this.onObjectFixedLength(obj, value);
    };
    MsgPackSerializerCodegen.prototype.onObjectVariableLength = function (obj, value) {
        var requiredFields = [];
        var optionalFields = [];
        for (var i = 0; i < obj.fields.length; i++) {
            var field = obj.fields[i];
            if (field.isOptional)
                optionalFields.push(field);
            else
                requiredFields.push(field);
        }
        var r = this.getRegister();
        var rk = this.getRegister();
        var rv = this.getRegister();
        this.execJs("var ".concat(r, " = ").concat(value.use(), ";"));
        this.execJs("var ".concat(rk, " = Object.keys(").concat(r, ");"));
        this.execJs("e.encodeObjectHeader(".concat(rk, ".length);"));
        this.onRequiredFields(requiredFields, value, r);
        var _loop_1 = function (i) {
            var field = optionalFields[i];
            var accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            this_1.execJs("var ".concat(rv, " = ").concat(r).concat(accessor, ";"));
            this_1.execJs("if (".concat(rv, " !== undefined) {"));
            this_1.genAndWriteBlob(function (encoder) { return encoder.encodeString(field.key); });
            this_1.onType(field.type, new JsExpression_1.JsExpression(function () { return rv; }));
            this_1.execJs('}');
        };
        var this_1 = this;
        for (var i = 0; i < optionalFields.length; i++) {
            _loop_1(i);
        }
    };
    MsgPackSerializerCodegen.prototype.onObjectFixedLength = function (obj, value) {
        var _this = this;
        var length = obj.fields.length;
        this.genAndWriteBlob(function (encoder) { return encoder.encodeObjectHeader(length); });
        var r = this.getRegister();
        value.addListener(function (expr) {
            _this.execJs("var ".concat(r, " = ").concat(expr, ";"));
        });
        this.onRequiredFields(obj.fields, value, r);
    };
    MsgPackSerializerCodegen.prototype.onRequiredFields = function (requiredFields, value, r) {
        var length = requiredFields.length;
        var _loop_2 = function (i) {
            var field = requiredFields[i];
            this_2.genAndWriteBlob(function (encoder) { return encoder.encodeString(field.key); });
            var accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            this_2.onType(field.type, value.chain(function () { return "".concat(r).concat(accessor); }));
        };
        var this_2 = this;
        for (var i = 0; i < length; i++) {
            _loop_2(i);
        }
    };
    MsgPackSerializerCodegen.prototype.onBinary = function (value) {
        this.execJs("e.encodeBinary(".concat(value.use(), ");"));
    };
    MsgPackSerializerCodegen.prototype.onRef = function (ref, value) {
        var type = this.options.ref(ref.ref);
        switch (typeof type) {
            case 'function': {
                var d = this.codegen.linkDependency(type);
                this.execJs("".concat(d, "(").concat(value.use(), ", e);"));
                break;
            }
            case 'object': {
                this.onType(type, value);
                break;
            }
            default:
                throw new Error("Unknown [ref = ".concat(ref.ref, "]."));
        }
    };
    MsgPackSerializerCodegen.prototype.onAny = function (value) {
        this.execJs("e.encodeAny(".concat(value.use(), ");"));
    };
    MsgPackSerializerCodegen.prototype.onType = function (type, value) {
        switch (type.__t) {
            case 'str': {
                this.onString(type, value);
                break;
            }
            case 'num': {
                this.onNumber(type, value);
                break;
            }
            case 'bool': {
                this.onBoolean(type, value);
                break;
            }
            case 'nil': {
                this.onNull();
                break;
            }
            case 'arr': {
                this.onArray(type, value);
                break;
            }
            case 'obj': {
                this.onObject(type, value);
                break;
            }
            case 'bin': {
                this.onBinary(value);
                break;
            }
            case 'ref': {
                this.onRef(type, value);
                break;
            }
            default: {
                this.onAny(value);
                break;
            }
        }
    };
    MsgPackSerializerCodegen.prototype.codegenBlob = function (step) {
        var lines = [];
        var ro = this.getRegister();
        var ru = this.getRegister();
        lines.push("var ".concat(ro, " = e.offset, ").concat(ru, " = e.uint8;"));
        lines.push("e.ensureCapacity(".concat(step.arr.length, ");"));
        lines.push("e.offset += ".concat(step.arr.length, ";"));
        for (var i = 0; i < step.arr.length; i++)
            lines.push("".concat(ru, "[").concat(ro, " + ").concat(i, "] = ").concat(step.arr[i], ";"));
        var js = lines.join('\n');
        return new codegen_1.CodegenStepExecJs(js);
    };
    MsgPackSerializerCodegen.prototype.run = function () {
        var r = this.getRegister();
        var value = new JsExpression_1.JsExpression(function () { return r; });
        this.onType(this.options.type, value);
        return this;
    };
    MsgPackSerializerCodegen.prototype.generate = function () {
        return this.codegen.generate();
    };
    MsgPackSerializerCodegen.prototype.compile = function () {
        return this.codegen.compile();
    };
    MsgPackSerializerCodegen.prototype.compilePartial = function () {
        return this.codegen.compile({
            name: 'partial',
            args: 'r0, e',
            epilogue: '',
            prologue: '',
        });
    };
    return MsgPackSerializerCodegen;
}());
exports.MsgPackSerializerCodegen = MsgPackSerializerCodegen;
