"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportDeclaration = void 0;
var tslib_1 = require("tslib");
var parseIdentifier = function (id) {
    var parts = id.split('.');
    return {
        name: parts.pop(),
        namespaces: parts,
    };
};
var augmentWithComment = function (type, node) {
    if (type.title || type.description) {
        var comment = '';
        if (type.title)
            comment += '# ' + type.title;
        if (type.title && type.description)
            comment += '\n\n';
        if (type.description)
            comment += type.description;
        node.comment = comment;
    }
};
var exportDeclaration = function (ref, ctx) {
    var e_1, _a, e_2, _b;
    var identifier = parseIdentifier(ref);
    var module = {
        node: 'ModuleDeclaration',
        name: '',
        statements: ctx.statements,
    };
    var _loop_1 = function (namespace) {
        var innerModule = module.statements.find(function (s) { return s.node === 'ModuleDeclaration' && s.name === namespace; });
        if (!innerModule) {
            innerModule = {
                node: 'ModuleDeclaration',
                name: namespace,
                statements: [],
            };
            module.statements.push(innerModule);
        }
        module = innerModule;
    };
    try {
        for (var _c = (0, tslib_1.__values)(identifier.namespaces), _d = _c.next(); !_d.done; _d = _c.next()) {
            var namespace = _d.value;
            _loop_1(namespace);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
        }
        finally { if (e_1) throw e_1.error; }
    }
    var exists = module.statements.find(function (s) { return s.name === identifier.name; });
    if (exists)
        return;
    var type = ctx.ref(ref);
    switch (type.__t) {
        case 'obj': {
            var node = {
                node: 'InterfaceDeclaration',
                name: identifier.name,
                members: [],
            };
            module.statements.push(node);
            try {
                for (var _e = (0, tslib_1.__values)(type.fields), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var field = _f.value;
                    var member = {
                        node: 'PropertySignature',
                        name: field.key,
                        type: toTsType(field.type, ctx),
                    };
                    if (field.isOptional)
                        member.optional = true;
                    augmentWithComment(field, member);
                    node.members.push(member);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                }
                finally { if (e_2) throw e_2.error; }
            }
            if (type.unknownFields)
                node.members.push({
                    node: 'IndexSignature',
                    type: { node: 'UnknownKeyword' },
                });
            augmentWithComment(type, node);
            return;
        }
        default: {
            var node = {
                node: 'TypeAliasDeclaration',
                name: identifier.name,
                type: toTsType(type, ctx),
            };
            augmentWithComment(type, node);
            module.statements.push(node);
            return;
        }
    }
};
exports.exportDeclaration = exportDeclaration;
var toTsType = function (type, ctx) {
    var e_3, _a;
    switch (type.__t) {
        case 'str': {
            var node = typeof type.const === 'string' ? { node: 'StringLiteral', text: type.const } : { node: 'StringKeyword' };
            return node;
        }
        case 'num': {
            var node = typeof type.const === 'number'
                ? { node: 'NumericLiteral', text: JSON.stringify(type.const) }
                : { node: 'NumberKeyword' };
            return node;
        }
        case 'bool': {
            var node = typeof type.const === 'boolean'
                ? type.const
                    ? { node: 'TrueKeyword' }
                    : { node: 'FalseKeyword' }
                : { node: 'BooleanKeyword' };
            return node;
        }
        case 'nil': {
            var node = { node: 'NullKeyword' };
            return node;
        }
        case 'arr': {
            var node = {
                node: 'ArrayType',
                elementType: toTsType(type.type, ctx),
            };
            return node;
        }
        case 'obj': {
            var node = {
                node: 'TypeLiteral',
                members: [],
            };
            try {
                for (var _b = (0, tslib_1.__values)(type.fields), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var field = _c.value;
                    var member = {
                        node: 'PropertySignature',
                        name: field.key,
                        type: toTsType(field.type, ctx),
                    };
                    if (field.isOptional)
                        member.optional = true;
                    augmentWithComment(field, member);
                    node.members.push(member);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
            if (type.unknownFields)
                node.members.push({
                    node: 'IndexSignature',
                    type: { node: 'UnknownKeyword' },
                });
            augmentWithComment(type, node);
            return node;
        }
        case 'enum': {
            var node = {
                node: 'UnionType',
                types: type.values.map(toTsLiteral),
            };
            return node;
        }
        case 'or': {
            var node = {
                node: 'UnionType',
                types: type.types.map(function (t) { return toTsType(t, ctx); }),
            };
            return node;
        }
        case 'ref': {
            var node = {
                node: 'TypeReference',
                typeName: type.ref,
            };
            (0, exports.exportDeclaration)(type.ref, ctx);
            return node;
        }
        default: {
            var node = { node: 'UnknownKeyword' };
            return node;
        }
    }
};
var toTsLiteral = function (value) {
    switch (typeof value) {
        case 'string': {
            var node = { node: 'StringLiteral', text: value };
            return node;
        }
        case 'number': {
            var node = { node: 'NumericLiteral', text: value.toString() };
            return node;
        }
        case 'boolean': {
            var node = { node: value ? 'TrueKeyword' : 'FalseKeyword' };
            return node;
        }
        default: {
            var node = { node: 'UnknownKeyword' };
            return node;
        }
    }
};
