"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toTypeScriptText = void 0;
var tslib_1 = require("tslib");
var wordWrap_1 = require("../util/wordWrap");
var util_1 = require("./util");
var TAB = '  ';
var formatComment = function (node, __) {
    var e_1, _a;
    if (node.title || node.description) {
        if (!node.description) {
            return "".concat(__, "/** ").concat(node.title, " */\n");
        }
        var res = '';
        res += "".concat(__, "/**\n");
        if (node.title)
            res += "".concat(__, " * # ").concat(node.title, "\n");
        if (node.title && node.description)
            res += "".concat(__, " *\n");
        if (node.description) {
            var lines = (0, wordWrap_1.wordWrap)(node.description);
            try {
                for (var lines_1 = (0, tslib_1.__values)(lines), lines_1_1 = lines_1.next(); !lines_1_1.done; lines_1_1 = lines_1.next()) {
                    var line = lines_1_1.value;
                    res += "".concat(__, " * ").concat(line.trim(), "\n");
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (lines_1_1 && !lines_1_1.done && (_a = lines_1.return)) _a.call(lines_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        res += "".concat(__, " */\n");
        return res;
    }
    return '';
};
var toTypeScriptText = function (type, tab) {
    if (type.const) {
        return JSON.stringify(type.const);
    }
    switch (type.__t) {
        case 'str': {
            return 'string';
        }
        case 'num': {
            return 'number';
        }
        case 'bool': {
            return 'boolean';
        }
        case 'nil': {
            return 'null';
        }
        case 'arr': {
            return "Array<".concat((0, exports.toTypeScriptText)(type.type, tab + TAB), ">");
        }
        case 'enum': {
            return "[".concat(type.values.map(function (t) { return (0, exports.toTypeScriptText)(t, tab + TAB); }).join(', '), "]");
        }
        case 'obj': {
            var ____ = tab + TAB;
            if (type.fields.length === 0)
                return 'object';
            var res = '{\n';
            for (var i = 0; i < type.fields.length; i++) {
                var field = type.fields[i];
                var key = (0, util_1.normalizeKey)(field.key) + (field.isOptional ? '?' : '');
                if (i)
                    if (field.key || field.description)
                        res += "\n";
                res += formatComment(field, ____);
                res += "".concat(____).concat(key, ": ").concat((0, exports.toTypeScriptText)(field.type, ____), ";\n");
            }
            res += tab + '}';
            return res;
        }
        case 'ref': {
            return type.ref;
        }
        case 'or': {
            return type.types.map(function (t) { return (0, exports.toTypeScriptText)(t, tab + TAB); }).join(' | ');
        }
        case 'bin': {
            return 'Uint8Array';
        }
        case 'any': {
            return 'any';
        }
        default: {
            return 'any';
        }
    }
};
exports.toTypeScriptText = toTypeScriptText;
