import type { JsonTypeValidator } from './types';
import type { TType, TArray, TBoolean, TNumber, TObject, TString, TRef, TOr, TEnum } from '../json-type/types';
import { JavaScriptLinked } from '../util/codegen';
import { BooleanValidator, CustomValidator, CustomValidatorType, ObjectValidator, StringValidator } from '.';
declare type Path = Array<string | number | {
    r: string;
}>;
declare class EncodingPlanStepExecJs {
    readonly js: string;
    constructor(js: string);
}
export declare enum JsonTypeValidatorError {
    STR = 0,
    STR_CONST = 1,
    NUM = 2,
    NUM_CONST = 3,
    BOOL = 4,
    BOOL_CONST = 5,
    NIL = 6,
    ARR = 7,
    OBJ = 8,
    KEY = 9,
    KEYS = 10,
    BIN = 11,
    OR = 12,
    REF = 13,
    ENUM = 14,
    VALIDATION = 15,
    INT = 16,
    UINT = 17,
    STR_LEN = 18
}
export declare const JsonTypeValidatorErrorMessage: {
    0: string;
    1: string;
    2: string;
    3: string;
    4: string;
    5: string;
    6: string;
    7: string;
    8: string;
    9: string;
    10: string;
    11: string;
    12: string;
    13: string;
    14: string;
    15: string;
    16: string;
    17: string;
    18: string;
};
export interface JsonTypeValidatorCodegenOptions {
    errorReporting?: 'boolean' | 'string' | 'object';
    skipObjectExtraFieldsCheck?: boolean;
    unsafeMode?: boolean;
    ref?: (id: string) => JsonTypeValidator | undefined;
    customValidators?: CustomValidator[];
}
export declare class JsonTypeValidatorCodegen {
    protected steps: EncodingPlanStepExecJs[];
    protected options: Required<JsonTypeValidatorCodegenOptions>;
    protected type: TType;
    constructor(options?: JsonTypeValidatorCodegenOptions);
    protected js(js: string): void;
    protected registerCounter: number;
    protected getRegister(): string;
    protected dependencies: unknown[];
    protected addDependencies(deps: unknown[]): string[];
    protected consts: string[];
    protected addConsts(consts: string[]): string[];
    protected err(code: JsonTypeValidatorError, path: Path, opts?: {
        refError?: string;
        validator?: string;
    }): string;
    protected refErr(path: Path, r: string): string;
    protected emitValidations(path: Path, validatorNames: string[], type: CustomValidatorType, r: string): void;
    protected linkValidator(name: string, type: CustomValidatorType): string;
    protected onString(path: Path, str: TString, r: string): void;
    protected onNumber(path: Path, num: TNumber, r: string): void;
    protected onBoolean(path: Path, bool: TBoolean, r: string): void;
    protected onNull(path: Path, r: string): void;
    protected onArray(path: Path, arr: TArray, expr: string): void;
    protected onObject(path: Path, obj: TObject, expr: string): void;
    protected onBinary(path: Path, expr: string): void;
    protected onRef(path: Path, ref: TRef, expr: string): void;
    protected onOr(path: Path, or: TOr, expr: string): void;
    protected onEnum(path: Path, type: TEnum, r: string): void;
    protected onType(path: Path, type: TType, expr: string): void;
    generate(type: TType): JavaScriptLinked<JsonTypeValidator>;
}
export declare const createValidator: (type: TType, options?: JsonTypeValidatorCodegenOptions) => JsonTypeValidator;
export declare const createBoolValidator: (type: TType, options?: Omit<JsonTypeValidatorCodegenOptions, 'errorReporting'>) => BooleanValidator;
export declare const createStrValidator: (type: TType, options?: Omit<JsonTypeValidatorCodegenOptions, 'errorReporting'>) => StringValidator;
export declare const createObjValidator: (type: TType, options?: Omit<JsonTypeValidatorCodegenOptions, 'errorReporting'>) => ObjectValidator;
export {};
