"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.createObjValidator = exports.createStrValidator = exports.createBoolValidator = exports.createValidator = exports.JsonTypeValidatorCodegen = exports.JsonTypeValidatorErrorMessage = exports.JsonTypeValidatorError = void 0;
var tslib_1 = require("tslib");
var codegen_1 = require("../util/codegen");
var __deepEqual_1 = require("../json-equal/$$deepEqual");
var normalizeAccessor_1 = require("../util/codegen/util/normalizeAccessor");
var util_1 = require("../json-type/util");
var EncodingPlanStepExecJs = (function () {
    function EncodingPlanStepExecJs(js) {
        this.js = js;
    }
    return EncodingPlanStepExecJs;
}());
var canSkipObjectKeyUndefinedCheck = function (type) {
    switch (type) {
        case 'bool':
        case 'num':
        case 'str':
        case 'nil':
        case 'obj':
        case 'arr':
        case 'bin':
            return true;
        default:
            return false;
    }
};
var JsonTypeValidatorError;
(function (JsonTypeValidatorError) {
    JsonTypeValidatorError[JsonTypeValidatorError["STR"] = 0] = "STR";
    JsonTypeValidatorError[JsonTypeValidatorError["STR_CONST"] = 1] = "STR_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["NUM"] = 2] = "NUM";
    JsonTypeValidatorError[JsonTypeValidatorError["NUM_CONST"] = 3] = "NUM_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["BOOL"] = 4] = "BOOL";
    JsonTypeValidatorError[JsonTypeValidatorError["BOOL_CONST"] = 5] = "BOOL_CONST";
    JsonTypeValidatorError[JsonTypeValidatorError["NIL"] = 6] = "NIL";
    JsonTypeValidatorError[JsonTypeValidatorError["ARR"] = 7] = "ARR";
    JsonTypeValidatorError[JsonTypeValidatorError["OBJ"] = 8] = "OBJ";
    JsonTypeValidatorError[JsonTypeValidatorError["KEY"] = 9] = "KEY";
    JsonTypeValidatorError[JsonTypeValidatorError["KEYS"] = 10] = "KEYS";
    JsonTypeValidatorError[JsonTypeValidatorError["BIN"] = 11] = "BIN";
    JsonTypeValidatorError[JsonTypeValidatorError["OR"] = 12] = "OR";
    JsonTypeValidatorError[JsonTypeValidatorError["REF"] = 13] = "REF";
    JsonTypeValidatorError[JsonTypeValidatorError["ENUM"] = 14] = "ENUM";
    JsonTypeValidatorError[JsonTypeValidatorError["VALIDATION"] = 15] = "VALIDATION";
    JsonTypeValidatorError[JsonTypeValidatorError["INT"] = 16] = "INT";
    JsonTypeValidatorError[JsonTypeValidatorError["UINT"] = 17] = "UINT";
    JsonTypeValidatorError[JsonTypeValidatorError["STR_LEN"] = 18] = "STR_LEN";
})(JsonTypeValidatorError = exports.JsonTypeValidatorError || (exports.JsonTypeValidatorError = {}));
exports.JsonTypeValidatorErrorMessage = (_a = {},
    _a[JsonTypeValidatorError.STR] = 'Not a string.',
    _a[JsonTypeValidatorError.STR_CONST] = 'Invalid string constant.',
    _a[JsonTypeValidatorError.NUM] = 'Not a number.',
    _a[JsonTypeValidatorError.NUM_CONST] = 'Invalid number constant.',
    _a[JsonTypeValidatorError.BOOL] = 'Not a boolean.',
    _a[JsonTypeValidatorError.BOOL_CONST] = 'Invalid boolean constant.',
    _a[JsonTypeValidatorError.NIL] = 'Not null.',
    _a[JsonTypeValidatorError.ARR] = 'Not an array.',
    _a[JsonTypeValidatorError.OBJ] = 'Not an object.',
    _a[JsonTypeValidatorError.KEY] = 'Missing key.',
    _a[JsonTypeValidatorError.KEYS] = 'Too many or missing object keys.',
    _a[JsonTypeValidatorError.BIN] = 'Not a binary.',
    _a[JsonTypeValidatorError.OR] = 'None of types matched.',
    _a[JsonTypeValidatorError.REF] = 'Validation error in referenced type.',
    _a[JsonTypeValidatorError.ENUM] = 'Not an enum value.',
    _a[JsonTypeValidatorError.VALIDATION] = 'Custom validator failed.',
    _a[JsonTypeValidatorError.INT] = 'Not an integer.',
    _a[JsonTypeValidatorError.UINT] = 'Not an unsigned integer.',
    _a[JsonTypeValidatorError.STR_LEN] = 'Invalid string length.',
    _a);
var JsonTypeValidatorCodegen = (function () {
    function JsonTypeValidatorCodegen(options) {
        if (options === void 0) { options = {}; }
        this.steps = [];
        this.registerCounter = 0;
        this.dependencies = [];
        this.consts = [];
        this.options = (0, tslib_1.__assign)({ errorReporting: 'boolean', skipObjectExtraFieldsCheck: false, unsafeMode: false, ref: function () { return undefined; }, customValidators: [] }, options);
    }
    JsonTypeValidatorCodegen.prototype.js = function (js) {
        this.steps.push(new EncodingPlanStepExecJs(js));
    };
    JsonTypeValidatorCodegen.prototype.getRegister = function () {
        return "r".concat(++this.registerCounter);
    };
    JsonTypeValidatorCodegen.prototype.addDependencies = function (deps) {
        var e_1, _a;
        var symbols = [];
        try {
            for (var deps_1 = (0, tslib_1.__values)(deps), deps_1_1 = deps_1.next(); !deps_1_1.done; deps_1_1 = deps_1.next()) {
                var dep = deps_1_1.value;
                this.dependencies.push(dep);
                symbols.push("d".concat(this.dependencies.length - 1));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (deps_1_1 && !deps_1_1.done && (_a = deps_1.return)) _a.call(deps_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return symbols;
    };
    JsonTypeValidatorCodegen.prototype.addConsts = function (consts) {
        var e_2, _a;
        var symbols = [];
        try {
            for (var consts_1 = (0, tslib_1.__values)(consts), consts_1_1 = consts_1.next(); !consts_1_1.done; consts_1_1 = consts_1.next()) {
                var const_ = consts_1_1.value;
                this.consts.push(const_);
                symbols.push("c".concat(this.consts.length - 1));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (consts_1_1 && !consts_1_1.done && (_a = consts_1.return)) _a.call(consts_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return symbols;
    };
    JsonTypeValidatorCodegen.prototype.err = function (code, path, opts) {
        var e_3, _a, e_4, _b;
        if (opts === void 0) { opts = {}; }
        switch (this.options.errorReporting) {
            case 'boolean':
                return 'true';
            case 'string': {
                var out = "'[" + JSON.stringify(JsonTypeValidatorError[code]);
                try {
                    for (var path_1 = (0, tslib_1.__values)(path), path_1_1 = path_1.next(); !path_1_1.done; path_1_1 = path_1.next()) {
                        var step = path_1_1.value;
                        if (typeof step === 'object') {
                            out += ",' + JSON.stringify(" + step.r + ") + '";
                        }
                        else {
                            out += ',' + JSON.stringify(step);
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (path_1_1 && !path_1_1.done && (_a = path_1.return)) _a.call(path_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
                return out + "]'";
            }
            case 'object':
            default: {
                var out = '{code: ' +
                    JSON.stringify(JsonTypeValidatorError[code]) +
                    ', errno: ' +
                    JSON.stringify(code) +
                    ', message: ' +
                    JSON.stringify(exports.JsonTypeValidatorErrorMessage[code]) +
                    ', path: [';
                var i = 0;
                try {
                    for (var path_2 = (0, tslib_1.__values)(path), path_2_1 = path_2.next(); !path_2_1.done; path_2_1 = path_2.next()) {
                        var step = path_2_1.value;
                        if (i)
                            out += ', ';
                        if (typeof step === 'object') {
                            out += step.r;
                        }
                        else {
                            out += JSON.stringify(step);
                        }
                        i++;
                    }
                }
                catch (e_4_1) { e_4 = { error: e_4_1 }; }
                finally {
                    try {
                        if (path_2_1 && !path_2_1.done && (_b = path_2.return)) _b.call(path_2);
                    }
                    finally { if (e_4) throw e_4.error; }
                }
                out += ']';
                if (this.type.id) {
                    out += ', type: ' + JSON.stringify(this.type.id || '');
                }
                if (opts.refError) {
                    out += ', ref: ' + opts.refError;
                }
                if (opts.validator) {
                    out += ', validator: ' + JSON.stringify(opts.validator);
                }
                return out + '}';
            }
        }
    };
    JsonTypeValidatorCodegen.prototype.refErr = function (path, r) {
        switch (this.options.errorReporting) {
            case 'boolean':
                return r;
            case 'string': {
                return this.err(JsonTypeValidatorError.REF, (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), [{ r: r }], false));
            }
            case 'object':
            default: {
                return this.err(JsonTypeValidatorError.REF, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), { refError: r });
            }
        }
    };
    JsonTypeValidatorCodegen.prototype.emitValidations = function (path, validatorNames, type, r) {
        var e_5, _a;
        try {
            for (var validatorNames_1 = (0, tslib_1.__values)(validatorNames), validatorNames_1_1 = validatorNames_1.next(); !validatorNames_1_1.done; validatorNames_1_1 = validatorNames_1.next()) {
                var validatorName = validatorNames_1_1.value;
                var v = this.linkValidator(validatorName, type);
                var rerr = this.getRegister();
                var rc = this.getRegister();
                var err = this.err(JsonTypeValidatorError.VALIDATION, path, { validator: validatorName, refError: rerr });
                var errInCatch = this.err(JsonTypeValidatorError.VALIDATION, path, { validator: validatorName, refError: rc });
                this.js("try {");
                this.js("const ".concat(rerr, " = ").concat(v, "(").concat(r, ");"));
                this.js("if (".concat(rerr, ") return ").concat(err, ";"));
                this.js("} catch (e) {");
                this.js("var ".concat(rc, " = e ? e : new Error('Validator ").concat(JSON.stringify(validatorName), " failed.');"));
                this.js("return ".concat(errInCatch, ";"));
                this.js("}");
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (validatorNames_1_1 && !validatorNames_1_1.done && (_a = validatorNames_1.return)) _a.call(validatorNames_1);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    JsonTypeValidatorCodegen.prototype.linkValidator = function (name, type) {
        var e_6, _a;
        try {
            for (var _b = (0, tslib_1.__values)(this.options.customValidators), _c = _b.next(); !_c.done; _c = _b.next()) {
                var validator = _c.value;
                if (name !== validator.name)
                    continue;
                if (validator.types.indexOf(type) === -1)
                    continue;
                return this.addDependencies([validator.fn])[0];
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
        throw new Error("Custom validator \"".concat(name, "\" not found."));
    };
    JsonTypeValidatorCodegen.prototype.onString = function (path, str, r) {
        if (str.const !== undefined) {
            var error = this.err(JsonTypeValidatorError.STR_CONST, path);
            this.js("if(".concat(r, " !== ").concat(JSON.stringify(str.const), ") return ").concat(error, ";"));
        }
        else {
            var error = this.err(JsonTypeValidatorError.STR, path);
            this.js("if(typeof ".concat(r, " !== \"string\") return ").concat(error, ";"));
            if (typeof str.minLength === 'number')
                if (!Number.isInteger(str.minLength) || str.minLength < 0)
                    throw new Error("Invalid minLength ".concat(str.minLength, " for string type."));
            if (typeof str.maxLength === 'number')
                if (!Number.isInteger(str.maxLength) || str.maxLength < 0)
                    throw new Error("Invalid maxLength ".concat(str.maxLength, " for string type."));
            if (typeof str.minLength === 'number' && typeof str.maxLength === 'number') {
                if (str.minLength > str.maxLength)
                    throw new Error("Invalid minLength ".concat(str.minLength, " and maxLength ").concat(str.maxLength, " for string type."));
            }
            if (typeof str.minLength === 'number' && typeof str.maxLength === 'number' && str.minLength === str.maxLength) {
                var strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                this.js("if(".concat(r, ".length !== ").concat(str.minLength, ") return ").concat(strLenError, ";"));
            }
            else {
                if (typeof str.minLength === 'number') {
                    var strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                    this.js("if(".concat(r, ".length < ").concat(str.minLength, ") return ").concat(strLenError, ";"));
                }
                if (typeof str.maxLength === 'number') {
                    var strLenError = this.err(JsonTypeValidatorError.STR_LEN, path);
                    this.js("if(".concat(r, ".length > ").concat(str.maxLength, ") return ").concat(strLenError, ";"));
                }
            }
        }
        if (str.validator) {
            this.emitValidations(path, str.validator instanceof Array ? str.validator : [str.validator], 'string', r);
        }
    };
    JsonTypeValidatorCodegen.prototype.onNumber = function (path, num, r) {
        if (num.const !== undefined) {
            var err = this.err(JsonTypeValidatorError.NUM_CONST, path);
            this.js("if(".concat(r, " !== ").concat(JSON.stringify(num.const), ") return ").concat(err, ";"));
        }
        else {
            if (num.format && util_1.INTS.indexOf(num.format) !== -1) {
                var errInt = this.err(JsonTypeValidatorError.INT, path);
                this.js("if(!Number.isInteger(".concat(r, ")) return ").concat(errInt, ";"));
                if (util_1.UINTS.indexOf(num.format) !== -1) {
                    var errUint = this.err(JsonTypeValidatorError.UINT, path);
                    this.js("if(".concat(r, " < 0) return ").concat(errUint, ";"));
                    switch (num.format) {
                        case 'u8': {
                            this.js("if(".concat(r, " > 0xFF) return ").concat(errUint, ";"));
                            break;
                        }
                        case 'u16': {
                            this.js("if(".concat(r, " > 0xFFFF) return ").concat(errUint, ";"));
                            break;
                        }
                        case 'u32': {
                            this.js("if(".concat(r, " > 0xFFFFFFFF) return ").concat(errUint, ";"));
                            break;
                        }
                    }
                }
                else {
                    switch (num.format) {
                        case 'i8': {
                            this.js("if(".concat(r, " > 0x7F || ").concat(r, " < -0x80) return ").concat(errInt, ";"));
                            break;
                        }
                        case 'i16': {
                            this.js("if(".concat(r, " > 0x7FFF || ").concat(r, " < -0x8000) return ").concat(errInt, ";"));
                            break;
                        }
                        case 'i32': {
                            this.js("if(".concat(r, " > 0x7FFFFFFF || ").concat(r, " < -0x80000000) return ").concat(errInt, ";"));
                            break;
                        }
                    }
                }
            }
            else {
                var errNum = this.err(JsonTypeValidatorError.NUM, path);
                this.js("if(typeof ".concat(r, " !== \"number\") return ").concat(errNum, ";"));
            }
        }
    };
    JsonTypeValidatorCodegen.prototype.onBoolean = function (path, bool, r) {
        if (typeof bool.const === 'boolean') {
            var err = this.err(JsonTypeValidatorError.BOOL_CONST, path);
            this.js("if(".concat(r, " !== ").concat(bool.const ? 'true' : 'false', ") return ").concat(err, ";"));
        }
        else {
            var err = this.err(JsonTypeValidatorError.BOOL, path);
            this.js("if(typeof ".concat(r, " !== \"boolean\") return ").concat(err, ";"));
        }
    };
    JsonTypeValidatorCodegen.prototype.onNull = function (path, r) {
        var err = this.err(JsonTypeValidatorError.NIL, path);
        this.js("if(".concat(r, " !== null) return ").concat(err, ";"));
    };
    JsonTypeValidatorCodegen.prototype.onArray = function (path, arr, expr) {
        var r = this.getRegister();
        var ri = this.getRegister();
        var rv = this.getRegister();
        this.js("var ".concat(r, " = ").concat(expr, ";"));
        var err = this.err(JsonTypeValidatorError.ARR, path);
        this.js("if (!(".concat(r, " instanceof Array)) return ").concat(err, ";"));
        this.js("for (var ".concat(rv, ", ").concat(ri, " = ").concat(r, ".length; ").concat(ri, "-- !== 0;) {"));
        this.js("".concat(rv, " = ").concat(r, "[").concat(ri, "];"));
        this.onType((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), [{ r: ri }], false), arr.type, rv);
        this.js("}");
    };
    JsonTypeValidatorCodegen.prototype.onObject = function (path, obj, expr) {
        var r = this.getRegister();
        this.js("var ".concat(r, " = ").concat(expr, ";"));
        var canSkipObjectTypeCheck = this.options.unsafeMode && obj.fields.length > 0;
        if (!canSkipObjectTypeCheck) {
            var err = this.err(JsonTypeValidatorError.OBJ, path);
            this.js("if (typeof ".concat(r, " !== 'object' || !").concat(r, " || (").concat(r, " instanceof Array)) return ").concat(err, ";"));
        }
        if (obj.fields.length && !obj.unknownFields && !this.options.skipObjectExtraFieldsCheck) {
            var rk = this.getRegister();
            this.js("for (var ".concat(rk, " in ").concat(r, ") {"));
            this.js("switch (".concat(rk, ") { case ").concat(obj.fields
                .map(function (field) { return JSON.stringify(field.key); })
                .join(': case '), ": break; default: return ").concat(this.err(JsonTypeValidatorError.KEYS, (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), [{ r: rk }], false)), ";}"));
            this.js("}");
        }
        for (var i = 0; i < obj.fields.length; i++) {
            var field = obj.fields[i];
            var rv = this.getRegister();
            var accessor = (0, normalizeAccessor_1.normalizeAccessor)(field.key);
            var keyPath = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), [field.key], false);
            if (field.isOptional) {
                this.js("var ".concat(rv, " = ").concat(r).concat(accessor, ";"));
                this.js("if (".concat(rv, " !== undefined) {"));
                this.onType(keyPath, field.type, rv);
                this.js("}");
            }
            else {
                if (canSkipObjectKeyUndefinedCheck(field.type.__t)) {
                    this.onType(keyPath, field.type, "".concat(r).concat(accessor));
                }
                else {
                    this.js("var ".concat(rv, " = ").concat(r).concat(accessor, ";"));
                    var err = this.err(JsonTypeValidatorError.KEY, (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(path), false), [field.key], false));
                    this.js("if (".concat(rv, " === undefined) return ").concat(err, ";"));
                    this.onType(keyPath, field.type, rv);
                }
            }
        }
    };
    JsonTypeValidatorCodegen.prototype.onBinary = function (path, expr) {
        var hasBuffer = typeof Buffer === 'function';
        var r = this.getRegister();
        this.js("var ".concat(r, " = ").concat(expr, ";"));
        var err = this.err(JsonTypeValidatorError.BIN, path);
        this.js("if(!(".concat(r, " instanceof Uint8Array)").concat(hasBuffer ? " && !Buffer.isBuffer(".concat(r, ")") : '', ") return ").concat(err, ";"));
    };
    JsonTypeValidatorCodegen.prototype.onRef = function (path, ref, expr) {
        var validator = this.options.ref(ref.ref);
        if (!validator) {
            throw new Error("Could not resolve validator for [ref = ".concat(ref.ref, "]. Provide it through .ref option."));
        }
        var _a = (0, tslib_1.__read)(this.addDependencies([validator]), 1), d = _a[0];
        var rerr = this.getRegister();
        this.js("var ".concat(rerr, " = ").concat(d, "(").concat(expr, ");"));
        this.js("if (".concat(rerr, ") return ").concat(this.refErr(path, rerr), ";"));
    };
    JsonTypeValidatorCodegen.prototype.onOr = function (path, or, expr) {
        var _this = this;
        if (or.types.length === 1) {
            this.onType(path, or.types[0], expr);
            return;
        }
        var consts = this.addConsts(or.types.map(function (type) {
            var codegen = new JsonTypeValidatorCodegen((0, tslib_1.__assign)((0, tslib_1.__assign)({}, _this.options), { errorReporting: 'boolean' }));
            var fn = codegen.generate(type);
            var symbols = _this.addDependencies(fn.deps);
            return "((".concat(fn.js, ")(").concat(symbols.join(', '), "))");
        })).map(function (c) { return "".concat(c, "(").concat(expr, ")"); });
        var err = this.err(JsonTypeValidatorError.OR, path);
        this.js("if (".concat(consts.join(' && '), ") return ").concat(err));
    };
    JsonTypeValidatorCodegen.prototype.onEnum = function (path, type, r) {
        if (!type.values.length)
            throw new Error('Enum values are not specified.');
        var equals = type.values.map(function (value) { return (0, __deepEqual_1.$$deepEqual)(value); });
        var consts = this.addConsts(equals);
        var exprs = consts.map(function (c) { return "!".concat(c, "(").concat(r, ")"); });
        this.js("if (".concat(exprs.join(' && '), ") return ").concat(this.err(JsonTypeValidatorError.ENUM, path)));
    };
    JsonTypeValidatorCodegen.prototype.onType = function (path, type, expr) {
        switch (type.__t) {
            case 'str': {
                this.onString(path, type, expr);
                break;
            }
            case 'num': {
                this.onNumber(path, type, expr);
                break;
            }
            case 'bool': {
                this.onBoolean(path, type, expr);
                break;
            }
            case 'nil': {
                this.onNull(path, expr);
                break;
            }
            case 'arr': {
                this.onArray(path, type, expr);
                break;
            }
            case 'obj': {
                this.onObject(path, type, expr);
                break;
            }
            case 'bin': {
                this.onBinary(path, expr);
                break;
            }
            case 'any': {
                break;
            }
            case 'ref': {
                this.onRef(path, type, expr);
                break;
            }
            case 'or': {
                this.onOr(path, type, expr);
                break;
            }
            case 'enum': {
                this.onEnum(path, type, expr);
                break;
            }
            default: {
                throw new Error("Unknown node type [__t = ".concat(type.__t, "]."));
            }
        }
    };
    JsonTypeValidatorCodegen.prototype.generate = function (type) {
        this.type = type;
        this.onType([], type, 'r0');
        var successResult = this.options.errorReporting === 'boolean' ? 'false' : this.options.errorReporting === 'string' ? "''" : 'null';
        var dependencyArgs = this.dependencies.map(function (_, index) { return "d".concat(index); }).join(', ');
        var js = "(function(".concat(dependencyArgs, "){\n").concat(this.consts.map(function (value, index) { return "var c".concat(index, " = (").concat(value, ");"); }).join('\n'), "\nreturn function(r0){\n").concat(this.steps.map(function (step) { return step.js; }).join('\n'), "\nreturn ").concat(successResult, ";\n}})");
        return {
            deps: this.dependencies,
            js: js,
        };
    };
    return JsonTypeValidatorCodegen;
}());
exports.JsonTypeValidatorCodegen = JsonTypeValidatorCodegen;
var createValidator = function (type, options) {
    if (options === void 0) { options = {}; }
    var codegen = new JsonTypeValidatorCodegen(options);
    var fn = codegen.generate(type);
    return (0, codegen_1.compileClosure)(fn);
};
exports.createValidator = createValidator;
var createBoolValidator = function (type, options) {
    if (options === void 0) { options = {}; }
    return (0, exports.createValidator)(type, (0, tslib_1.__assign)((0, tslib_1.__assign)({}, options), { errorReporting: 'boolean' }));
};
exports.createBoolValidator = createBoolValidator;
var createStrValidator = function (type, options) {
    if (options === void 0) { options = {}; }
    return (0, exports.createValidator)(type, (0, tslib_1.__assign)((0, tslib_1.__assign)({}, options), { errorReporting: 'string' }));
};
exports.createStrValidator = createStrValidator;
var createObjValidator = function (type, options) {
    if (options === void 0) { options = {}; }
    return (0, exports.createValidator)(type, (0, tslib_1.__assign)((0, tslib_1.__assign)({}, options), { errorReporting: 'object' }));
};
exports.createObjValidator = createObjValidator;
