"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonTypeBuilder = void 0;
var tslib_1 = require("tslib");
var JsonTypeBuilder = (function () {
    function JsonTypeBuilder() {
    }
    Object.defineProperty(JsonTypeBuilder.prototype, "str", {
        get: function () {
            return this.String();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "num", {
        get: function () {
            return this.Number();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "bool", {
        get: function () {
            return this.Boolean();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "nil", {
        get: function () {
            return this.Null();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "arr", {
        get: function () {
            return this.Array(this.any);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "obj", {
        get: function () {
            return this.Object({
                fields: [],
            });
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "bin", {
        get: function () {
            return this.Binary();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(JsonTypeBuilder.prototype, "any", {
        get: function () {
            return this.Any();
        },
        enumerable: false,
        configurable: true
    });
    JsonTypeBuilder.prototype.Null = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__assign)({ __t: 'nil' }, options);
    };
    JsonTypeBuilder.prototype.Boolean = function (a, b) {
        if (typeof a === 'string')
            return this.Boolean((0, tslib_1.__assign)({ id: a }, (b || {})));
        return (0, tslib_1.__assign)({ __t: 'bool' }, (a || {}));
    };
    JsonTypeBuilder.prototype.Number = function (a, b) {
        if (typeof a === 'string')
            return this.Number((0, tslib_1.__assign)({ id: a }, (b || {})));
        return (0, tslib_1.__assign)({ __t: 'num' }, (a || {}));
    };
    JsonTypeBuilder.prototype.String = function (a, b) {
        if (typeof a === 'string')
            return this.String((0, tslib_1.__assign)({ id: a }, (b || {})));
        return (0, tslib_1.__assign)({ __t: 'str' }, (a || {}));
    };
    JsonTypeBuilder.prototype.Array = function (a, b, c) {
        if (typeof a === 'string')
            return this.Array(b, (0, tslib_1.__assign)({ id: a }, (c || {})));
        return (0, tslib_1.__assign)((0, tslib_1.__assign)({ __t: 'arr' }, b), { type: a });
    };
    JsonTypeBuilder.prototype.Object = function (a, b, c) {
        if (typeof a === 'string') {
            if (Array.isArray(b))
                return this.Object((0, tslib_1.__assign)({ id: a, fields: b }, (c || {})));
            return this.Object((0, tslib_1.__assign)({ id: a }, (b || {})));
        }
        else if (Array.isArray(a)) {
            return this.Object((0, tslib_1.__assign)({ fields: a }, (b || {})));
        }
        return (0, tslib_1.__assign)({ __t: 'obj' }, (a || {}));
    };
    JsonTypeBuilder.prototype.Field = function (key, type, options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__assign)({ key: key, type: type }, options);
    };
    JsonTypeBuilder.prototype.Binary = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__assign)({ __t: 'bin' }, options);
    };
    JsonTypeBuilder.prototype.Any = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__assign)({ __t: 'any' }, options);
    };
    JsonTypeBuilder.prototype.Ref = function (ref) {
        return {
            __t: 'ref',
            ref: ref,
        };
    };
    JsonTypeBuilder.prototype.Or = function () {
        var a = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            a[_i] = arguments[_i];
        }
        return {
            __t: 'or',
            types: a[0] instanceof Array ? a[0] : a,
        };
    };
    JsonTypeBuilder.prototype.Enum = function (a, b) {
        if (Array.isArray(a)) {
            if (!a.length)
                throw new Error('Enum must have at least one value.');
            return this.Enum((0, tslib_1.__assign)((0, tslib_1.__assign)({}, b), { values: a }));
        }
        return (0, tslib_1.__assign)({ __t: 'enum' }, a);
    };
    return JsonTypeBuilder;
}());
exports.JsonTypeBuilder = JsonTypeBuilder;
