"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var header_1 = require("./header");
var binary_1 = require("../../messages/binary");
var Encoder = (function () {
    function Encoder() {
        this.uint8 = new Uint8Array(0);
        this.offset = 0;
    }
    Encoder.prototype.encode = function (messages) {
        var size = 0;
        var length = messages.length;
        for (var i = 0; i < length; i++)
            size += messages[i].size();
        this.uint8 = new Uint8Array(size);
        this.offset = 0;
        for (var i = 0; i < length; i++) {
            var message = messages[i];
            if (message instanceof binary_1.BinaryNotificationMessage)
                this.notif(message);
            else if (message instanceof binary_1.BinaryRequestDataMessage)
                this.msgWithIdAndMethod(0, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestCompleteMessage)
                this.msgWithIdAndMethod(1, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestErrorMessage)
                this.msgWithIdAndMethod(2, message.id, message.method, message.data);
            else if (message instanceof binary_1.BinaryRequestUnsubscribeMessage)
                this.reqUnsub(message);
            else if (message instanceof binary_1.BinaryResponseDataMessage)
                this.resData(message);
            else if (message instanceof binary_1.BinaryResponseCompleteMessage)
                this.resComp(message);
            else if (message instanceof binary_1.BinaryResponseErrorMessage)
                this.resErr(message);
            else if (message instanceof binary_1.BinaryResponseUnsubscribeMessage)
                this.resUnsub(message);
        }
        return this.uint8;
    };
    Encoder.prototype.notif = function (message) {
        var method = message.method, data = message.data;
        var length = data ? data.byteLength : 0;
        this.header(3, length);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.reqUnsub = function (message) {
        this.uint8[this.offset++] = 254;
        this.id(message.id);
    };
    Encoder.prototype.resData = function (message) {
        this.msgWithId(4, message.id, message.data);
    };
    Encoder.prototype.resComp = function (message) {
        this.msgWithId(5, message.id, message.data);
    };
    Encoder.prototype.resErr = function (message) {
        this.msgWithId(6, message.id, message.data);
    };
    Encoder.prototype.resUnsub = function (message) {
        this.uint8[this.offset++] = 255;
        this.id(message.id);
    };
    Encoder.prototype.msgWithIdAndMethod = function (code, id, method, data) {
        var length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        this.method(method);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.msgWithId = function (code, id, data) {
        var length = data ? data.byteLength : 0;
        this.header(code, length);
        this.id(id);
        if (length) {
            this.uint8.set(data, this.offset);
            this.offset += length;
        }
    };
    Encoder.prototype.header = function (code, length) {
        this.offset = (0, header_1.writeHeader)(this.uint8, this.offset, code, length);
    };
    Encoder.prototype.id = function (id) {
        this.uint8[this.offset++] = id >>> 8;
        this.uint8[this.offset++] = id & 0xff;
    };
    Encoder.prototype.method = function (method) {
        var uint8 = this.uint8;
        var length = method.length;
        var offset = this.offset;
        uint8[offset++] = length;
        for (var i = 0; i < length; i++)
            uint8[offset++] = method.charCodeAt(i);
        this.offset = offset;
    };
    return Encoder;
}());
exports.Encoder = Encoder;
