"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeFullMessages = exports.decodeFullMessage = void 0;
var tslib_1 = require("tslib");
var binary_1 = require("../../messages/binary");
var header_1 = require("./header");
var readMethod = function (arr, offset) {
    var size = arr[offset++];
    var str = '';
    for (var i = 0; i < size; i++)
        str += String.fromCharCode(arr[offset++]);
    return [str, offset];
};
var decodeFullMessage = function (arr, offset) {
    var byte1 = arr[offset++];
    var code = byte1 >>> 5;
    switch (code) {
        case 3: {
            var _a = (0, tslib_1.__read)((0, header_1.readHeader)(byte1, arr, offset), 2), length_1 = _a[0], off1 = _a[1];
            var _b = (0, tslib_1.__read)(readMethod(arr, off1), 2), method = _b[0], off2 = _b[1];
            var data = arr.subarray(off2, off2 + length_1);
            return [new binary_1.BinaryNotificationMessage(method, data), off2 + length_1];
        }
        case 4:
        case 5:
        case 6: {
            var _c = (0, tslib_1.__read)((0, header_1.readHeader)(byte1, arr, offset), 2), length_2 = _c[0], off1 = _c[1];
            var o1 = arr[off1];
            var o2 = arr[off1 + 1];
            var id = (o1 << 8) | o2;
            offset = off1 + 2;
            var data = arr.subarray(offset, offset + length_2);
            var message = code === 4
                ? new binary_1.BinaryResponseDataMessage(id, data)
                : code === 5
                    ? new binary_1.BinaryResponseCompleteMessage(id, data)
                    : new binary_1.BinaryResponseErrorMessage(id, data);
            return [message, offset + length_2];
        }
        case 0:
        case 1:
        case 2: {
            var _d = (0, tslib_1.__read)((0, header_1.readHeader)(byte1, arr, offset), 2), length_3 = _d[0], off1 = _d[1];
            var o1 = arr[off1];
            var o2 = arr[off1 + 1];
            var id = (o1 << 8) | o2;
            var _e = (0, tslib_1.__read)(readMethod(arr, off1 + 2), 2), method = _e[0], off2 = _e[1];
            var data = arr.subarray(off2, off2 + length_3);
            var message = code === 0
                ? new binary_1.BinaryRequestDataMessage(id, method, data)
                : code === 1
                    ? new binary_1.BinaryRequestCompleteMessage(id, method, data)
                    : new binary_1.BinaryRequestErrorMessage(id, method, data);
            return [message, off2 + length_3];
        }
        case 7: {
            switch (byte1) {
                case 255: {
                    var o1 = arr[offset++];
                    var o2 = arr[offset++];
                    var id = (o1 << 8) | o2;
                    return [new binary_1.BinaryResponseUnsubscribeMessage(id), offset];
                }
                case 254: {
                    var o1 = arr[offset++];
                    var o2 = arr[offset++];
                    var id = (o1 << 8) | o2;
                    return [new binary_1.BinaryRequestUnsubscribeMessage(id), offset];
                }
            }
        }
    }
    throw new Error('UNKNOWN_MESSAGE');
};
exports.decodeFullMessage = decodeFullMessage;
var decodeFullMessages = function (arr, offset, end) {
    var messages = [];
    while (offset < end) {
        var _a = (0, tslib_1.__read)((0, exports.decodeFullMessage)(arr, offset), 2), message = _a[0], off = _a[1];
        offset = off;
        messages.push(message);
    }
    return messages;
};
exports.decodeFullMessages = decodeFullMessages;
