"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.decodeMsg = void 0;
var tslib_1 = require("tslib");
var nominal_1 = require("../../messages/nominal");
function decodeMsg(message) {
    if (!(message instanceof Array)) {
        throw new Error('Invalid message');
    }
    var length = message.length;
    if (length < 1 || length > 4) {
        throw new Error('Invalid message');
    }
    var first = message[0];
    if (typeof first === 'number') {
        switch (first) {
            case 0: {
                var _a = (0, tslib_1.__read)(message, 3), id = _a[1], data_1 = _a[2];
                return new nominal_1.ResponseCompleteMessage(id, data_1);
            }
            case -1: {
                var _b = (0, tslib_1.__read)(message, 3), id = _b[1], data_2 = _b[2];
                return new nominal_1.ResponseErrorMessage(id, data_2);
            }
            case -2: {
                var _c = (0, tslib_1.__read)(message, 3), id = _c[1], data_3 = _c[2];
                return new nominal_1.ResponseDataMessage(id, data_3);
            }
            case -3: {
                var _d = (0, tslib_1.__read)(message, 2), id = _d[1];
                return new nominal_1.ResponseUnsubscribeMessage(id);
            }
            default: {
                var _e = (0, tslib_1.__read)(message, 2), second = _e[1];
                switch (second) {
                    case 0: {
                        var _f = (0, tslib_1.__read)(message, 4), name_1 = _f[2], data_4 = _f[3];
                        return new nominal_1.RequestDataMessage(first, name_1, data_4);
                    }
                    case 1: {
                        var _g = (0, tslib_1.__read)(message, 4), method_1 = _g[2], data_5 = _g[3];
                        return new nominal_1.RequestErrorMessage(first, method_1, data_5);
                    }
                    case 2: {
                        return new nominal_1.RequestUnsubscribeMessage(first);
                    }
                    default: {
                        var _h = (0, tslib_1.__read)(message, 3), name_2 = _h[1], data_6 = _h[2];
                        return new nominal_1.RequestCompleteMessage(first, name_2, data_6);
                    }
                }
            }
        }
    }
    var _j = (0, tslib_1.__read)(message, 2), method = _j[0], data = _j[1];
    return new nominal_1.NotificationMessage(method, data);
}
exports.decodeMsg = decodeMsg;
function decode(messages) {
    if (messages[0] instanceof Array) {
        var length_1 = messages.length;
        var out = [];
        for (var i = 0; i < length_1; i++)
            out.push(decodeMsg(messages[i]));
        return out;
    }
    else
        return decodeMsg(messages);
}
exports.decode = decode;
