"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpc2Server = void 0;
var tslib_1 = require("tslib");
var isArray = Array.isArray;
var JsonRpc2Server = (function () {
    function JsonRpc2Server(params) {
        this.caller = params.caller;
        this.strict = !!params.strict;
        this.onNotification = params.onNotification;
        this.codec = params.codec;
    }
    JsonRpc2Server.prototype.onMessage = function (ctx, message) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var id, isNotification, method, result, error_1, rpcError;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        id = null;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        if (!message || typeof message !== 'object' || isArray(message))
                            return [2, this.codec.encodeInvalidRequestError(id)];
                        isNotification = typeof message.id !== 'number' &&
                            typeof message.id !== 'string';
                        if (!isNotification)
                            id = message.id;
                        if (message.method === undefined)
                            return [2, this.codec.encodeMethodNotFoundError(id)];
                        method = message.method;
                        if (typeof method !== 'string')
                            return [2, this.codec.encodeMethodNotFoundError(id)];
                        if (this.strict && message.jsonrpc !== '2.0')
                            return [2, this.codec.encodeInvalidRequestError(id)];
                        if (isNotification)
                            return [2, this.onNotification(method, message.params, ctx)];
                        if (!this.caller.exists(method))
                            return [2, this.codec.encodeMethodNotFoundError(id)];
                        return [4, this.caller.call(method, message.params, ctx)];
                    case 2:
                        result = _a.sent();
                        if (result === undefined)
                            return [2, undefined];
                        return [2, this.codec.encodeResponse(id, result)];
                    case 3:
                        error_1 = _a.sent();
                        rpcError = {
                            code: !!error_1 && typeof error_1 === 'object' && typeof error_1.code === 'number' ? error_1.code : 0,
                            message: error_1 instanceof Error
                                ? error_1.message
                                : !!error_1 && typeof error_1 === 'object'
                                    ? String(error_1.message)
                                    : String(error_1),
                        };
                        if (!!error_1 && typeof error_1 === 'object' && error_1.data !== undefined)
                            rpcError.data = error_1.data;
                        return [2, this.codec.encodeError(rpcError, id)];
                    case 4: return [2];
                }
            });
        });
    };
    JsonRpc2Server.prototype.onMessages = function (ctx, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var messages, results, resultsFiltered, error_2;
            var _this = this;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        messages = this.codec.decode(data);
                        return [4, Promise.all(messages.map(function (message) { return (0, tslib_1.__awaiter)(_this, void 0, void 0, function () { return (0, tslib_1.__generator)(this, function (_a) {
                                return [2, this.onMessage(ctx, message)];
                            }); }); }))];
                    case 1:
                        results = _a.sent();
                        resultsFiltered = results.filter(function (res) { return res !== undefined; });
                        if (resultsFiltered.length < 2)
                            return [2, resultsFiltered[0]];
                        return [2, this.codec.encodeBatch(resultsFiltered)];
                    case 2:
                        error_2 = _a.sent();
                        return [2, this.codec.encodeParseError()];
                    case 3: return [2];
                }
            });
        });
    };
    return JsonRpc2Server;
}());
exports.JsonRpc2Server = JsonRpc2Server;
