"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpc2CodecJsonString = void 0;
var json_brand_1 = require("../../../../../json-brand");
var asString_1 = require("../../../../../util/asString");
var KEY_ID = '{"jsonrpc":"2.0","id":';
var KEY_RESULT = ',"result":';
var KEY_ERROR = ',"error":';
var KEY_CODE = '{"code":';
var KEY_MESSAGE = ',"message":';
var KEY_DATA = ',"data":';
var BRACKET1_CLOSE = '}';
var BRACKET1_CLOSE_TWICE = '}}';
var BRACKET2_OPEN = '[';
var BRACKET2_CLOSE = ']';
var ERROR_PARSE = '{"jsonrpc":"2.0","id":null,"error":{"code":-32700,"message":"Parse error"}}';
var ERROR_INVALID_REQUEST = ',"error":{"code":-32600,"message":"Invalid Request"}}';
var ERROR_METHOD_NOT_FOUND = ',"error":{"code":-32601,"message":"Method not found"}}';
var ERROR_INVALID_PARAMS = ',"error":{"code":-32602,"message":"Invalid params"}}';
var ERROR_INTERNAL = ',"error":{"code":-32603,"message":"Internal error"}}';
var JsonRpc2CodecJsonString = (function () {
    function JsonRpc2CodecJsonString() {
    }
    JsonRpc2CodecJsonString.prototype.decode = function (data) {
        var messages = json_brand_1.JSON.parse(data);
        return Array.isArray(messages) ? messages : [messages];
    };
    JsonRpc2CodecJsonString.prototype.encodeError = function (error, id) {
        var errorJson = KEY_CODE +
            (error.code || 0) +
            KEY_MESSAGE +
            (0, asString_1.asString)(String(error.message)) +
            (error.data !== undefined ? KEY_DATA + json_brand_1.JSON.stringify(error.data) : '');
        return (KEY_ID +
            json_brand_1.JSON.stringify(id) +
            KEY_ERROR +
            errorJson +
            BRACKET1_CLOSE_TWICE);
    };
    JsonRpc2CodecJsonString.prototype.encodeParseError = function () {
        return ERROR_PARSE;
    };
    JsonRpc2CodecJsonString.prototype.encodeInvalidRequestError = function (id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INVALID_REQUEST);
    };
    JsonRpc2CodecJsonString.prototype.encodeMethodNotFoundError = function (id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_METHOD_NOT_FOUND);
    };
    JsonRpc2CodecJsonString.prototype.encodeInvalidParamsError = function (id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INVALID_PARAMS);
    };
    JsonRpc2CodecJsonString.prototype.encodeInternalError = function (id) {
        return (KEY_ID + json_brand_1.JSON.stringify(id) + ERROR_INTERNAL);
    };
    JsonRpc2CodecJsonString.prototype.encodeResponse = function (id, result) {
        return (KEY_ID +
            json_brand_1.JSON.stringify(id) +
            KEY_RESULT +
            json_brand_1.JSON.stringify(result) +
            BRACKET1_CLOSE);
    };
    JsonRpc2CodecJsonString.prototype.encodeBatch = function (messages) {
        return (BRACKET2_OPEN + messages.join(',') + BRACKET2_CLOSE);
    };
    return JsonRpc2CodecJsonString;
}());
exports.JsonRpc2CodecJsonString = JsonRpc2CodecJsonString;
