"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcApiCaller = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var constants_1 = require("./constants");
var BufferSubject_1 = require("../../../util/BufferSubject");
var error_1 = require("./error");
var methods_1 = require("./methods");
var RpcApiCaller = (function () {
    function RpcApiCaller(_a) {
        var types = _a.types, api = _a.api, error = _a.error, _b = _a.preCallBufferSize, preCallBufferSize = _b === void 0 ? 10 : _b, _c = _a.maxActiveCalls, maxActiveCalls = _c === void 0 ? 50 : _c;
        this._calls = 0;
        this.api = {};
        for (var key in api)
            this.api[key] = (0, methods_1.wrapMethod)(api[key], types);
        this.error = error || new error_1.ErrorLikeErrorFormatter();
        this.preCallBufferSize = preCallBufferSize;
        this.maxActiveCalls = maxActiveCalls;
    }
    Object.defineProperty(RpcApiCaller.prototype, "activeCalls", {
        get: function () {
            return this._calls;
        },
        enumerable: false,
        configurable: true
    });
    RpcApiCaller.prototype.exists = function (name) {
        return this.api.hasOwnProperty(name);
    };
    RpcApiCaller.prototype.get = function (name) {
        if (!this.exists(name))
            throw new error_1.RpcError(constants_1.RpcServerError.NoMethodSpecified);
        return this.api[name];
    };
    RpcApiCaller.prototype.call = function (name, request, ctx) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var method, error_2;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 7, , 8]);
                        if (this._calls >= this.maxActiveCalls)
                            throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
                        method = this.get(name);
                        if (method.validate)
                            method.validate(request);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 5, 6]);
                        this._calls++;
                        if (!method.onPreCall) return [3, 3];
                        return [4, method.onPreCall(ctx, request)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [4, method.call(ctx, request)];
                    case 4: return [2, _a.sent()];
                    case 5:
                        this._calls--;
                        return [7];
                    case 6: return [3, 8];
                    case 7:
                        error_2 = _a.sent();
                        throw this.error.format(error_2);
                    case 8: return [2];
                }
            });
        });
    };
    RpcApiCaller.prototype.callJson = function (name, request, ctx) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var method, error_3;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 7, , 8]);
                        if (this._calls >= this.maxActiveCalls)
                            throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
                        method = this.get(name);
                        if (method.validate)
                            method.validate(request);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 5, 6]);
                        this._calls++;
                        if (!method.onPreCall) return [3, 3];
                        return [4, method.onPreCall(ctx, request)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [4, method.callJson(ctx, request)];
                    case 4: return [2, _a.sent()];
                    case 5:
                        this._calls--;
                        return [7];
                    case 6: return [3, 8];
                    case 7:
                        error_3 = _a.sent();
                        throw this.error.format(error_3);
                    case 8: return [2];
                }
            });
        });
    };
    RpcApiCaller.prototype.callMsgPack = function (name, request, ctx) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var method, error_4;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 7, , 8]);
                        if (this._calls >= this.maxActiveCalls)
                            throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
                        method = this.get(name);
                        if (method.validate)
                            method.validate(request);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 5, 6]);
                        this._calls++;
                        if (!method.onPreCall) return [3, 3];
                        return [4, method.onPreCall(ctx, request)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [4, method.callMsgPack(ctx, request)];
                    case 4: return [2, _a.sent()];
                    case 5:
                        this._calls--;
                        return [7];
                    case 6: return [3, 8];
                    case 7:
                        error_4 = _a.sent();
                        throw this.error.format(error_4);
                    case 8: return [2];
                }
            });
        });
    };
    RpcApiCaller.prototype.createCall = function (name, ctx, callMethod, callMethod$) {
        var _this = this;
        var req$ = new rxjs_1.Subject();
        var reqUnsubscribe$ = new rxjs_1.Subject();
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new error_1.RpcError(constants_1.RpcServerError.TooManyActiveCalls);
            var method = this.get(name);
            if (!method.isStreaming) {
                var response$ = (0, rxjs_1.from)((function () { return (0, tslib_1.__awaiter)(_this, void 0, void 0, function () {
                    var request, response;
                    return (0, tslib_1.__generator)(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4, (0, rxjs_1.firstValueFrom)(req$.pipe((0, operators_1.first)()))];
                            case 1:
                                request = _a.sent();
                                return [4, callMethod(name, request, ctx)];
                            case 2:
                                response = _a.sent();
                                return [2, response];
                        }
                    });
                }); })());
                var res$ = new rxjs_1.Subject();
                response$.subscribe(res$);
                var $resWithErrorsFormatted_1 = res$.pipe((0, operators_1.catchError)(function (error) {
                    throw _this.error.format(error);
                }));
                return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: $resWithErrorsFormatted_1 };
            }
            var methodStreaming_1 = method;
            var requestValidated$ = req$.pipe((0, operators_1.map)(function (request) {
                if (methodStreaming_1.validate)
                    methodStreaming_1.validate(request);
                return request;
            }));
            var bufferSize = methodStreaming_1.preCallBufferSize || this.preCallBufferSize;
            var requestBuffered$_1 = new BufferSubject_1.BufferSubject(bufferSize);
            var error$_1 = new rxjs_1.Subject();
            requestBuffered$_1.subscribe({
                error: function (error) {
                    error$_1.error(error);
                },
            });
            requestValidated$.subscribe(requestBuffered$_1);
            var result$_1 = requestBuffered$_1.pipe((0, operators_1.take)(1), (0, operators_1.switchMap)(function (request) {
                return methodStreaming_1.onPreCall ? (0, rxjs_1.from)(methodStreaming_1.onPreCall(ctx, request)) : (0, rxjs_1.from)([0]);
            }), (0, operators_1.switchMap)(function () {
                Promise.resolve().then(function () {
                    requestBuffered$_1.flush();
                });
                return callMethod$(methodStreaming_1, ctx, requestBuffered$_1).pipe((0, operators_1.finalize)(function () {
                    error$_1.complete();
                }));
            }), (0, operators_1.share)(), (0, operators_1.mergeWith)(error$_1));
            var resultWithActiveCallTracking$ = (0, rxjs_1.of)(null).pipe((0, operators_1.tap)(function () {
                _this._calls++;
            }), (0, operators_1.switchMap)(function () { return result$_1; }), (0, operators_1.finalize)(function () {
                error$_1.complete();
                _this._calls--;
            }));
            var $resWithErrorsFormatted = resultWithActiveCallTracking$.pipe((0, operators_1.catchError)(function (error) {
                throw _this.error.format(error);
            }));
            return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: $resWithErrorsFormatted };
        }
        catch (error) {
            var errorFormatted = this.error.format(error);
            req$.error(errorFormatted);
            var res$ = new rxjs_1.Subject();
            res$.error(errorFormatted);
            return { req$: req$, reqUnsubscribe$: reqUnsubscribe$, res$: res$ };
        }
    };
    RpcApiCaller.prototype.call$ = function (name, request$, ctx) {
        var _this = this;
        var call = this.createCall(name, ctx, function (name, req, ctx) { return _this.call(name, req, ctx); }, function (method, ctx, req$) { return method.call$(ctx, req$); });
        request$.subscribe(call.req$);
        return call.res$;
    };
    RpcApiCaller.prototype.callJson$ = function (name, request$, ctx) {
        var _this = this;
        var call = this.createCall(name, ctx, function (name, req, ctx) { return _this.callJson(name, req, ctx); }, function (method, ctx, req$) { return method.callJson$(ctx, req$); });
        request$.subscribe(call.req$);
        return call.res$;
    };
    RpcApiCaller.prototype.callMsgPack$ = function (name, request$, ctx) {
        var _this = this;
        var call = this.createCall(name, ctx, function (name, req, ctx) { return _this.callMsgPack(name, req, ctx); }, function (method, ctx, req$) { return method.callMsgPack$(ctx, req$); });
        request$.subscribe(call.req$);
        return call.res$;
    };
    return RpcApiCaller;
}());
exports.RpcApiCaller = RpcApiCaller;
