"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcDuplex = void 0;
var nominal_1 = require("../messages/nominal");
var RpcDuplex = (function () {
    function RpcDuplex(params) {
        this.client = params.client;
        this.server = params.server;
    }
    RpcDuplex.prototype.onMessages = function (messages, ctx) {
        var length = messages.length;
        for (var i = 0; i < length; i++)
            this.onMessage(messages[i], ctx);
    };
    RpcDuplex.prototype.onMessage = function (message, ctx) {
        if (message instanceof nominal_1.RequestDataMessage)
            this.server.onRequestDataMessage(message, ctx);
        else if (message instanceof nominal_1.RequestCompleteMessage)
            this.server.onRequestCompleteMessage(message, ctx);
        else if (message instanceof nominal_1.RequestErrorMessage)
            this.server.onRequestErrorMessage(message, ctx);
        else if (message instanceof nominal_1.ResponseUnsubscribeMessage)
            this.server.onUnsubscribeMessage(message);
        else if (message instanceof nominal_1.NotificationMessage)
            this.server.onNotificationMessage(message, ctx);
        else if (message instanceof nominal_1.ResponseCompleteMessage)
            return this.client.onResponseComplete(message);
        else if (message instanceof nominal_1.ResponseDataMessage)
            return this.client.onResponseData(message);
        else if (message instanceof nominal_1.ResponseErrorMessage)
            return this.client.onResponseError(message);
        else if (message instanceof nominal_1.RequestUnsubscribeMessage)
            return this.client.onRequestUnsubscribe(message);
    };
    RpcDuplex.prototype.call$ = function (method, data) {
        return this.client.call$(method, data);
    };
    RpcDuplex.prototype.call = function (method, data) {
        return this.client.call(method, data);
    };
    RpcDuplex.prototype.notify = function (method, data) {
        this.client.notify(method, data);
    };
    RpcDuplex.prototype.stop = function () {
        this.client.stop();
        this.server.stop();
    };
    RpcDuplex.prototype.disconnect = function () {
        this.client.disconnect();
        this.server.disconnect();
    };
    return RpcDuplex;
}());
exports.RpcDuplex = RpcDuplex;
