"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpJsonRPC2Api = void 0;
var context_1 = require("../../context");
var util_1 = require("../../util");
var JsonRpc2Server_1 = require("../../../../common/json-rpc/JsonRpc2Server");
var constants_1 = require("../rpc/constants");
var JsonRpc2CodecJsonString_1 = require("../../../../common/json-rpc/codec/json-string/JsonRpc2CodecJsonString");
var STATUS_NO_RESPONSE = Buffer.from('204 No Response');
var HDR_KEY_CONTENT_TYPE = Buffer.from('Content-Type');
var HDR_VALUE_APPLICATION_JSON = Buffer.from('application/json');
function processHttpRpcRequest(res, ctx, body, server) {
    try {
        if (typeof body === 'string') {
            ctx.payloadSize = body.length;
        }
        else {
            ctx.payloadSize = body.byteLength;
            body = body.toString('utf8') || 'null';
        }
        server
            .onMessages(ctx, body)
            .then(function (result) {
            if (res.aborted)
                return;
            res.cork(function () {
                if (!result)
                    res.writeStatus(STATUS_NO_RESPONSE).end();
                else
                    res.writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(result);
            });
        })
            .catch(function () {
            if (!res.aborted)
                res.writeStatus(constants_1.STATUS_400).end();
        });
    }
    catch (_a) {
        if (!res.aborted)
            res.writeStatus(constants_1.STATUS_400).end();
    }
}
var enableHttpJsonRPC2Api = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/json-rpc' : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller, _c = params.onNotification, onNotification = _c === void 0 ? function () { } : _c;
    var server = new JsonRpc2Server_1.JsonRpc2Server({
        caller: caller,
        codec: new JsonRpc2CodecJsonString_1.JsonRpc2CodecJsonString(),
        onNotification: onNotification,
        strict: false,
    });
    uws.post(route, function (res, req) {
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        (0, util_1.readBody)(res, function (buffer) {
            processHttpRpcRequest(res, ctx, buffer, server);
        });
    });
    uws.get(route, function (res, req) {
        var query = req.getQuery();
        var params = new URLSearchParams(query);
        var body = String(params.get('a') || 'null');
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, body, server);
    });
};
exports.enableHttpJsonRPC2Api = enableHttpJsonRPC2Api;
