"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableNdjsonGetRpcApi = exports.enableNdjsonPostRpcApi = void 0;
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rpc_1 = require("../../../common/rpc");
var context_1 = require("../context");
var util_1 = require("../util");
var util_2 = require("./util");
var enableNdjsonPostRpcApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/ndjson/*' : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var origin = req.getHeader('origin');
        var ctx = createContext(req, res);
        var aborted$ = new rxjs_1.Subject();
        res.onAborted(function () {
            res.aborted = true;
            aborted$.next(true);
        });
        (0, util_1.readBody)(res, function (buffer) {
            processNdjsonRequest(res, ctx, name, buffer, aborted$, origin, caller);
        });
    });
};
exports.enableNdjsonPostRpcApi = enableNdjsonPostRpcApi;
var enableNdjsonGetRpcApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/ndjson/*' : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var origin = req.getHeader('origin');
        var ctx = createContext(req, res);
        var aborted$ = new rxjs_1.Subject();
        var query = req.getQuery();
        var params = new URLSearchParams(query);
        var body = String(params.get('a') || 'null');
        res.onAborted(function () {
            res.aborted = true;
            aborted$.next(true);
        });
        processNdjsonRequest(res, ctx, name, body, aborted$, origin, caller);
    });
};
exports.enableNdjsonGetRpcApi = enableNdjsonGetRpcApi;
var sendNdjsonError = function (res, error) {
    if (res.aborted)
        return;
    res.aborted = true;
    var errorFormatted = (0, rpc_1.formatError)(error);
    res.end('[2,' + JSON.stringify(errorFormatted) + ']\n');
};
function processNdjsonRequest(res, ctx, name, body, aborted$, origin, caller) {
    var closed = false;
    try {
        var json = (0, util_2.parsePayload)(ctx, body);
        res.cork(function () {
            (0, util_2.writeSseAndNdjsonHeaders)(res, origin);
        });
        caller
            .call$(name, (0, rxjs_1.of)(json), ctx)
            .pipe((0, operators_1.takeUntil)(aborted$))
            .subscribe({
            next: function (data) {
                if (closed)
                    return;
                if (res.aborted)
                    return;
                res.write('[1,' + JSON.stringify(data) + ']\n');
            },
            error: function (error) {
                if (closed)
                    return;
                closed = true;
                sendNdjsonError(res, error);
            },
            complete: function () {
                if (closed)
                    return;
                closed = true;
                if (!res.aborted)
                    res.end();
            },
        });
    }
    catch (_a) {
        if (closed)
            return;
        closed = true;
        var error = new Error('Could not parse payload');
        sendNdjsonError(res, error);
    }
}
