"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpRpcJsonGetApi = exports.enableHttpRpcJsonPostApi = void 0;
var rpc_1 = require("../../../../common/rpc");
var context_1 = require("../../context");
var util_1 = require("../../util");
var util_2 = require("../util");
var constants_1 = require("./constants");
var HDR_KEY_CONTENT_TYPE = Buffer.from('Content-Type');
var HDR_VALUE_APPLICATION_JSON = Buffer.from('application/json');
var sendError = function (res, error, pretty) {
    if (res.aborted)
        return;
    res.cork(function () {
        var errorFormatted = (0, rpc_1.formatError)(error);
        var body = JSON.stringify(errorFormatted, null, pretty ? 4 : 0);
        res.writeStatus(constants_1.STATUS_400).writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(body);
    });
};
function processHttpRpcRequest(res, ctx, name, body, caller) {
    try {
        var json = (0, util_2.parsePayload)(ctx, body);
        caller
            .call(name, json, ctx)
            .then(function (result) {
            if (res.aborted)
                return;
            res.cork(function () {
                var method = caller.get(name);
                var formatted = JSON.stringify(result, null, method.pretty ? 4 : 0);
                res.writeHeader(HDR_KEY_CONTENT_TYPE, HDR_VALUE_APPLICATION_JSON).end(formatted);
            });
        })
            .catch(function (error) {
            sendError(res, error, false);
        });
    }
    catch (_a) {
        var error = new Error('Could not parse payload');
        sendError(res, error, false);
    }
}
var enableHttpRpcJsonPostApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/rpc/json/*' : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        (0, util_1.readBody)(res, function (buffer) {
            processHttpRpcRequest(res, ctx, name, buffer, caller);
        });
    });
};
exports.enableHttpRpcJsonPostApi = enableHttpRpcJsonPostApi;
var enableHttpRpcJsonGetApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? '/rpc/json/*' : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var query = req.getQuery();
        var params = new URLSearchParams(query);
        var body = String(params.get('a') || 'null');
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        processHttpRpcRequest(res, ctx, name, body, caller);
    });
};
exports.enableHttpRpcJsonGetApi = enableHttpRpcJsonGetApi;
