"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableHttpRpcMsgPackGetApi = exports.enableHttpRpcMsgPackPostApi = void 0;
var rpc_1 = require("../../../../common/rpc");
var util_1 = require("../../../../../json-pack/util");
var context_1 = require("../../context");
var util_2 = require("../../util");
var constants_1 = require("./constants");
var bufferToUint8Array_1 = require("../../../../../util/bufferToUint8Array");
var util_3 = require("../util");
var DEFAULT_ROUTE = '/rpc/msgpack/*';
function sendError(res, error) {
    if (res.aborted)
        return;
    res.cork(function () {
        var body = (0, util_1.encode)((0, rpc_1.formatError)(error));
        res.writeStatus(constants_1.STATUS_400).end(body);
    });
}
function processRequest(res, ctx, name, json, caller) {
    try {
        caller
            .call(name, json, ctx)
            .then(function (result) {
            if (res.aborted)
                return;
            res.cork(function () {
                res.end((0, util_1.encode)(result));
            });
        })
            .catch(function (error) {
            sendError(res, error);
        });
    }
    catch (_a) {
        var error = new Error('Could not execute request');
        sendError(res, error);
    }
}
var enableHttpRpcMsgPackPostApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? DEFAULT_ROUTE : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.post(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        (0, util_2.readBody)(res, function (buffer) {
            try {
                var length_1 = (ctx.payloadSize = buffer.length);
                var json = length_1 ? (0, util_1.decode)((0, bufferToUint8Array_1.bufferToUint8Array)(buffer)) : null;
                processRequest(res, ctx, name, json, caller);
            }
            catch (_a) {
                var error = new Error('Could not parse payload');
                sendError(res, error);
            }
        });
    });
};
exports.enableHttpRpcMsgPackPostApi = enableHttpRpcMsgPackPostApi;
var enableHttpRpcMsgPackGetApi = function (params) {
    var uws = params.uws, _a = params.route, route = _a === void 0 ? DEFAULT_ROUTE : _a, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, caller = params.caller;
    if (!route.endsWith('/*'))
        throw new Error('"route" must end with "/*".');
    uws.get(route, function (res, req) {
        var url = req.getUrl();
        var name = url.substr(route.length - 1);
        var query = req.getQuery();
        var params = new URLSearchParams(query);
        var body = String(params.get('a') || 'null');
        var ctx = createContext(req, res);
        res.onAborted(function () {
            res.aborted = true;
        });
        try {
            var json = (0, util_3.parsePayload)(ctx, body);
            processRequest(res, ctx, name, json, caller);
        }
        catch (_a) {
            var error = new Error('Could not parse payload');
            sendError(res, error);
        }
    });
};
exports.enableHttpRpcMsgPackGetApi = enableHttpRpcMsgPackGetApi;
