"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsCompactReactiveRpcApi = void 0;
var common_1 = require("../../../common");
var Decoder_1 = require("../../../common/codec/compact-json/Decoder");
var Decoder_2 = require("../../../common/codec/compact-msgpack/Decoder");
var compact_msgpack_binary_1 = require("../../../common/codec/compact-msgpack-binary");
var compact_json_string_1 = require("../../../common/codec/compact-json_string");
var context_1 = require("../context");
var json_1 = require("../../../common/messages/json");
var json_brand_1 = require("../../../../json-brand");
var util_1 = require("../../../../json-pack/util");
var enableWsCompactReactiveRpcApi = function (params) {
    var decoderJson = new Decoder_1.Decoder();
    var decoderMsgPack = new Decoder_2.Decoder();
    var encoderCompactJsonString = new compact_json_string_1.Encoder();
    var encoderCompactMsgPackBinary = new compact_msgpack_binary_1.Encoder();
    var invalidPayloadJson = encoderCompactJsonString.encode([
        new json_1.JsonNotificationMessage('.err', json_brand_1.JSON.stringify('CODING')),
    ]);
    var invalidPayloadMsgPack = encoderCompactMsgPackBinary.encode([
        new common_1.BinaryNotificationMessage('.err', (0, util_1.encode)('CODING')),
    ]);
    var _a = params.route, route = _a === void 0 ? '/rpc/compact' : _a, uws = params.uws, createRpcServerJson = params.createRpcServerJson, createRpcServerMsgPack = params.createRpcServerMsgPack, onNotification = params.onNotification, _b = params.createContext, createContext = _b === void 0 ? context_1.createConnectionContext : _b, compression = params.compression, _c = params.idleTimeout, idleTimeout = _c === void 0 ? 0 : _c, _d = params.maxBackpressure, maxBackpressure = _d === void 0 ? 3145728 : _d, _e = params.maxPayloadLength, maxPayloadLength = _e === void 0 ? 1048576 : _e;
    uws.ws(route, {
        idleTimeout: idleTimeout,
        compression: compression,
        maxBackpressure: maxBackpressure,
        maxPayloadLength: maxPayloadLength,
        upgrade: function (res, req, context) {
            var secWebSocketKey = req.getHeader('sec-websocket-key');
            var secWebSocketProtocol = req.getHeader('sec-websocket-protocol');
            var secWebSocketExtensions = req.getHeader('sec-websocket-extensions');
            var ctx = createContext(req, res);
            var isBinary = secWebSocketProtocol.indexOf('MsgPack') > -1;
            res.upgrade({ ctx: ctx, isBinary: isBinary }, secWebSocketKey, secWebSocketProtocol, secWebSocketExtensions, context);
        },
        open: function (ws) {
            var isBinary = ws.isBinary;
            var rpc;
            if (isBinary) {
                rpc = createRpcServerMsgPack({
                    send: function (messages) {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        var encoded = encoderCompactMsgPackBinary.encode(messages);
                        ws.send(encoded, true);
                    },
                });
            }
            else {
                rpc = createRpcServerJson({
                    send: function (messages) {
                        if (ws.getBufferedAmount() > maxBackpressure)
                            return;
                        var encoded = encoderCompactJsonString.encode(messages);
                        ws.send(encoded, false);
                    },
                });
            }
            if (onNotification) {
                rpc.onNotification = function (name, data, ctx) {
                    onNotification(ws, name, data, ctx);
                };
            }
            ws.rpc = rpc;
        },
        message: function (ws, buf, isBinary) {
            var _a = ws, ctx = _a.ctx, rpc = _a.rpc;
            var messages;
            try {
                messages = isBinary
                    ? decoderMsgPack.decode(new Uint8Array(buf))
                    : decoderJson.decode(Buffer.from(buf).toString('utf8'));
            }
            catch (error) {
                if (ws.getBufferedAmount() > maxBackpressure)
                    return;
                ws.send(isBinary ? invalidPayloadMsgPack : invalidPayloadJson, isBinary);
                return;
            }
            if (messages instanceof Array)
                rpc.onMessages(messages, ctx);
            else
                rpc.onMessage(messages, ctx);
        },
        close: function (ws, code, message) {
            ws.rpc.stop();
        },
    });
};
exports.enableWsCompactReactiveRpcApi = enableWsCompactReactiveRpcApi;
