"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableWsBinaryReactiveRpcApi = void 0;
var context_1 = require("./context");
var binary_1 = require("../../common/codec/binary");
var binary_msgpack_1 = require("../../common/codec/binary-msgpack");
var bufferToUint8Array_1 = require("../../../util/bufferToUint8Array");
var enableWsBinaryReactiveRpcApi = function (params) {
    var encoder = new binary_1.Encoder();
    var decoder = new binary_msgpack_1.Decoder();
    var wss = params.wss, createContext = params.createContext, createRpcServer = params.createRpcServer, onNotification = params.onNotification;
    wss.on('connection', function (ws, req) {
        var ctx;
        var rpc = createRpcServer({
            send: function (messages) {
                var uint8 = encoder.encode(messages);
                ws.send(uint8);
            },
        });
        if (onNotification) {
            rpc.onNotification = function (name, data, ctx) {
                onNotification(ws, name, data, ctx);
            };
        }
        ws.on('upgrade', function (req) {
            ctx = (createContext || context_1.createConnectionContext)(req);
        });
        ws.on('message', function (buf) {
            var uint8 = (0, bufferToUint8Array_1.bufferToUint8Array)(buf);
            var messages = decoder.decode(uint8);
            var length = messages.length;
            for (var i = 0; i < length; i++)
                rpc.onMessage(messages[i], ctx);
        });
        ws.on('close', function () {
            rpc.stop();
        });
    });
};
exports.enableWsBinaryReactiveRpcApi = enableWsBinaryReactiveRpcApi;
