"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromBase64 = exports.createFromBase64 = void 0;
var bufferToUint8Array_1 = require("../bufferToUint8Array");
var constants_1 = require("./constants");
var E = '=';
var createFromBase64 = function (chars) {
    if (chars === void 0) { chars = constants_1.alphabet; }
    if (chars.length !== 64)
        throw new Error('chars must be 64 characters long');
    var list = chars.split('');
    var table = {};
    for (var i = 0; i < list.length; i++)
        table[list[i]] = i;
    return function (encoded) {
        if (!encoded)
            return new Uint8Array(0);
        var length = encoded.length;
        if (length % 4 !== 0)
            throw new Error('Base64 string length must be a multiple of 4');
        var mainLength = encoded[length - 1] !== E ? length : length - 4;
        var bufferLength = (length / 4) * 3;
        var padding = 0;
        if (encoded[length - 2] === E) {
            padding = 2;
            bufferLength -= 2;
        }
        else if (encoded[length - 1] === E) {
            padding = 1;
            bufferLength -= 1;
        }
        var buf = new Uint8Array(bufferLength);
        var j = 0;
        var i = 0;
        for (; i < mainLength; i += 4) {
            var c0 = encoded[i];
            var c1 = encoded[i + 1];
            var c2 = encoded[i + 2];
            var c3 = encoded[i + 3];
            var sextet0 = table[c0];
            var sextet1 = table[c1];
            var sextet2 = table[c2];
            var sextet3 = table[c3];
            if ((sextet0 === undefined) || (sextet1 === undefined) || (sextet2 === undefined) || (sextet3 === undefined))
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
            buf[j + 2] = (sextet2 << 6) | sextet3;
            j += 3;
        }
        if (padding === 2) {
            var c0 = encoded[mainLength];
            var c1 = encoded[mainLength + 1];
            var sextet0 = table[c0];
            var sextet1 = table[c1];
            if ((sextet0 === undefined) || (sextet1 === undefined))
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
        }
        else if (padding === 1) {
            var c0 = encoded[mainLength];
            var c1 = encoded[mainLength + 1];
            var c2 = encoded[mainLength + 2];
            var sextet0 = table[c0];
            var sextet1 = table[c1];
            var sextet2 = table[c2];
            if ((sextet0 === undefined) || (sextet1 === undefined) || (sextet2 === undefined))
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
        }
        return buf;
    };
};
exports.createFromBase64 = createFromBase64;
var fromBase64Cpp = constants_1.hasBuffer ? function (encoded) { return (0, bufferToUint8Array_1.bufferToUint8Array)(Buffer.from(encoded, 'base64')); } : null;
var fromBase64Native = (0, exports.createFromBase64)();
exports.fromBase64 = !fromBase64Cpp
    ? fromBase64Native
    : function (encoded) { return (encoded.length > 50 ? fromBase64Cpp(encoded) : fromBase64Native(encoded)); };
