"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Codegen = exports.CodegenStepExecJs = void 0;
var tslib_1 = require("tslib");
var _1 = require(".");
var CodegenStepExecJs = (function () {
    function CodegenStepExecJs(js) {
        this.js = js;
    }
    return CodegenStepExecJs;
}());
exports.CodegenStepExecJs = CodegenStepExecJs;
var Codegen = (function () {
    function Codegen(opts) {
        this.steps = [];
        this.registerCounter = 0;
        this.dependencies = [];
        this.dependencyNames = [];
        this.linked = {};
        this.constants = [];
        this.constantNames = [];
        this.options = (0, tslib_1.__assign)({ args: 'r0', name: '', prologue: '', epilogue: '', processSteps: function (steps) { return steps.filter(function (step) { return step instanceof CodegenStepExecJs; }); }, linkable: {} }, opts);
    }
    Codegen.prototype.js = function (js) {
        this.steps.push(new CodegenStepExecJs(js));
    };
    Codegen.prototype.step = function (step) {
        this.steps.push(step);
    };
    Codegen.prototype.getRegister = function () {
        return "r".concat(this.registerCounter++);
    };
    Codegen.prototype.linkDependency = function (dep, name) {
        if (name === void 0) { name = 'd' + this.dependencies.length; }
        this.dependencies.push(dep);
        this.dependencyNames.push(name);
        return name;
    };
    Codegen.prototype.linkDependencies = function (deps) {
        var _this = this;
        return deps.map(function (dep) { return _this.linkDependency(dep); });
    };
    Codegen.prototype.link = function (name) {
        if (this.linked[name])
            return;
        this.linked[name] = 1;
        this.linkDependency(this.options.linkable[name], name);
    };
    Codegen.prototype.addConstant = function (constant, name) {
        if (name === void 0) { name = 'c' + this.constants.length; }
        this.constants.push(constant);
        this.constantNames.push(name);
        return name;
    };
    Codegen.prototype.addConstants = function (constants) {
        var _this = this;
        return constants.map(function (constant) { return _this.addConstant(constant); });
    };
    Codegen.prototype.generate = function (opts) {
        var _this = this;
        if (opts === void 0) { opts = {}; }
        var _a = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, this.options), opts), name = _a.name, args = _a.args, prologue = _a.prologue, epilogue = _a.epilogue;
        var steps = this.options.processSteps(this.steps);
        var js = "(function(".concat(this.dependencyNames.join(', '), ") {\n").concat(this.constants.map(function (constant, index) { return "var ".concat(_this.constantNames[index], " = (").concat(constant, ");"); }).join('\n'), "\nreturn ").concat(name ? "function ".concat(name) : 'function', "(").concat(args, "){\n").concat(prologue, "\n").concat(steps.map(function (step) { return step.js; }).join('\n'), "\n").concat(epilogue, "\n}})");
        return {
            deps: this.dependencies,
            js: js,
        };
    };
    Codegen.prototype.compile = function (opts) {
        return (0, _1.compileClosure)(this.generate(opts));
    };
    return Codegen;
}());
exports.Codegen = Codegen;
