"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
var EMPTY_UINT8 = new Uint8Array([]);
var EMPTY_VIEW = new DataView(EMPTY_UINT8.buffer);
var Encoder = (function () {
    function Encoder() {
        this.uint8 = EMPTY_UINT8;
        this.view = EMPTY_VIEW;
        this.offset = 0;
    }
    Encoder.prototype.grow = function (size) {
        var newUint8 = new Uint8Array(size);
        newUint8.set(this.uint8);
        this.uint8 = newUint8;
        this.view = new DataView(newUint8.buffer);
    };
    Encoder.prototype.ensureCapacity = function (capacity) {
        var size = this.offset + capacity;
        var len = this.uint8.byteLength;
        if (len < size)
            this.grow(Math.max(size, len * 4));
    };
    Encoder.prototype.reset = function () {
        this.uint8 = new Uint8Array(1024);
        this.view = new DataView(this.uint8.buffer);
        this.offset = 0;
    };
    Encoder.prototype.flush = function () {
        return this.uint8.subarray(0, this.offset);
    };
    Encoder.prototype.u8 = function (char) {
        this.ensureCapacity(1);
        this.uint8[this.offset++] = char;
    };
    Encoder.prototype.u16 = function (word) {
        this.ensureCapacity(2);
        this.view.setUint16(this.offset, word);
        this.offset += 2;
    };
    Encoder.prototype.u32 = function (dword) {
        this.ensureCapacity(4);
        this.view.setUint32(this.offset, dword);
        this.offset += 4;
    };
    return Encoder;
}());
exports.Encoder = Encoder;
